@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2000-2022 ircd-hybrid development team
+ *  Copyright (c) 2000-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,31 +28,33 @@
 #include "irc_string.h"
 #include "ircd.h"
 #include "list.h"
+#include "nuh.h"
 #include "numeric.h"
 #include "conf.h"
 #include "send.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 #include "memory.h"
+#include "accept.h"
 
 
 /*! \brief Sends a list of nick!user\@host masks a Client
  *         has on its acceptlist.
  *
- * \param source_p The actual Client the list will be sent to.
+ * \param source The actual Client the list will be sent to.
  */
 static void
-list_accepts(struct Client *source_p)
+accept_list(struct Client *source)
 {
   char buf[IRCD_BUFSIZE];
   char *bufptr = buf;
-  dlink_node *node;
 
-  /* :me.name 281 source_p->name :n1!u1@h1 n2!u2@h2 ...\r\n */
-  /* 1       23456              78                     9 10 */
-  size_t len = strlen(me.name) + strlen(source_p->name) + 10;
+  /* :me.name 281 source->name :n1!u1@h1 n2!u2@h2 ...\r\n */
+  /* 1       23456            78                     9 0  */
+  size_t len = strlen(me.name) + strlen(source->name) + 10;
 
-  DLINK_FOREACH(node, source_p->connection->acceptlist.head)
+  list_node_t *node;
+  LIST_FOREACH(node, source->connection->acceptlist.head)
   {
     const struct AcceptItem *const accept = node->data;
     size_t masklen = strlen(accept->nick) +
@@ -61,49 +63,23 @@ list_accepts(struct Client *source_p)
 
     if ((bufptr - buf) + masklen + len > sizeof(buf))
     {
-      sendto_one_numeric(source_p, &me, RPL_ACCEPTLIST, buf);
+      sendto_one_numeric(source, &me, RPL_ACCEPTLIST, buf);
       bufptr = buf;
     }
 
     bufptr += snprintf(bufptr, sizeof(buf) - (bufptr - buf), bufptr != buf ? " %s!%s@%s" : "%s!%s@%s",
-                       accept->nick,
-                       accept->user,
-                       accept->host);
+                       accept->nick, accept->user, accept->host);
   }
 
   if (bufptr != buf)
-    sendto_one_numeric(source_p, &me, RPL_ACCEPTLIST, buf);
+    sendto_one_numeric(source, &me, RPL_ACCEPTLIST, buf);
 
-  sendto_one_numeric(source_p, &me, RPL_ENDOFACCEPT);
-}
-
-/*! \brief Allocates and adds a struct AccepItem holding a nick!user\@host
- *         mask to a Client's acceptlist.
- *
- * \param nick nick portion of the nick!user\@host mask.
- * \param user user portion of the nick!user\@host mask.
- * \param host host portion of the nick!user\@host mask.
- * \param source_p The actual Client the new accept is added to.
- */
-static void
-accept_add(const char *nick,
-           const char *user,
-           const char *host, struct Client *source_p)
-{
-  struct AcceptItem *accept = xcalloc(sizeof(*accept));
-
-  accept->nick = xstrdup(nick);
-  accept->user = xstrdup(user);
-  accept->host = xstrdup(host);
-
-  dlinkAdd(accept, &accept->node, &source_p->connection->acceptlist);
-
-  list_accepts(source_p);
+  sendto_one_numeric(source, &me, RPL_ENDOFACCEPT);
 }
 
 /*! \brief ACCEPT command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -113,82 +89,76 @@ accept_add(const char *nick,
  *      - parv[1] = comma-separated list of masks to be accepted or removed
  */
 static void
-m_accept(struct Client *source_p, int parc, char *parv[])
+m_accept(struct Client *source, int parc, char *parv[])
 {
-  struct split_nuh_item nuh;
-  char nick[NICKLEN + 1];
-  char user[USERLEN + 1];
-  char host[HOSTLEN + 1];
-  char *p = NULL;
   char *mask = collapse(parv[1]);
 
   if (EmptyString(mask) || strcmp(mask, "*") == 0)
   {
-    list_accepts(source_p);
+    accept_list(source);
     return;
   }
 
+  char nick[NICKLEN + 1];
+  char user[USERLEN + 1];
+  char host[HOSTLEN + 1];
+  struct nuh_split nuh =
+  {
+    .nickptr  = nick,
+    .userptr  = user,
+    .hostptr  = host,
+    .nicksize = sizeof(nick),
+    .usersize = sizeof(user),
+    .hostsize = sizeof(host)
+  };
+
+  char *p = NULL;
   for (mask = strtok_r(mask, ",", &p); mask;
        mask = strtok_r(NULL, ",", &p))
   {
     if (*mask == '-' && *++mask)
     {
-      nuh.nuhmask  = mask;
-      nuh.nickptr  = nick;
-      nuh.userptr  = user;
-      nuh.hostptr  = host;
-
-      nuh.nicksize = sizeof(nick);
-      nuh.usersize = sizeof(user);
-      nuh.hostsize = sizeof(host);
-
-      split_nuh(&nuh);
+      nuh.nuhmask = mask;
+      nuh_split(&nuh);
 
       struct AcceptItem *accept =
-        accept_find(nick, user, host, &source_p->connection->acceptlist, irccmp);
+        accept_find(nick, user, host, &source->connection->acceptlist, irccmp);
       if (accept == NULL)
       {
-        sendto_one_numeric(source_p, &me, ERR_ACCEPTNOT, nick, user, host);
+        sendto_one_numeric(source, &me, ERR_ACCEPTNOT, nick, user, host);
         continue;
       }
 
-      accept_del(accept, &source_p->connection->acceptlist);
+      accept_del(accept, &source->connection->acceptlist);
     }
     else if (*mask)
     {
-      if (dlink_list_length(&source_p->connection->acceptlist) >= ConfigGeneral.max_accept)
+      if (list_length(&source->connection->acceptlist) >= ConfigGeneral.max_accept)
       {
-        sendto_one_numeric(source_p, &me, ERR_ACCEPTFULL);
+        sendto_one_numeric(source, &me, ERR_ACCEPTFULL);
         return;
       }
 
-      nuh.nuhmask  = mask;
-      nuh.nickptr  = nick;
-      nuh.userptr  = user;
-      nuh.hostptr  = host;
-
-      nuh.nicksize = sizeof(nick);
-      nuh.usersize = sizeof(user);
-      nuh.hostsize = sizeof(host);
-
-      split_nuh(&nuh);
+      nuh.nuhmask = mask;
+      nuh_split(&nuh);
 
       struct AcceptItem *accept =
-        accept_find(nick, user, host, &source_p->connection->acceptlist, irccmp);
+        accept_find(nick, user, host, &source->connection->acceptlist, irccmp);
       if (accept)
       {
-        sendto_one_numeric(source_p, &me, ERR_ACCEPTEXIST, nick, user, host);
+        sendto_one_numeric(source, &me, ERR_ACCEPTEXIST, nick, user, host);
         continue;
       }
 
-      accept_add(nick, user, host, source_p);
+      accept_add(nick, user, host, &source->connection->acceptlist);
+      accept_list(source);
     }
   }
 }
 
-static struct Message accept_msgtab =
+static struct Command command_table =
 {
-  .cmd = "ACCEPT",
+  .name = "ACCEPT",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_accept },
   .handlers[SERVER_HANDLER] = { .handler = m_ignore },
@@ -197,19 +167,19 @@ static struct Message accept_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&accept_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&accept_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
