@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "client.h"
 #include "irc_string.h"
 #include "ircd.h"
@@ -32,34 +33,32 @@
 #include "server.h"
 #include "send.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 
 
 /*! \brief Sends administrative information about this server.
  *
- * \param source_p Pointer to client to report to
+ * \param source Pointer to client to report to
  */
 static void
-do_admin(struct Client *source_p)
+do_admin(struct Client *source)
 {
-  sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE,
-                       "ADMIN requested by %s (%s@%s) [%s]",
-                       source_p->name, source_p->username,
-                       source_p->host, source_p->servptr->name);
+  sendto_clients(UMODE_SPY, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "ADMIN requested by %s (%s@%s) [%s]",
+                 source->name, source->username, source->host, source->servptr->name);
 
-  sendto_one_numeric(source_p, &me, RPL_ADMINME, me.name);
+  sendto_one_numeric(source, &me, RPL_ADMINME, me.name);
 
   if (!EmptyString(ConfigAdminInfo.name))
-    sendto_one_numeric(source_p, &me, RPL_ADMINLOC1, ConfigAdminInfo.name);
+    sendto_one_numeric(source, &me, RPL_ADMINLOC1, ConfigAdminInfo.name);
   if (!EmptyString(ConfigAdminInfo.description))
-    sendto_one_numeric(source_p, &me, RPL_ADMINLOC2, ConfigAdminInfo.description);
+    sendto_one_numeric(source, &me, RPL_ADMINLOC2, ConfigAdminInfo.description);
   if (!EmptyString(ConfigAdminInfo.email))
-    sendto_one_numeric(source_p, &me, RPL_ADMINEMAIL, ConfigAdminInfo.email);
+    sendto_one_numeric(source, &me, RPL_ADMINEMAIL, ConfigAdminInfo.email);
 }
 
 /*! \brief ADMIN command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -69,28 +68,28 @@ do_admin(struct Client *source_p)
  *      - parv[1] = nickname/servername
  */
 static void
-m_admin(struct Client *source_p, int parc, char *parv[])
+m_admin(struct Client *source, int parc, char *parv[])
 {
   static uintmax_t last_used = 0;
 
-  if ((last_used + ConfigGeneral.pace_wait_simple) > event_base->time.sec_monotonic)
+  if ((last_used + ConfigGeneral.pace_wait_simple) > io_time_get(IO_TIME_MONOTONIC_SEC))
   {
-    sendto_one_numeric(source_p, &me, RPL_LOAD2HI, "ADMIN");
+    sendto_one_numeric(source, &me, RPL_LOAD2HI, "ADMIN");
     return;
   }
 
-  last_used = event_base->time.sec_monotonic;
+  last_used = io_time_get(IO_TIME_MONOTONIC_SEC);
 
   if (ConfigServerHide.disable_remote_commands == 0)
-    if (server_hunt(source_p, ":%s ADMIN :%s", 1, parv)->ret != HUNTED_ISME)
+    if (server_route_command(source, ":%s ADMIN :%s", 1, parv)->result != SERVER_ROUTE_ISME)
       return;
 
-  do_admin(source_p);
+  do_admin(source);
 }
 
 /*! \brief ADMIN command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -100,17 +99,17 @@ m_admin(struct Client *source_p, int parc, char *parv[])
  *      - parv[1] = nickname/servername
  */
 static void
-ms_admin(struct Client *source_p, int parc, char *parv[])
+ms_admin(struct Client *source, int parc, char *parv[])
 {
-  if (server_hunt(source_p, ":%s ADMIN :%s", 1, parv)->ret != HUNTED_ISME)
+  if (server_route_command(source, ":%s ADMIN :%s", 1, parv)->result != SERVER_ROUTE_ISME)
     return;
 
-  do_admin(source_p);
+  do_admin(source);
 }
 
-static struct Message admin_msgtab =
+static struct Command command_table =
 {
-  .cmd = "ADMIN",
+  .name = "ADMIN",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_admin },
   .handlers[SERVER_HANDLER] = { .handler = ms_admin },
@@ -119,19 +118,19 @@ static struct Message admin_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&admin_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&admin_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
