@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1998-2022 ircd-hybrid development team
+ *  Copyright (c) 1998-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,15 +25,14 @@
 
 #include "stdinc.h"
 #include "client.h"
-#include "irc_string.h"
 #include "server_capab.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 
 
 /*! \brief CAPAB command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -43,23 +42,23 @@
  *      - parv[1] = space-separated list of capabilities
  */
 static void
-mr_capab(struct Client *source_p, int parc, char *parv[])
+mr_capab(struct Client *source, int parc, char *parv[])
 {
   char *const list = parv[1];
   char *p = NULL;
 
-  for (const char *s = strtok_r(list, " ", &p); s;
-                   s = strtok_r(NULL, " ", &p))
+  for (const char *name = strtok_r(list, " ", &p); name;
+                   name = strtok_r(NULL, " ", &p))
   {
-    unsigned int cap = capab_find(s);
+    unsigned int cap = capab_find(name);
     if (cap)
-      SetCapable(source_p, cap);
+      SetCapable(source, cap);
   }
 }
 
-static struct Message capab_msgtab =
+static struct Command command_table =
 {
-  .cmd = "CAPAB",
+  .name = "CAPAB",
   .handlers[UNREGISTERED_HANDLER] = { .handler = mr_capab, .args_min = 2 },
   .handlers[CLIENT_HANDLER] = { .handler = m_ignore },
   .handlers[SERVER_HANDLER] = { .handler = m_ignore },
@@ -68,19 +67,19 @@ static struct Message capab_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&capab_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&capab_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
