@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2013-2022 ircd-hybrid development team
+ *  Copyright (c) 2013-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,13 +28,13 @@
 #include "ircd.h"
 #include "send.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 #include "memory.h"
 
 
 /*! \brief CERTFP command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -44,21 +44,21 @@
  *      - parv[1] = certificate fingerprint
  */
 static void
-ms_certfp(struct Client *source_p, int parc, char *parv[])
+ms_certfp(struct Client *source, int parc, char *parv[])
 {
-  if (!IsClient(source_p))
+  if (!IsClient(source))
     return;
 
-  xfree(source_p->tls_certfp);
-  source_p->tls_certfp = xstrdup(parv[1]);
+  io_free(source->tls_certfp);
+  source->tls_certfp = io_strdup(parv[1]);
 
-  sendto_server(source_p, 0, 0, ":%s CERTFP %s",
-                source_p->id, source_p->tls_certfp);
+  sendto_servers(source, 0, 0, ":%s CERTFP %s",
+                 source->id, source->tls_certfp);
 }
 
-static struct Message certfp_msgtab =
+static struct Command command_table =
 {
-  .cmd = "CERTFP",
+  .name = "CERTFP",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_ignore },
   .handlers[CLIENT_HANDLER] = { .handler = m_ignore },
   .handlers[SERVER_HANDLER] = { .handler = ms_certfp, .args_min = 2 },
@@ -67,19 +67,19 @@ static struct Message certfp_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&certfp_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&certfp_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
