@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,12 +30,12 @@
 #include "numeric.h"
 #include "send.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 
 
 /*! \brief CLOSE command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -44,36 +44,35 @@
  *      - parv[0] = command
  */
 static void
-mo_close(struct Client *source_p, int parc, char *parv[])
+mo_close(struct Client *source, int parc, char *parv[])
 {
-  if (!HasOFlag(source_p, OPER_FLAG_CLOSE))
+  if (!HasOFlag(source, OPER_FLAG_CLOSE))
   {
-    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "close");
+    sendto_one_numeric(source, &me, ERR_NOPRIVS, "close");
     return;
   }
 
-  const unsigned int closed = dlink_list_length(&unknown_list);
+  const unsigned int closed = list_length(&unknown_list);
   while (unknown_list.head)
   {
-    struct Client *target_p = unknown_list.head->data;
+    struct Client *target = unknown_list.head->data;
 
-    sendto_one_numeric(source_p, &me, RPL_CLOSING,
-                       client_get_name(target_p, SHOW_IP),
-                       target_p->status);
+    sendto_one_numeric(source, &me, RPL_CLOSING,
+                       client_get_name(target, SHOW_IP), target->status);
 
     /*
-     * Exit here is safe, because it is guaranteed not to be source_p
-     * because it is unregistered and source_p is an oper.
+     * Exit here is safe, because it is guaranteed not to be source
+     * because it is unregistered and source is an oper.
      */
-    exit_client(target_p, "Oper Closing");
+    client_exit(target, "Oper Closing");
   }
 
-  sendto_one_numeric(source_p, &me, RPL_CLOSEEND, closed);
+  sendto_one_numeric(source, &me, RPL_CLOSEEND, closed);
 }
 
-static struct Message close_msgtab =
+static struct Command command_table =
 {
-  .cmd = "CLOSE",
+  .name = "CLOSE",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_not_oper },
   .handlers[SERVER_HANDLER] = { .handler = m_ignore },
@@ -82,19 +81,19 @@ static struct Message close_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&close_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&close_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
