@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,14 +32,78 @@
 #include "log.h"
 #include "server.h"
 #include "send.h"
+#include "user_mode.h"
 #include "parse.h"
 #include "hash.h"
-#include "modules.h"
+#include "module.h"
 
 
+static void
+do_connect(struct Client *source, const char *name)
+{
+  assert(IsClient(source));
+
+  /*
+   * Try to find the name. If it fails, notify and bail.
+   */
+  struct MaskItem *conf = connect_find(name, match);
+  if (conf == NULL)
+  {
+    sendto_one_notice(source, &me, ":Connect: Server %s not listed in configuration file", name);
+    return;
+  }
+
+  const struct Client *target = hash_find_server(conf->name);
+  if (target)
+  {
+    sendto_one_notice(source, &me, ":Connect: Server %s already exists from %s",
+                      target->name, target->from->name);
+    return;
+  }
+
+  if (find_servconn_in_progress(conf->name))
+  {
+    sendto_one_notice(source, &me, ":Connect: a connection to %s is already in progress",
+                      conf->name);
+    return;
+  }
+
+  /* Notify all operators about connect requests. */
+  static const char *const type[] = { "Remote", "Local" };
+  const char *const type_p = type[MyConnect(source) != 0];
+
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_GLOBAL, "from %s: %s CONNECT %s %u from %s",
+                 me.name, type_p, name, conf->port, get_oper_name(source));
+  sendto_servers(NULL, 0, 0, ":%s GLOBOPS :%s CONNECT %s %u from %s",
+                 me.id, type_p, name, conf->port, get_oper_name(source));
+
+  log_write(LOG_TYPE_IRCD, "%s CONNECT %s %u from %s",
+            type_p, name, conf->port, get_oper_name(source));
+
+  /*
+   * At this point we should be calling connect_server with a valid
+   * connect{} block and a valid port in the connect{} block.
+   */
+  if (server_connect(conf, source) == false)
+    sendto_one_notice(source, &me, ":*** Couldn't connect to %s.%u",
+                      conf->name, conf->port);
+  else if (MyConnect(source) &&
+           (ConfigServerHide.hide_server_ips == 0 && user_mode_has_flag(source, UMODE_ADMIN)))
+    sendto_one_notice(source, &me, ":*** Connecting to %s[%s].%u",
+                      conf->name, conf->host, conf->port);
+  else
+    sendto_one_notice(source, &me, ":*** Connecting to %s.%u",
+                      conf->name, conf->port);
+
+  /*
+   * Client is either connecting with all the data it needs or has been
+   * destroyed.
+   */
+}
+
 /*! \brief CONNECT command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -51,82 +115,33 @@
  *      - parv[3] = nickname/servername
  */
 static void
-mo_connect(struct Client *source_p, int parc, char *parv[])
+mo_connect(struct Client *source, int parc, char *parv[])
 {
-  const char *const name = parv[1];
-
   if (!EmptyString(parv[3]))
   {
-    if (!HasOFlag(source_p, OPER_FLAG_CONNECT_REMOTE))
+    if (!HasOFlag(source, OPER_FLAG_CONNECT_REMOTE))
     {
-      sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "connect:remote");
+      sendto_one_numeric(source, &me, ERR_NOPRIVS, "connect:remote");
       return;
     }
 
-    if (server_hunt(source_p, ":%s CONNECT %s %s :%s", 3, parv)->ret != HUNTED_ISME)
+    if (server_route_command(source, ":%s CONNECT %s %s :%s", 3, parv)->result != SERVER_ROUTE_ISME)
       return;
   }
 
-  if (!HasOFlag(source_p, OPER_FLAG_CONNECT))
+  if (!HasOFlag(source, OPER_FLAG_CONNECT))
   {
-    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "connect");
+    sendto_one_numeric(source, &me, ERR_NOPRIVS, "connect");
     return;
   }
 
-  /*
-   * Try to find the name. If it fails, notify and bail.
-   */
-  struct MaskItem *conf = connect_find(name, match);
-  if (conf == NULL)
-  {
-    sendto_one_notice(source_p, &me, ":Connect: Server %s not listed in configuration file", name);
-    return;
-  }
-
-  const struct Client *target_p = hash_find_server(conf->name);
-  if (target_p)
-  {
-    sendto_one_notice(source_p, &me, ":Connect: Server %s already exists from %s",
-                      target_p->name, target_p->from->name);
-    return;
-  }
-
-  if (find_servconn_in_progress(conf->name))
-  {
-    sendto_one_notice(source_p, &me, ":Connect: a connection to %s is already in progress",
-                      conf->name);
-    return;
-  }
-
-  ilog(LOG_TYPE_IRCD, "CONNECT %s %u from %s",
-       name, conf->port, get_oper_name(source_p));
-
-  /*
-   * At this point we should be calling connect_server with a valid
-   * connect{} block and a valid port in the connect{} block
-   */
-  if (server_connect(conf, source_p) == true)
-  {
-    if (ConfigServerHide.hide_server_ips == 0 && HasUMode(source_p, UMODE_ADMIN))
-      sendto_one_notice(source_p, &me, ":*** Connecting to %s[%s].%u",
-                        conf->name, conf->host, conf->port);
-    else
-      sendto_one_notice(source_p, &me, ":*** Connecting to %s.%u",
-                        conf->name, conf->port);
-  }
-  else
-    sendto_one_notice(source_p, &me, ":*** Couldn't connect to %s.%u",
-                      conf->name, conf->port);
-
-  /*
-   * Client is either connecting with all the data it needs or has been
-   * destroyed
-   */
+  const char *const name = parv[1];
+  do_connect(source, name);
 }
 
 /*! \brief CONNECT command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -138,69 +153,18 @@ mo_connect(struct Client *source_p, int parc, char *parv[])
  *      - parv[3] = nickname/servername
  */
 static void
-ms_connect(struct Client *source_p, int parc, char *parv[])
+ms_connect(struct Client *source, int parc, char *parv[])
 {
+  if (server_route_command(source, ":%s CONNECT %s %s :%s", 3, parv)->result != SERVER_ROUTE_ISME)
+    return;
+
   const char *const name = parv[1];
-
-  if (server_hunt(source_p, ":%s CONNECT %s %s :%s", 3, parv)->ret != HUNTED_ISME)
-    return;
-
-  /*
-   * Try to find the name. If it fails, notify and bail.
-   */
-  struct MaskItem *conf = connect_find(name, match);
-  if (conf == NULL)
-  {
-    sendto_one_notice(source_p, &me, ":Connect: Server %s not listed in configuration file", name);
-    return;
-  }
-
-  const struct Client *target_p = hash_find_server(conf->name);
-  if (target_p)
-  {
-    sendto_one_notice(source_p, &me, ":Connect: Server %s already exists from %s",
-                      target_p->name, target_p->from->name);
-    return;
-  }
-
-  if (find_servconn_in_progress(conf->name))
-  {
-    sendto_one_notice(source_p, &me, ":Connect: a connection to %s is already in progress",
-                      conf->name);
-    return;
-  }
-
-  /*
-   * Notify all operators about remote connect requests
-   */
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_GLOBAL, "from %s: Remote CONNECT %s %u from %s",
-                       me.name, name, conf->port, get_oper_name(source_p));
-  sendto_server(NULL, 0, 0, ":%s GLOBOPS :Remote CONNECT %s %u from %s",
-                me.id, name, conf->port, get_oper_name(source_p));
-
-  ilog(LOG_TYPE_IRCD, "Remote CONNECT %s %u from %s",
-       name, conf->port, get_oper_name(source_p));
-
-  /*
-   * At this point we should be calling connect_server with a valid
-   * connect{} block and a valid port in the connect{} block
-   */
-  if (server_connect(conf, source_p) == true)
-    sendto_one_notice(source_p, &me, ":*** Connecting to %s.%u",
-                      conf->name, conf->port);
-  else
-    sendto_one_notice(source_p, &me, ":*** Couldn't connect to %s.%u",
-                      conf->name, conf->port);
-
-  /*
-   * Client is either connecting with all the data it needs or has been
-   * destroyed
-   */
+  do_connect(source, name);
 }
 
-static struct Message connect_msgtab =
+static struct Command command_table =
 {
-  .cmd = "CONNECT",
+  .name = "CONNECT",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_not_oper },
   .handlers[SERVER_HANDLER] = { .handler = ms_connect, .args_min = 4 },
@@ -209,19 +173,19 @@ static struct Message connect_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&connect_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&connect_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
