@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "list.h"
 #include "client.h"
 #include "irc_string.h"
@@ -31,27 +32,28 @@
 #include "conf_cluster.h"
 #include "conf_shared.h"
 #include "ircd.h"
-#include "hostmask.h"
+#include "address.h"
 #include "numeric.h"
 #include "log.h"
 #include "misc.h"
 #include "send.h"
 #include "server_capab.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 #include "memory.h"
+#include "aline.h"
 
 
 static void
 dline_check(const struct AddressRec *arec)
 {
-  dlink_list *tab[] = { &local_client_list, &unknown_list, NULL };
+  list_t *tab[] = { &local_client_list, &unknown_list, NULL };
 
-  for (dlink_list **list = tab; *list; ++list)
+  for (list_t **list = tab; *list; ++list)
   {
-    dlink_node *node, *node_next;
+    list_node_t *node, *node_next;
 
-    DLINK_FOREACH_SAFE(node, node_next, (*list)->head)
+    LIST_FOREACH_SAFE(node, node_next, (*list)->head)
     {
       struct Client *client = node->data;
 
@@ -62,7 +64,7 @@ dline_check(const struct AddressRec *arec)
       {
         case HM_IPV6:
         case HM_IPV4:
-          if (address_compare(&client->ip, &arec->Mask.ipa.addr, false, false, arec->Mask.ipa.bits) == true)
+          if (address_compare(&client->addr, &arec->Mask.ipa.addr, false, false, arec->Mask.ipa.bits))
             conf_try_ban(client, CLIENT_BAN_DLINE, arec->conf->reason);
           break;
         default:
@@ -79,46 +81,39 @@ dline_check(const struct AddressRec *arec)
  * side effects	- dline as given is placed
  */
 static void
-dline_handle(struct Client *source_p, const struct aline_ctx *aline)
+dline_handle(struct Client *source, const struct aline_ctx *aline)
 {
   char buf[IRCD_BUFSIZE];
-  struct irc_ssaddr addr;
+  struct io_addr addr;
   int bits = 0;
+  unsigned int min_cidr = 0;
 
   switch (parse_netmask(aline->host, &addr, &bits))
   {
     case HM_IPV4:
-      if (!HasFlag(source_p, FLAGS_SERVICE) && (unsigned int)bits < ConfigGeneral.dline_min_cidr)
-      {
-        if (IsClient(source_p))
-          sendto_one_notice(source_p, &me, ":For safety, bitmasks less than %u require conf access.",
-                            ConfigGeneral.dline_min_cidr);
-        return;
-      }
-
+      min_cidr = ConfigGeneral.dline_min_cidr;
       break;
     case HM_IPV6:
-      if (!HasFlag(source_p, FLAGS_SERVICE) && (unsigned int)bits < ConfigGeneral.dline_min_cidr6)
-      {
-        if (IsClient(source_p))
-          sendto_one_notice(source_p, &me, ":For safety, bitmasks less than %u require conf access.",
-                            ConfigGeneral.dline_min_cidr6);
-        return;
-      }
-
+      min_cidr = ConfigGeneral.dline_min_cidr6;
       break;
     default:  /* HM_HOST */
-      if (IsClient(source_p))
-        sendto_one_notice(source_p, &me, ":Invalid D-Line");
+      if (IsClient(source))
+        sendto_one_notice(source, &me, ":Invalid D-Line");
+      return;
+  }
 
-     return;
+  if (min_cidr > 0 && !HasFlag(source, FLAGS_SERVICE) && (unsigned int)bits < min_cidr)
+  {
+    if (IsClient(source))
+      sendto_one_notice(source, &me, ":For safety, bitmasks less than %u require conf access.", min_cidr);
+    return;
   }
 
   struct MaskItem *conf;
   if ((conf = find_conf_by_address(NULL, &addr, CONF_DLINE, NULL, NULL, 1)))
   {
-    if (IsClient(source_p))
-      sendto_one_notice(source_p, &me, ":[%s] already D-lined by [%s] - %s",
+    if (IsClient(source))
+      sendto_one_notice(source, &me, ":[%s] already D-lined by [%s] - %s",
                         aline->host, conf->host, conf->reason);
     return;
   }
@@ -130,36 +125,34 @@ dline_handle(struct Client *source_p, const struct aline_ctx *aline)
     snprintf(buf, sizeof(buf), "%.*s (%s)", REASONLEN, aline->reason, date_iso8601(0));
 
   conf = conf_make(CONF_DLINE);
-  conf->host = xstrdup(aline->host);
-  conf->reason = xstrdup(buf);
-  conf->setat = event_base->time.sec_real;
+  conf->host = io_strdup(aline->host);
+  conf->reason = io_strdup(buf);
+  conf->setat = io_time_get(IO_TIME_REALTIME_SEC);
   SetConfDatabase(conf);
 
   if (aline->duration)
   {
-    conf->until = event_base->time.sec_real + aline->duration;
+    conf->until = conf->setat + aline->duration;
 
-    if (IsClient(source_p))
-      sendto_one_notice(source_p, &me, ":Added temporary %ju min. D-Line [%s]",
+    if (IsClient(source))
+      sendto_one_notice(source, &me, ":Added temporary %ju min. D-Line [%s]",
                         aline->duration / 60, conf->host);
 
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "%s added temporary %ju min. D-Line for [%s] [%s]",
-                         get_oper_name(source_p), aline->duration / 60,
-                         conf->host, conf->reason);
-    ilog(LOG_TYPE_DLINE, "%s added temporary %ju min. D-Line for [%s] [%s]",
-         get_oper_name(source_p), aline->duration / 60, conf->host, conf->reason);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "%s added temporary %ju min. D-Line for [%s] [%s]",
+                   get_oper_name(source), aline->duration / 60, conf->host, conf->reason);
+    log_write(LOG_TYPE_DLINE, "%s added temporary %ju min. D-Line for [%s] [%s]",
+              get_oper_name(source), aline->duration / 60, conf->host, conf->reason);
   }
   else
   {
-    if (IsClient(source_p))
-      sendto_one_notice(source_p, &me, ":Added D-Line [%s]", conf->host);
+    if (IsClient(source))
+      sendto_one_notice(source, &me, ":Added D-Line [%s]", conf->host);
 
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "%s added D-Line for [%s] [%s]",
-                         get_oper_name(source_p), conf->host, conf->reason);
-    ilog(LOG_TYPE_DLINE, "%s added D-Line for [%s] [%s]",
-         get_oper_name(source_p), conf->host, conf->reason);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "%s added D-Line for [%s] [%s]",
+                   get_oper_name(source), conf->host, conf->reason);
+    log_write(LOG_TYPE_DLINE, "%s added D-Line for [%s] [%s]",
+              get_oper_name(source), conf->host, conf->reason);
   }
 
   dline_check(add_conf_by_address(CONF_DLINE, conf));
@@ -176,22 +169,22 @@ dline_handle(struct Client *source_p, const struct aline_ctx *aline)
  *
  */
 static void
-mo_dline(struct Client *source_p, int parc, char *parv[])
+mo_dline(struct Client *source, int parc, char *parv[])
 {
   struct aline_ctx aline = { .add = true, .simple_mask = false };
 
-  if (!HasOFlag(source_p, OPER_FLAG_DLINE))
+  if (!HasOFlag(source, OPER_FLAG_DLINE))
   {
-    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "dline");
+    sendto_one_numeric(source, &me, ERR_NOPRIVS, "dline");
     return;
   }
 
-  if (parse_aline("DLINE", source_p, parc, parv, &aline) == false)
+  if (aline_parse("DLINE", source, parc, parv, &aline) == false)
     return;
 
   if (aline.server)
   {
-    sendto_match_servs(source_p, aline.server, CAPAB_DLN, "DLINE %s %ju %s :%s",
+    sendto_match_servs(source, aline.server, CAPAB_DLN, "DLINE %s %ju %s :%s",
                        aline.server, aline.duration, aline.host, aline.reason);
 
     /* Allow ON to apply local dline as well if it matches */
@@ -199,15 +192,15 @@ mo_dline(struct Client *source_p, int parc, char *parv[])
       return;
   }
   else
-    cluster_distribute(source_p, "DLINE", CAPAB_DLN, CLUSTER_DLINE,
-                       "%ju %s :%s", aline.duration, aline.host, aline.reason);
+    cluster_distribute(source, "DLINE", CAPAB_DLN, CLUSTER_DLINE, "%ju %s :%s",
+                       aline.duration, aline.host, aline.reason);
 
-  dline_handle(source_p, &aline);
+  dline_handle(source, &aline);
 }
 
 /*! \brief DLINE command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -220,7 +213,7 @@ mo_dline(struct Client *source_p, int parc, char *parv[])
  *      - parv[4] = reason
  */
 static void
-ms_dline(struct Client *source_p, int parc, char *parv[])
+ms_dline(struct Client *source, int parc, char *parv[])
 {
   struct aline_ctx aline =
   {
@@ -232,21 +225,20 @@ ms_dline(struct Client *source_p, int parc, char *parv[])
     .duration = strtoumax(parv[2], NULL, 10)
   };
 
-  sendto_match_servs(source_p, aline.server, CAPAB_DLN, "DLINE %s %ju %s :%s",
+  sendto_match_servs(source, aline.server, CAPAB_DLN, "DLINE %s %ju %s :%s",
                      aline.server, aline.duration, aline.host, aline.reason);
 
   if (match(aline.server, me.name))
     return;
 
-  if (HasFlag(source_p, FLAGS_SERVICE) ||
-      shared_find(SHARED_DLINE, source_p->servptr->name,
-                  source_p->username, source_p->host))
-    dline_handle(source_p, &aline);
+  if (HasFlag(source, FLAGS_SERVICE) ||
+      shared_find(SHARED_DLINE, source->servptr->name, source->username, source->host))
+    dline_handle(source, &aline);
 }
 
-static struct Message dline_msgtab =
+static struct Command command_table =
 {
-  .cmd = "DLINE",
+  .name = "DLINE",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_not_oper },
   .handlers[SERVER_HANDLER] = { .handler = ms_dline, .args_min = 5 },
@@ -255,21 +247,21 @@ static struct Message dline_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&dline_msgtab);
+  command_add(&command_table);
   capab_add("DLN", CAPAB_DLN, true);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&dline_msgtab);
+  command_del(&command_table);
   capab_del("DLN");
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
