@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2003-2022 ircd-hybrid development team
+ *  Copyright (c) 2003-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
 #include "parse.h"
 #include "server_capab.h"
 #include "send.h"
-#include "modules.h"
+#include "module.h"
 #include "irc_string.h"
 
 
@@ -40,7 +40,7 @@
  * side effects	- propagates subcommand to locally connected servers
  */
 static void
-ms_encap(struct Client *source_p, int parc, char *parv[])
+ms_encap(struct Client *source, int parc, char *parv[])
 {
   char buf[IRCD_BUFSIZE];
   char *bufptr = buf;
@@ -65,18 +65,18 @@ ms_encap(struct Client *source_p, int parc, char *parv[])
   else
     snprintf(bufptr, sizeof(buf) - cur_len, ":%s", parv[parc - 1]);
 
-  sendto_match_servs(source_p, parv[1], CAPAB_ENCAP, "ENCAP %s", buf);
+  sendto_match_servs(source, parv[1], CAPAB_ENCAP, "ENCAP %s", buf);
 
   if (match(parv[1], me.name))
     return;
 
-  struct Message *message = find_command(parv[2]);
-  if (message == NULL)
+  struct Command *command = command_find(parv[2]);
+  if (command == NULL)
     return;
 
-  const struct MessageHandler *const handler = &message->handlers[ENCAP_HANDLER];
-  message->bytes += strlen(buf);
-  message->ecount++;
+  const struct CommandHandler *const handler = &command->handlers[ENCAP_HANDLER];
+  command->bytes += strlen(buf);
+  command->ecount++;
 
   parv += 2;
   parc -= 2;
@@ -86,12 +86,12 @@ ms_encap(struct Client *source_p, int parc, char *parv[])
        (handler->empty_last_arg != true && EmptyString(parv[handler->args_min - 1]))))
     return;
 
-  handler->handler(source_p, parc, parv);
+  handler->handler(source, parc, parv);
 }
 
-static struct Message encap_msgtab =
+static struct Command command_table =
 {
-  .cmd = "ENCAP",
+  .name = "ENCAP",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_ignore },
   .handlers[CLIENT_HANDLER] = { .handler = m_ignore },
   .handlers[SERVER_HANDLER] = { .handler = ms_encap, .args_min = 3 },
@@ -100,21 +100,21 @@ static struct Message encap_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&encap_msgtab);
+  command_add(&command_table);
   capab_add("ENCAP", CAPAB_ENCAP, true);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&encap_msgtab);
+  command_del(&command_table);
   capab_del("ENCAP");
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
