@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2000-2022 ircd-hybrid development team
+ *  Copyright (c) 2000-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,17 +24,17 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
+#include "module.h"
 #include "client.h"
 #include "ircd.h"
 #include "send.h"
 #include "parse.h"
-#include "modules.h"
-#include "server.h"
 
 
 /*! \brief EOB command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -43,22 +43,22 @@
  *      - parv[0] = command
  */
 static void
-ms_eob(struct Client *source_p, int parc, char *parv[])
+ms_eob(struct Client *source, int parc, char *parv[])
 {
-  assert(IsServer(source_p));
+  assert(IsServer(source));
 
-  if (MyConnect(source_p))
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "End of burst from %s (%ju seconds)", source_p->name,
-                         (event_base->time.sec_monotonic - source_p->connection->created_monotonic));
+  if (MyConnect(source))
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "End of burst from %s (%ju seconds)", source->name,
+                   (io_time_get(IO_TIME_MONOTONIC_SEC) - source->connection->created_monotonic));
 
-  AddFlag(source_p, FLAGS_EOB);
-  sendto_server(source_p, 0, 0, ":%s EOB", source_p->id);
+  AddFlag(source, FLAGS_EOB);
+  sendto_servers(source, 0, 0, ":%s EOB", source->id);
 }
 
-static struct Message eob_msgtab =
+static struct Command command_table =
 {
-  .cmd = "EOB",
+  .name = "EOB",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered  },
   .handlers[CLIENT_HANDLER] = { .handler = m_ignore },
   .handlers[SERVER_HANDLER] = { .handler = ms_eob },
@@ -67,19 +67,19 @@ static struct Message eob_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&eob_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&eob_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
