@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2004-2022 ircd-hybrid development team
+ *  Copyright (c) 2004-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,8 +32,9 @@
 #include "numeric.h"
 #include "server.h"
 #include "send.h"
+#include "user_mode.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 #include "conf.h"
 
 
@@ -46,56 +47,49 @@
  * side effects - NONE
  */
 static void
-report_this_status(struct Client *source_p, const struct Client *target_p)
+trace_send_status(struct Client *source, const struct Client *target)
 {
-  if (target_p->status != STAT_CLIENT)
+  if (target->status != STAT_CLIENT)
     return;
 
-  sendto_one_numeric(source_p, &me, RPL_ETRACE,
-                     HasUMode(target_p, UMODE_OPER) ? "Oper" : "User",
-                     class_get_name(&target_p->connection->confs),
-                     target_p->name,
-                     target_p->username,
-                     target_p->host,
-                     target_p->sockhost,
-                     target_p->info);
+  sendto_one_numeric(source, &me, RPL_ETRACE,
+                     user_mode_has_flag(target, UMODE_OPER) ? "Oper" : "User",
+                     class_get_name(&target->connection->confs),
+                     target->name, target->username, target->host, target->sockhost, target->info);
 }
 
 /*
  * do_etrace()
  */
 static void
-do_etrace(struct Client *source_p, const char *name)
+do_etrace(struct Client *source, const char *name)
 {
+  sendto_clients(UMODE_SPY, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "ETRACE requested by %s (%s@%s) [%s]",
+                 source->name, source->username, source->host, source->servptr->name);
+
   bool doall = false;
-  dlink_node *node;
-
-  sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE,
-                       "ETRACE requested by %s (%s@%s) [%s]",
-                       source_p->name, source_p->username,
-                       source_p->host, source_p->servptr->name);
-
   if (EmptyString(name))
     doall = true;
   else if (match(name, me.name) == 0)
     doall = true;
-  else if (!MyClient(source_p) && strcmp(name, me.id) == 0)
+  else if (!MyClient(source) && strcmp(name, me.id) == 0)
     doall = true;
 
-  DLINK_FOREACH(node, local_client_list.head)
+  list_node_t *node;
+  LIST_FOREACH(node, local_client_list.head)
   {
-    const struct Client *target_p = node->data;
+    const struct Client *target = node->data;
 
-    if (doall == true || match(name, target_p->name) == 0)
-      report_this_status(source_p, target_p);
+    if (doall || match(name, target->name) == 0)
+      trace_send_status(source, target);
   }
 
-  sendto_one_numeric(source_p, &me, RPL_ETRACEEND, me.name);
+  sendto_one_numeric(source, &me, RPL_ETRACEEND, me.name);
 }
 
 /*! \brief ETRACE command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -106,30 +100,30 @@ do_etrace(struct Client *source_p, const char *name)
  *      - parv[2] = nick or server name to forward the etrace to
  */
 static void
-mo_etrace(struct Client *source_p, int parc, char *parv[])
+mo_etrace(struct Client *source, int parc, char *parv[])
 {
   if (parc > 2)
-    if (server_hunt(source_p, ":%s ETRACE %s :%s", 2, parv)->ret != HUNTED_ISME)
+    if (server_route_command(source, ":%s ETRACE %s :%s", 2, parv)->result != SERVER_ROUTE_ISME)
       return;
 
-  const struct server_hunt *hunt = server_hunt(source_p, ":%s ETRACE :%s", 1, parv);
-  switch (hunt->ret)
+  const server_route_t *route = server_route_command(source, ":%s ETRACE :%s", 1, parv);
+  switch (route->result)
   {
-    case HUNTED_PASS:
-      sendto_one_numeric(source_p, &me, RPL_TRACELINK,
-                         IRCD_VERSION, hunt->target_p->name, hunt->target_p->from->name);
+    case SERVER_ROUTE_PASS:
+      sendto_one_numeric(source, &me, RPL_TRACELINK,
+                         IRCD_VERSION, route->target->name, route->target->from->name);
       break;
-    case HUNTED_ISME:
-      do_etrace(source_p, parv[1]);
+    case SERVER_ROUTE_ISME:
+      do_etrace(source, parv[1]);
       break;
     default:
       break;
   }
 }
 
-static struct Message etrace_msgtab =
+static struct Command command_table =
 {
-  .cmd = "ETRACE",
+  .name = "ETRACE",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_not_oper },
   .handlers[SERVER_HANDLER] = { .handler = mo_etrace },
@@ -138,19 +132,19 @@ static struct Message etrace_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&etrace_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&etrace_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
