@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2011-2022 ircd-hybrid development team
+ *  Copyright (c) 2011-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,17 +26,15 @@
 #include "stdinc.h"
 #include "client.h"
 #include "ircd.h"
-#include "irc_string.h"
 #include "numeric.h"
 #include "send.h"
-#include "server.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 
 
 /*! \brief GLOBOPS command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -46,24 +44,24 @@
  *      - parv[1] = message text
  */
 static void
-mo_globops(struct Client *source_p, int parc, char *parv[])
+mo_globops(struct Client *source, int parc, char *parv[])
 {
   const char *const message = parv[1];
 
-  if (!HasOFlag(source_p, OPER_FLAG_GLOBOPS))
+  if (!HasOFlag(source, OPER_FLAG_GLOBOPS))
   {
-    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "globops");
+    sendto_one_numeric(source, &me, ERR_NOPRIVS, "globops");
     return;
   }
 
-  sendto_server(source_p, 0, 0, ":%s GLOBOPS :%s", source_p->id, message);
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_GLOBAL, "from %s: %s",
-                       source_p->name, message);
+  sendto_servers(source, 0, 0, ":%s GLOBOPS :%s", source->id, message);
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_GLOBAL, "from %s: %s",
+                 source->name, message);
 }
 
 /*! \brief GLOBOPS command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -73,18 +71,18 @@ mo_globops(struct Client *source_p, int parc, char *parv[])
  *      - parv[1] = message text
  */
 static void
-ms_globops(struct Client *source_p, int parc, char *parv[])
+ms_globops(struct Client *source, int parc, char *parv[])
 {
   const char *const message = parv[1];
 
-  sendto_server(source_p, 0, 0, ":%s GLOBOPS :%s", source_p->id, message);
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_GLOBAL, "from %s: %s",
-                       source_p->name, message);
+  sendto_servers(source, 0, 0, ":%s GLOBOPS :%s", source->id, message);
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_GLOBAL, "from %s: %s",
+                 source->name, message);
 }
 
-static struct Message globops_msgtab =
+static struct Command command_table =
 {
-  .cmd = "GLOBOPS",
+  .name = "GLOBOPS",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_not_oper },
   .handlers[SERVER_HANDLER] = { .handler = ms_globops, .args_min = 2 },
@@ -93,19 +91,19 @@ static struct Message globops_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&globops_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&globops_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
