@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,16 +28,59 @@
 #include "client.h"
 #include "hash.h"
 #include "ircd.h"
-#include "numeric.h"
 #include "send.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 #include "channel.h"
 
 
+static void
+do_hash(struct Client *source, unsigned int type)
+{
+  unsigned int max_chain = 0;
+  unsigned int buckets   = 0;
+  unsigned int count     = 0;
+
+  assert(type == HASH_TYPE_CLIENT || type == HASH_TYPE_ID || type == HASH_TYPE_CHANNEL);
+
+  for (unsigned int i = 0; i < HASHSIZE; ++i)
+  {
+    const void *ptr = hash_get_bucket(type, i);
+    if (ptr)
+    {
+      unsigned int len = 0;
+      ++buckets;
+
+      while (ptr)
+      {
+        if (type == HASH_TYPE_CLIENT)
+          ptr = ((const struct Client *)ptr)->hnext;
+        else if (type == HASH_TYPE_ID)
+          ptr = ((const struct Client *)ptr)->idhnext;
+        else /* type == HASH_TYPE_CHANNEL */
+          ptr = ((const struct Channel *)ptr)->hnextch;
+
+        if (++len > max_chain)
+          max_chain = len;
+        count += len;
+      }
+    }
+  }
+
+  static const char *const strtype[] =
+  {
+    [HASH_TYPE_CLIENT] = "Client",
+    [HASH_TYPE_ID] = "Id",
+    [HASH_TYPE_CHANNEL] = "Channel",
+  };
+
+  sendto_one_notice(source, &me, ":%s: entries: %u buckets: %u max chain: %u",
+                    strtype[type], count, buckets, max_chain);
+}
+
 /*! \brief HASH command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -46,82 +89,16 @@
  *      - parv[0] = command
  */
 static void
-mo_hash(struct Client *source_p, int parc, char *parv[])
+mo_hash(struct Client *source, int parc, char *parv[])
 {
-  unsigned int i = 0;
-  unsigned int max_chain = 0;
-  unsigned int buckets   = 0;
-  unsigned int count     = 0;
-  const struct Client *cl = NULL;
-  const struct Client *icl = NULL;
-  const struct Channel *ch = NULL;
-
-  for (i = 0; i < HASHSIZE; ++i)
-  {
-    if ((cl = hash_get_bucket(HASH_TYPE_CLIENT, i)))
-    {
-      unsigned int len = 0;
-
-      ++buckets;
-      for (; cl; cl = cl->hnext)
-        ++len;
-      if (len > max_chain)
-        max_chain = len;
-      count += len;
-    }
-  }
-
-  sendto_one_notice(source_p, &me, ":Client: entries: %u buckets: %u "
-                    "max chain: %u", count, buckets, max_chain);
-
-  count     = 0;
-  buckets   = 0;
-  max_chain = 0;
-
-  for (i = 0; i < HASHSIZE; ++i)
-  {
-    if ((ch = hash_get_bucket(HASH_TYPE_CHANNEL, i)))
-    {
-      unsigned int len = 0;
-
-      ++buckets;
-      for (; ch; ch = ch->hnextch)
-        ++len;
-      if (len > max_chain)
-        max_chain = len;
-      count += len;
-    }
-  }
-
-  sendto_one_notice(source_p, &me, ":Channel: entries: %u buckets: %u "
-                    "max chain: %u", count, buckets, max_chain);
-
-  count     = 0;
-  buckets   = 0;
-  max_chain = 0;
-
-  for (i = 0; i < HASHSIZE; ++i)
-  {
-    if ((icl = hash_get_bucket(HASH_TYPE_ID, i)))
-    {
-      unsigned int len = 0;
-
-      ++buckets;
-      for (; icl; icl = icl->idhnext)
-        ++len;
-      if (len > max_chain)
-        max_chain = len;
-      count += len;
-    }
-  }
-
-  sendto_one_notice(source_p, &me, ":Id: entries: %u buckets: %u "
-                    "max chain: %u", count, buckets, max_chain);
+  do_hash(source, HASH_TYPE_CLIENT);
+  do_hash(source, HASH_TYPE_ID);
+  do_hash(source, HASH_TYPE_CHANNEL);
 }
 
-static struct Message hash_msgtab =
+static struct Command command_table =
 {
-  .cmd = "HASH",
+  .name = "HASH",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_not_oper },
   .handlers[SERVER_HANDLER] = { .handler = m_ignore },
@@ -130,19 +107,19 @@ static struct Message hash_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&hash_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&hash_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
