@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1999-2022 ircd-hybrid development team
+ *  Copyright (c) 1999-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,60 +24,62 @@
  */
 
 #include "stdinc.h"
+#include "io.h"
+#include "io_time.h"
+#include "defaults.h"
 #include "client.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "send.h"
 #include "conf.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 #include "irc_string.h"
 
 
 enum { HELPLEN = 400 };
 
 static void
-sendhelpfile(struct Client *source_p, const char *path, const char *topic)
+sendhelpfile(struct Client *source, const char *path, const char *topic)
 {
-  FILE *file = NULL;
-  char *p = NULL;
-  char line[HELPLEN] = "";
-
-  if ((file = fopen(path, "r")) == NULL)
+  FILE *file = fopen(path, "r");
+  if (file == NULL)
   {
-    sendto_one_numeric(source_p, &me, ERR_HELPNOTFOUND, topic);
+    sendto_one_numeric(source, &me, ERR_HELPNOTFOUND, topic);
     return;
   }
 
+  char line[HELPLEN];
   if (fgets(line, sizeof(line), file) == NULL)
   {
-    sendto_one_numeric(source_p, &me, ERR_HELPNOTFOUND, topic);
+    sendto_one_numeric(source, &me, ERR_HELPNOTFOUND, topic);
     fclose(file);
     return;
   }
 
-  if ((p = strpbrk(line, "\r\n")))
+  char *p = strpbrk(line, "\r\n");
+  if (p)
     *p = '\0';
 
-  sendto_one_numeric(source_p, &me, RPL_HELPSTART, topic, line);
+  sendto_one_numeric(source, &me, RPL_HELPSTART, topic, line);
 
   while (fgets(line, sizeof(line), file))
   {
-    if ((p = strpbrk(line, "\r\n")))
+    p = strpbrk(line, "\r\n");
+    if (p)
       *p = '\0';
 
-    sendto_one_numeric(source_p, &me, RPL_HELPTXT, topic, line);
+    sendto_one_numeric(source, &me, RPL_HELPTXT, topic, line);
   }
 
   fclose(file);
-  sendto_one_numeric(source_p, &me, RPL_ENDOFHELP, topic);
+  sendto_one_numeric(source, &me, RPL_ENDOFHELP, topic);
 }
 
 static void
-do_help(struct Client *source_p, char *topic)
+do_help(struct Client *source, char *topic)
 {
   char h_index[] = "index";
-  struct stat sb;
 
   if (EmptyString(topic))
     topic = h_index;
@@ -85,33 +87,34 @@ do_help(struct Client *source_p, char *topic)
     for (char *p = topic; *p; ++p)
       *p = ToLower(*p);
 
-  if (strpbrk(topic, "/\\"))
+  if (strchr(topic, '/'))
   {
-    sendto_one_numeric(source_p, &me, ERR_HELPNOTFOUND, topic);
+    sendto_one_numeric(source, &me, ERR_HELPNOTFOUND, topic);
     return;
   }
 
-  char path[sizeof(HPATH) + IRCD_BUFSIZE + 1];  /* +1 for / */
+  char path[IO_PATH_MAX];
   snprintf(path, sizeof(path), "%s/%s", HPATH, topic);
 
-  if (stat(path, &sb) < 0)
+  struct stat sb;
+  if (stat(path, &sb))
   {
-    sendto_one_numeric(source_p, &me, ERR_HELPNOTFOUND, topic);
+    sendto_one_numeric(source, &me, ERR_HELPNOTFOUND, topic);
     return;
   }
 
   if (!S_ISREG(sb.st_mode))
   {
-    sendto_one_numeric(source_p, &me, ERR_HELPNOTFOUND, topic);
+    sendto_one_numeric(source, &me, ERR_HELPNOTFOUND, topic);
     return;
   }
 
-  sendhelpfile(source_p, path, topic);
+  sendhelpfile(source, path, topic);
 }
 
 /*! \brief HELP command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -121,24 +124,24 @@ do_help(struct Client *source_p, char *topic)
  *      - parv[1] = help topic
  */
 static void
-m_help(struct Client *source_p, int parc, char *parv[])
+m_help(struct Client *source, int parc, char *parv[])
 {
   static uintmax_t last_used = 0;
 
-  if ((last_used + ConfigGeneral.pace_wait_simple) > event_base->time.sec_monotonic)
+  if ((last_used + ConfigGeneral.pace_wait_simple) > io_time_get(IO_TIME_MONOTONIC_SEC))
   {
-    sendto_one_numeric(source_p, &me, RPL_LOAD2HI, "HELP");
+    sendto_one_numeric(source, &me, RPL_LOAD2HI, "HELP");
     return;
   }
 
-  last_used = event_base->time.sec_monotonic;
+  last_used = io_time_get(IO_TIME_MONOTONIC_SEC);
 
-  do_help(source_p, parv[1]);
+  do_help(source, parv[1]);
 }
 
 /*! \brief HELP command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -148,14 +151,14 @@ m_help(struct Client *source_p, int parc, char *parv[])
  *      - parv[1] = help topic
  */
 static void
-mo_help(struct Client *source_p, int parc, char *parv[])
+mo_help(struct Client *source, int parc, char *parv[])
 {
-  do_help(source_p, parv[1]);
+  do_help(source, parv[1]);
 }
 
-static struct Message help_msgtab =
+static struct Command command_table =
 {
-  .cmd = "HELP",
+  .name = "HELP",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_help },
   .handlers[SERVER_HANDLER] = { .handler = m_ignore },
@@ -164,19 +167,19 @@ static struct Message help_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&help_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&help_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
