@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "list.h"
 #include "client.h"
 #include "irc_string.h"
@@ -31,23 +32,24 @@
 #include "conf.h"
 #include "conf_cluster.h"
 #include "conf_shared.h"
-#include "hostmask.h"
+#include "address.h"
 #include "numeric.h"
 #include "log.h"
 #include "misc.h"
 #include "send.h"
 #include "server_capab.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 #include "memory.h"
+#include "aline.h"
 
 
 static void
 kline_check(const struct AddressRec *arec)
 {
-  dlink_node *node, *node_next;
+  list_node_t *node, *node_next;
 
-  DLINK_FOREACH_SAFE(node, node_next, local_client_list.head)
+  LIST_FOREACH_SAFE(node, node_next, local_client_list.head)
   {
     struct Client *client = node->data;
 
@@ -66,7 +68,7 @@ kline_check(const struct AddressRec *arec)
         break;
       case HM_IPV6:
       case HM_IPV4:
-        if (address_compare(&client->ip, &arec->Mask.ipa.addr, false, false, arec->Mask.ipa.bits) == true)
+        if (address_compare(&client->addr, &arec->Mask.ipa.addr, false, false, arec->Mask.ipa.bits))
           conf_try_ban(client, CLIENT_BAN_KLINE, arec->conf->reason);
         break;
       default:
@@ -82,16 +84,17 @@ kline_check(const struct AddressRec *arec)
  * side effects - tkline as given is placed
  */
 static void
-kline_handle(struct Client *source_p, const struct aline_ctx *aline)
+kline_handle(struct Client *source, const struct aline_ctx *aline)
 {
   char buf[IRCD_BUFSIZE];
   int bits = 0;
-  struct irc_ssaddr iphost, *piphost = NULL;
+  unsigned int min_cidr = 0;
+  struct io_addr iphost, *piphost = NULL;
 
-  if (!HasFlag(source_p, FLAGS_SERVICE) && valid_wild_card(2, aline->user, aline->host) == false)
+  if (!HasFlag(source, FLAGS_SERVICE) && aline_valid_mask(2, aline->user, aline->host) == false)
   {
-    if (IsClient(source_p))
-      sendto_one_notice(source_p, &me,
+    if (IsClient(source))
+      sendto_one_notice(source, &me,
                         ":Please include at least %u non-wildcard characters with the mask",
                         ConfigGeneral.min_nonwildcard);
     return;
@@ -100,36 +103,29 @@ kline_handle(struct Client *source_p, const struct aline_ctx *aline)
   switch (parse_netmask(aline->host, &iphost, &bits))
   {
     case HM_IPV4:
-      if (!HasFlag(source_p, FLAGS_SERVICE) && (unsigned int)bits < ConfigGeneral.kline_min_cidr)
-      {
-        if (IsClient(source_p))
-          sendto_one_notice(source_p, &me, ":For safety, bitmasks less than %u require conf access.",
-                            ConfigGeneral.kline_min_cidr);
-        return;
-      }
-
+      min_cidr = ConfigGeneral.kline_min_cidr;
       piphost = &iphost;
       break;
     case HM_IPV6:
-      if (!HasFlag(source_p, FLAGS_SERVICE) && (unsigned int)bits < ConfigGeneral.kline_min_cidr6)
-      {
-        if (IsClient(source_p))
-          sendto_one_notice(source_p, &me, ":For safety, bitmasks less than %u require conf access.",
-                            ConfigGeneral.kline_min_cidr6);
-        return;
-      }
-
+      min_cidr = ConfigGeneral.kline_min_cidr6;
       piphost = &iphost;
       break;
     default:  /* HM_HOST */
       break;
   }
 
+  if (min_cidr > 0 && !HasFlag(source, FLAGS_SERVICE) && (unsigned int)bits < min_cidr)
+  {
+    if (IsClient(source))
+      sendto_one_notice(source, &me, ":For safety, bitmasks less than %u require conf access.", min_cidr);
+    return;
+  }
+
   struct MaskItem *conf;
   if ((conf = find_conf_by_address(aline->host, piphost, CONF_KLINE, aline->user, NULL, 0)))
   {
-    if (IsClient(source_p))
-      sendto_one_notice(source_p, &me, ":[%s@%s] already K-Lined by [%s@%s] - %s",
+    if (IsClient(source))
+      sendto_one_notice(source, &me, ":[%s@%s] already K-Lined by [%s@%s] - %s",
                         aline->user, aline->host, conf->user, conf->host, conf->reason);
     return;
   }
@@ -141,41 +137,36 @@ kline_handle(struct Client *source_p, const struct aline_ctx *aline)
     snprintf(buf, sizeof(buf), "%.*s (%s)", REASONLEN, aline->reason, date_iso8601(0));
 
   conf = conf_make(CONF_KLINE);
-  conf->user = xstrdup(aline->user);
-  conf->host = xstrdup(aline->host);
-  conf->setat = event_base->time.sec_real;
-  conf->reason = xstrdup(buf);
+  conf->user = io_strdup(aline->user);
+  conf->host = io_strdup(aline->host);
+  conf->setat = io_time_get(IO_TIME_REALTIME_SEC);
+  conf->reason = io_strdup(buf);
   SetConfDatabase(conf);
 
   if (aline->duration)
   {
-    conf->until = event_base->time.sec_real + aline->duration;
+    conf->until = conf->setat + aline->duration;
 
-    if (IsClient(source_p))
-      sendto_one_notice(source_p, &me, ":Added temporary %ju min. K-Line [%s@%s]",
+    if (IsClient(source))
+      sendto_one_notice(source, &me, ":Added temporary %ju min. K-Line [%s@%s]",
                         aline->duration / 60, conf->user, conf->host);
 
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "%s added temporary %ju min. K-Line for [%s@%s] [%s]",
-                         get_oper_name(source_p), aline->duration / 60,
-                         conf->user, conf->host,
-                         conf->reason);
-    ilog(LOG_TYPE_KLINE, "%s added temporary %ju min. K-Line for [%s@%s] [%s]",
-         get_oper_name(source_p), aline->duration / 60,
-         conf->user, conf->host, conf->reason);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "%s added temporary %ju min. K-Line for [%s@%s] [%s]",
+                   get_oper_name(source), aline->duration / 60, conf->user, conf->host, conf->reason);
+    log_write(LOG_TYPE_KLINE, "%s added temporary %ju min. K-Line for [%s@%s] [%s]",
+              get_oper_name(source), aline->duration / 60, conf->user, conf->host, conf->reason);
   }
   else
   {
-    if (IsClient(source_p))
-      sendto_one_notice(source_p, &me, ":Added K-Line [%s@%s]",
+    if (IsClient(source))
+      sendto_one_notice(source, &me, ":Added K-Line [%s@%s]",
                         conf->user, conf->host);
 
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "%s added K-Line for [%s@%s] [%s]",
-                         get_oper_name(source_p),
-                         conf->user, conf->host, conf->reason);
-    ilog(LOG_TYPE_KLINE, "%s added K-Line for [%s@%s] [%s]",
-         get_oper_name(source_p), conf->user, conf->host, conf->reason);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "%s added K-Line for [%s@%s] [%s]",
+                   get_oper_name(source), conf->user, conf->host, conf->reason);
+    log_write(LOG_TYPE_KLINE, "%s added K-Line for [%s@%s] [%s]",
+              get_oper_name(source), conf->user, conf->host, conf->reason);
   }
 
   kline_check(add_conf_by_address(CONF_KLINE, conf));
@@ -191,39 +182,38 @@ kline_handle(struct Client *source_p, const struct aline_ctx *aline)
  * side effects - k line is added
  */
 static void
-mo_kline(struct Client *source_p, int parc, char *parv[])
+mo_kline(struct Client *source, int parc, char *parv[])
 {
   struct aline_ctx aline = { .add = true, .simple_mask = false };
 
-  if (!HasOFlag(source_p, OPER_FLAG_KLINE))
+  if (!HasOFlag(source, OPER_FLAG_KLINE))
   {
-    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "kline");
+    sendto_one_numeric(source, &me, ERR_NOPRIVS, "kline");
     return;
   }
 
-  if (parse_aline("KLINE", source_p, parc, parv, &aline) == false)
+  if (aline_parse("KLINE", source, parc, parv, &aline) == false)
     return;
 
   if (aline.server)
   {
-    sendto_match_servs(source_p, aline.server, CAPAB_KLN, "KLINE %s %ju %s %s :%s",
-                       aline.server, aline.duration,
-                       aline.user, aline.host, aline.reason);
+    sendto_match_servs(source, aline.server, CAPAB_KLN, "KLINE %s %ju %s %s :%s",
+                       aline.server, aline.duration, aline.user, aline.host, aline.reason);
 
     /* Allow ON to apply local kline as well if it matches */
     if (match(aline.server, me.name))
       return;
   }
   else
-    cluster_distribute(source_p, "KLINE", CAPAB_KLN, CLUSTER_KLINE, "%ju %s %s :%s",
+    cluster_distribute(source, "KLINE", CAPAB_KLN, CLUSTER_KLINE, "%ju %s %s :%s",
                        aline.duration, aline.user, aline.host, aline.reason);
 
-  kline_handle(source_p, &aline);
+  kline_handle(source, &aline);
 }
 
 /*! \brief KLINE command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -237,7 +227,7 @@ mo_kline(struct Client *source_p, int parc, char *parv[])
  *      - parv[5] = reason
  */
 static void
-ms_kline(struct Client *source_p, int parc, char *parv[])
+ms_kline(struct Client *source, int parc, char *parv[])
 {
   struct aline_ctx aline =
   {
@@ -250,21 +240,20 @@ ms_kline(struct Client *source_p, int parc, char *parv[])
     .duration = strtoumax(parv[2], NULL, 10)
   };
 
-  sendto_match_servs(source_p, aline.server, CAPAB_KLN, "KLINE %s %ju %s %s :%s", aline.server,
-                     aline.duration, aline.user, aline.host, aline.reason);
+  sendto_match_servs(source, aline.server, CAPAB_KLN, "KLINE %s %ju %s %s :%s",
+                     aline.server, aline.duration, aline.user, aline.host, aline.reason);
 
   if (match(aline.server, me.name))
     return;
 
-  if (HasFlag(source_p, FLAGS_SERVICE) ||
-      shared_find(SHARED_KLINE, source_p->servptr->name,
-                  source_p->username, source_p->host))
-    kline_handle(source_p, &aline);
+  if (HasFlag(source, FLAGS_SERVICE) ||
+      shared_find(SHARED_KLINE, source->servptr->name, source->username, source->host))
+    kline_handle(source, &aline);
 }
 
-static struct Message kline_msgtab =
+static struct Command command_table =
 {
-  .cmd = "KLINE",
+  .name = "KLINE",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_not_oper },
   .handlers[SERVER_HANDLER] = { .handler = ms_kline, .args_min = 6 },
@@ -273,21 +262,21 @@ static struct Message kline_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&kline_msgtab);
+  command_add(&command_table);
   capab_add("KLN", CAPAB_KLN, true);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&kline_msgtab);
+  command_del(&command_table);
   capab_del("KLN");
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
