@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2020-2022 ircd-hybrid development team
+ *  Copyright (c) 2020-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,13 +30,13 @@
 #include "send.h"
 #include "parse.h"
 #include "memory.h"
-#include "modules.h"
+#include "module.h"
 #include "irc_string.h"
 
 
 /*! \brief METADATA command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -45,49 +45,47 @@
  *      - parv[0] = command
  */
 static void
-ms_metadata(struct Client *source_p, int parc, char *parv[])
+ms_metadata(struct Client *source, int parc, char *parv[])
 {
-  if (!HasFlag(source_p, FLAGS_SERVICE) && !IsServer(source_p))
+  if (!HasFlag(source, FLAGS_SERVICE) && !IsServer(source))
     return;
 
   if (irccmp(parv[1], "client") == 0)
   {
-    struct Client *target_p = find_person(source_p, parv[2]);
-    if (target_p == NULL)
+    struct Client *target = find_person(source, parv[2]);
+    if (target == NULL)
       return;
 
     if (irccmp(parv[3], "cipher") == 0)
     {
-      xfree(target_p->tls_cipher);
+      io_free(target->tls_cipher);
       if (!EmptyString(parv[4]))
-        target_p->tls_cipher = xstrdup(parv[4]);
+        target->tls_cipher = io_strdup(parv[4]);
       else
-        target_p->tls_cipher = NULL;
+        target->tls_cipher = NULL;
     }
 
     if (irccmp(parv[3], "certfp") == 0)
     {
-      xfree(target_p->tls_certfp);
+      io_free(target->tls_certfp);
       if (!EmptyString(parv[4]))
-        target_p->tls_certfp = xstrdup(parv[4]);
+        target->tls_certfp = io_strdup(parv[4]);
       else
-        target_p->tls_certfp = NULL;
+        target->tls_certfp = NULL;
     }
   }
 
   if (parc == 4)
-    sendto_server(source_p, 0, 0, ":%s METADATA %s %s %s",
-                  source_p->id,
-                  parv[1], parv[2], parv[3]);
+    sendto_servers(source, 0, 0, ":%s METADATA %s %s %s",
+                   source->id, parv[1], parv[2], parv[3]);
   else  /* parc == 5 */
-    sendto_server(source_p, 0, 0, ":%s METADATA %s %s %s :%s",
-                  source_p->id,
-                  parv[1], parv[2], parv[3], parv[4]);
+    sendto_servers(source, 0, 0, ":%s METADATA %s %s %s :%s",
+                   source->id, parv[1], parv[2], parv[3], parv[4]);
 }
 
-static struct Message metadata_msgtab =
+static struct Command command_table =
 {
-  .cmd = "METADATA",
+  .name = "METADATA",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_ignore },
   .handlers[SERVER_HANDLER] = { .handler = ms_metadata, .args_min = 4 },
@@ -96,19 +94,19 @@ static struct Message metadata_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&metadata_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&metadata_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
