@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "list.h"
 #include "client.h"
 #include "motd.h"
@@ -32,28 +33,26 @@
 #include "numeric.h"
 #include "server.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 #include "conf.h"
 
 
 /*! \brief Sends the "message of the day" and notifies irc-operators
  *         about the MOTD request
  *
- * \param source_p Pointer to client to report to
+ * \param source Pointer to client to report to
  */
 static void
-do_motd(struct Client *source_p)
+do_motd(struct Client *source)
 {
-  sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE,
-                       "MOTD requested by %s (%s@%s) [%s]",
-                       source_p->name, source_p->username,
-                       source_p->host, source_p->servptr->name);
-  motd_send(source_p);
+  sendto_clients(UMODE_SPY, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "MOTD requested by %s (%s@%s) [%s]",
+                 source->name, source->username, source->host, source->servptr->name);
+  motd_send(source);
 }
 
 /*! \brief MOTD command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -63,28 +62,28 @@ do_motd(struct Client *source_p)
  *      - parv[1] = nickname/servername
  */
 static void
-m_motd(struct Client *source_p, int parc, char *parv[])
+m_motd(struct Client *source, int parc, char *parv[])
 {
   static uintmax_t last_used = 0;
 
-  if ((last_used + ConfigGeneral.pace_wait) > event_base->time.sec_monotonic)
+  if ((last_used + ConfigGeneral.pace_wait) > io_time_get(IO_TIME_MONOTONIC_SEC))
   {
-    sendto_one_numeric(source_p, &me, RPL_LOAD2HI, "MOTD");
+    sendto_one_numeric(source, &me, RPL_LOAD2HI, "MOTD");
     return;
   }
 
-  last_used = event_base->time.sec_monotonic;
+  last_used = io_time_get(IO_TIME_MONOTONIC_SEC);
 
   if (ConfigServerHide.disable_remote_commands == 0)
-    if (server_hunt(source_p, ":%s MOTD :%s", 1, parv)->ret != HUNTED_ISME)
+    if (server_route_command(source, ":%s MOTD :%s", 1, parv)->result != SERVER_ROUTE_ISME)
       return;
 
-  do_motd(source_p);
+  do_motd(source);
 }
 
 /*! \brief MOTD command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -94,17 +93,17 @@ m_motd(struct Client *source_p, int parc, char *parv[])
  *      - parv[1] = nickname/servername
  */
 static void
-ms_motd(struct Client *source_p, int parc, char *parv[])
+ms_motd(struct Client *source, int parc, char *parv[])
 {
-  if (server_hunt(source_p, ":%s MOTD :%s", 1, parv)->ret != HUNTED_ISME)
+  if (server_route_command(source, ":%s MOTD :%s", 1, parv)->result != SERVER_ROUTE_ISME)
     return;
 
-  do_motd(source_p);
+  do_motd(source);
 }
 
-static struct Message motd_msgtab =
+static struct Command command_table =
 {
-  .cmd = "MOTD",
+  .name = "MOTD",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_motd },
   .handlers[SERVER_HANDLER] = { .handler = ms_motd },
@@ -113,19 +112,19 @@ static struct Message motd_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&motd_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&motd_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
