@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,12 +33,12 @@
 #include "numeric.h"
 #include "send.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 
 
 /*! \brief NAMES command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -48,7 +48,7 @@
  *      - parv[1] = channel name
  */
 static void
-m_names(struct Client *source_p, int parc, char *parv[])
+m_names(struct Client *source, int parc, char *parv[])
 {
   const char *const name = parv[1];
 
@@ -56,17 +56,17 @@ m_names(struct Client *source_p, int parc, char *parv[])
   {
     struct Channel *channel = hash_find_channel(name);
     if (channel)
-      channel_send_namereply(source_p, channel);
+      channel_send_namereply(source, channel);
     else
-      sendto_one_numeric(source_p, &me, RPL_ENDOFNAMES, name);
+      sendto_one_numeric(source, &me, RPL_ENDOFNAMES, name);
   }
   else
-    sendto_one_numeric(source_p, &me, RPL_ENDOFNAMES, "*");
+    sendto_one_numeric(source, &me, RPL_ENDOFNAMES, "*");
 }
 
-static struct Message names_msgtab =
+static struct Command command_table =
 {
-  .cmd = "NAMES",
+  .name = "NAMES",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_names },
   .handlers[SERVER_HANDLER] = { .handler = m_ignore },
@@ -75,19 +75,19 @@ static struct Message names_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&names_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&names_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
