@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,15 +28,15 @@
 #include "irc_string.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "restart.h"
+#include "ircd_exit.h"
 #include "send.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 
 
 /*! \brief RESTART command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -46,31 +46,29 @@
  *      - parv[1] = server name
  */
 static void
-mo_restart(struct Client *source_p, int parc, char *parv[])
+mo_restart(struct Client *source, int parc, char *parv[])
 {
   const char *const name = parv[1];
 
-  if (!HasOFlag(source_p, OPER_FLAG_RESTART))
+  if (!HasOFlag(source, OPER_FLAG_RESTART))
   {
-    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "restart");
+    sendto_one_numeric(source, &me, ERR_NOPRIVS, "restart");
     return;
   }
 
   if (irccmp(name, me.name))
   {
-    sendto_one_notice(source_p, &me, ":Mismatch on /restart %s", me.name);
+    sendto_one_notice(source, &me, ":Mismatch on /restart %s", me.name);
     return;
   }
 
-  char buf[IRCD_BUFSIZE];
-  snprintf(buf, sizeof(buf), "received RESTART command from %s",
-           client_get_name(source_p, HIDE_IP));
-  server_die(buf, true);
+  ircd_exit_fmt(IRCD_EXIT_RESTART, "received RESTART command from %s",
+                client_get_name(source, HIDE_IP));
 }
 
-static struct Message restart_msgtab =
+static struct Command command_table =
 {
-  .cmd = "RESTART",
+  .name = "RESTART",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_not_oper },
   .handlers[SERVER_HANDLER] = { .handler = m_ignore },
@@ -79,19 +77,19 @@ static struct Message restart_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&restart_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&restart_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
