@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2001-2022 ircd-hybrid development team
+ *  Copyright (c) 2001-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "client.h"
 #include "ircd.h"
 #include "irc_string.h"
@@ -31,17 +32,18 @@
 #include "server_capab.h"
 #include "send.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 #include "conf.h"
 #include "conf_cluster.h"
 #include "conf_resv.h"
 #include "conf_shared.h"
 #include "log.h"
+#include "aline.h"
 
 
 /* parse_resv()
  *
- * inputs	- source_p
+ * inputs	- source
  *		- thing to resv
  *		- !0 if temporary
  *		- reason
@@ -49,16 +51,15 @@
  * side effects	- parse resv, create if valid
  */
 static void
-resv_handle(struct Client *source_p, const struct aline_ctx *aline)
+resv_handle(struct Client *source, const struct aline_ctx *aline)
 {
-  if (!HasFlag(source_p, FLAGS_SERVICE))
+  if (!HasFlag(source, FLAGS_SERVICE))
   {
-    if (valid_wild_card_simple(aline->mask + !!IsChanPrefix(*aline->mask)) == false)
+    if (aline_valid_mask_simple(aline->mask + !!IsChanPrefix(*aline->mask)) == false)
     {
-      if (IsClient(source_p))
-        sendto_one_notice(source_p, &me, ":Please include at least %u non-wildcard characters with the RESV",
+      if (IsClient(source))
+        sendto_one_notice(source, &me, ":Please include at least %u non-wildcard characters with the RESV",
                           ConfigGeneral.min_nonwildcard_simple);
-
       return;
     }
   }
@@ -66,43 +67,39 @@ resv_handle(struct Client *source_p, const struct aline_ctx *aline)
   struct ResvItem *resv = resv_find(aline->mask, irccmp);
   if (resv)
   {
-    if (IsClient(source_p))
-      sendto_one_notice(source_p, &me, ":A RESV has already been placed on: %s", resv->mask);
-
+    if (IsClient(source))
+      sendto_one_notice(source, &me, ":A RESV has already been placed on: %s", resv->mask);
     return;
   }
 
   resv = resv_make(aline->mask, aline->reason, NULL);
-  resv->setat = event_base->time.sec_real;
+  resv->setat = io_time_get(IO_TIME_REALTIME_SEC);
   resv->in_database = true;
 
   if (aline->duration)
   {
-    resv->expire = event_base->time.sec_real + aline->duration;
+    resv->expire = resv->setat + aline->duration;
 
-    if (IsClient(source_p))
-      sendto_one_notice(source_p, &me, ":Added temporary %ju min. RESV [%s]",
+    if (IsClient(source))
+      sendto_one_notice(source, &me, ":Added temporary %ju min. RESV [%s]",
                         aline->duration / 60, resv->mask);
 
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "%s added temporary %ju min. RESV for [%s] [%s]",
-                         get_oper_name(source_p), aline->duration / 60,
-                         resv->mask, resv->reason);
-    ilog(LOG_TYPE_RESV, "%s added temporary %ju min. RESV for [%s] [%s]",
-         get_oper_name(source_p), aline->duration / 60, resv->mask, resv->reason);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "%s added temporary %ju min. RESV for [%s] [%s]",
+                   get_oper_name(source), aline->duration / 60, resv->mask, resv->reason);
+    log_write(LOG_TYPE_RESV, "%s added temporary %ju min. RESV for [%s] [%s]",
+              get_oper_name(source), aline->duration / 60, resv->mask, resv->reason);
   }
   else
   {
-    if (IsClient(source_p))
-      sendto_one_notice(source_p, &me, ":Added RESV [%s] [%s]",
+    if (IsClient(source))
+      sendto_one_notice(source, &me, ":Added RESV [%s] [%s]",
                         resv->mask, resv->reason);
 
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "%s added RESV for [%s] [%s]",
-                         get_oper_name(source_p), resv->mask,
-                         resv->reason);
-    ilog(LOG_TYPE_RESV, "%s added RESV for [%s] [%s]",
-         get_oper_name(source_p), resv->mask, resv->reason);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "%s added RESV for [%s] [%s]",
+                   get_oper_name(source), resv->mask, resv->reason);
+    log_write(LOG_TYPE_RESV, "%s added RESV for [%s] [%s]",
+              get_oper_name(source), resv->mask, resv->reason);
   }
 }
 
@@ -111,22 +108,22 @@ resv_handle(struct Client *source_p, const struct aline_ctx *aline)
  *   parv[1] = channel/nick to forbid
  */
 static void
-mo_resv(struct Client *source_p, int parc, char *parv[])
+mo_resv(struct Client *source, int parc, char *parv[])
 {
   struct aline_ctx aline = { .add = true, .simple_mask = true };
 
-  if (!HasOFlag(source_p, OPER_FLAG_RESV))
+  if (!HasOFlag(source, OPER_FLAG_RESV))
   {
-    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "resv");
+    sendto_one_numeric(source, &me, ERR_NOPRIVS, "resv");
     return;
   }
 
-  if (parse_aline("RESV", source_p, parc, parv, &aline) == false)
+  if (aline_parse("RESV", source, parc, parv, &aline) == false)
     return;
 
   if (aline.server)
   {
-    sendto_match_servs(source_p, aline.server, CAPAB_CLUSTER, "RESV %s %ju %s :%s",
+    sendto_match_servs(source, aline.server, CAPAB_CLUSTER, "RESV %s %ju %s :%s",
                        aline.server, aline.duration, aline.mask, aline.reason);
 
     /* Allow ON to apply local resv as well if it matches */
@@ -134,15 +131,15 @@ mo_resv(struct Client *source_p, int parc, char *parv[])
       return;
   }
   else
-    cluster_distribute(source_p, "RESV", CAPAB_KLN, CLUSTER_RESV, "%ju %s :%s",
+    cluster_distribute(source, "RESV", CAPAB_KLN, CLUSTER_RESV, "%ju %s :%s",
                        aline.duration, aline.mask, aline.reason);
 
-  resv_handle(source_p, &aline);
+  resv_handle(source, &aline);
 }
 
 /*! \brief RESV command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -155,7 +152,7 @@ mo_resv(struct Client *source_p, int parc, char *parv[])
  *      - parv[4] = reason
  */
 static void
-ms_resv(struct Client *source_p, int parc, char *parv[])
+ms_resv(struct Client *source, int parc, char *parv[])
 {
   struct aline_ctx aline =
   {
@@ -167,21 +164,20 @@ ms_resv(struct Client *source_p, int parc, char *parv[])
     .duration = strtoumax(parv[2], NULL, 10)
   };
 
-  sendto_match_servs(source_p, aline.server, CAPAB_CLUSTER, "RESV %s %ju %s :%s",
+  sendto_match_servs(source, aline.server, CAPAB_CLUSTER, "RESV %s %ju %s :%s",
                      aline.server, aline.duration, aline.mask, aline.reason);
 
   if (match(aline.server, me.name))
     return;
 
-  if (HasFlag(source_p, FLAGS_SERVICE) ||
-      shared_find(SHARED_RESV, source_p->servptr->name,
-                  source_p->username, source_p->host))
-    resv_handle(source_p, &aline);
+  if (HasFlag(source, FLAGS_SERVICE) ||
+      shared_find(SHARED_RESV, source->servptr->name, source->username, source->host))
+    resv_handle(source, &aline);
 }
 
-static struct Message resv_msgtab =
+static struct Command command_table =
 {
-  .cmd = "RESV",
+  .name = "RESV",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_not_oper },
   .handlers[SERVER_HANDLER] = { .handler = ms_resv, .args_min = 5 },
@@ -190,19 +186,19 @@ static struct Message resv_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&resv_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&resv_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
