@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2016-2022 ircd-hybrid development team
+ *  Copyright (c) 2016-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,16 +27,14 @@
 #include "client.h"
 #include "ircd.h"
 #include "send.h"
-#include "channel.h"
-#include "channel_mode.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 #include "user.h"
 
 
 /*! \brief SVSHOST command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -48,31 +46,30 @@
  *      - parv[3] = host name
  */
 static void
-ms_svshost(struct Client *source_p, int parc, char *parv[])
+ms_svshost(struct Client *source, int parc, char *parv[])
 {
-  if (!HasFlag(source_p, FLAGS_SERVICE))
+  if (!HasFlag(source, FLAGS_SERVICE) && !IsServer(source))
     return;
 
-  struct Client *target_p = find_person(source_p, parv[1]);
-  if (target_p == NULL)
+  struct Client *target = find_person(source, parv[1]);
+  if (target == NULL)
     return;
 
   uintmax_t ts = strtoumax(parv[2], NULL, 10);
-  if (ts && (ts != target_p->tsinfo))
+  if (ts && (ts != target->tsinfo))
     return;
 
   if (valid_hostname(parv[3]) == false)
     return;
 
-  user_set_hostmask(target_p, parv[3]);
-  sendto_server(source_p, 0, 0, ":%s SVSHOST %s %ju %s",
-                source_p->id,
-                target_p->id, target_p->tsinfo, parv[3]);
+  user_set_hostmask(target, parv[3], false);
+  sendto_servers(source, 0, 0, ":%s SVSHOST %s %ju %s",
+                 source->id, target->id, target->tsinfo, parv[3]);
 }
 
-static struct Message svshost_msgtab =
+static struct Command command_table =
 {
-  .cmd = "SVSHOST",
+  .name = "SVSHOST",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_ignore },
   .handlers[SERVER_HANDLER] = { .handler = ms_svshost, .args_min = 4 },
@@ -81,19 +78,19 @@ static struct Message svshost_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&svshost_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&svshost_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
