@@ -2,7 +2,7 @@
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
  *  Copyright (c) 1999 Bahamut development team.
- *  Copyright (c) 2013-2022 ircd-hybrid development team
+ *  Copyright (c) 2013-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,14 +29,13 @@
 #include "ircd.h"
 #include "send.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 #include "irc_string.h"
-#include "conf.h"
 
 
 /*! \brief SVSKILL command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -48,71 +47,62 @@
  *      - parv[3] = kill message
  */
 static void
-ms_svskill(struct Client *source_p, int parc, char *parv[])
+ms_svskill(struct Client *source, int parc, char *parv[])
 {
-  const char *comment = parv[3];  /* Either defined or NULL */
-
-  if (!HasFlag(source_p, FLAGS_SERVICE))
+  if (!HasFlag(source, FLAGS_SERVICE) && !IsServer(source))
     return;
 
-  if (EmptyString(comment))
-    comment = CONF_NOREASON;
-
-  struct Client *target_p = find_person(source_p, parv[1]);
-  if (target_p == NULL)
+  struct Client *target = find_person(source, parv[1]);
+  if (target == NULL)
     return;
 
   uintmax_t ts = strtoumax(parv[2], NULL, 10);
-  if (ts && (ts != target_p->tsinfo))
+  if (ts && (ts != target->tsinfo))
     return;
 
-  if (MyConnect(target_p))
+  const char *const comment = parv[3];
+  if (MyConnect(target))
   {
-    char reason[REASONLEN + 1] = "SVSKilled: ";
-    strlcpy(reason + 11, comment, sizeof(reason) - 11);
-
-    exit_client(target_p, reason);
+    client_exit_fmt(target, "SVSKilled: %.*s", REASONLEN, comment);
     return;
   }
 
-  if (target_p->from == source_p->from)
+  if (target->from == source->from)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "Received wrong-direction SVSKILL "
-                         "for %s (behind %s) from %s",
-                         target_p->name, source_p->from->name,
-                         client_get_name(source_p, HIDE_IP));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "Received wrong-direction SVSKILL for %s (behind %s) from %s",
+                   target->name, source->from->name, client_get_name(source, HIDE_IP));
     return;
   }
 
-  sendto_one(target_p, ":%s SVSKILL %s %ju :%s", source_p->id,
-             target_p->id, ts, comment);
+  sendto_one(target, ":%s SVSKILL %s %ju :%s",
+             source->id, target->id, ts, comment);
 }
 
-static struct Message svskill_msgtab =
+static struct Command command_table =
 {
-  .cmd = "SVSKILL",
+  .name = "SVSKILL",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_ignore },
-  .handlers[SERVER_HANDLER] = { .handler = ms_svskill, .args_min = 3 },
+  .handlers[SERVER_HANDLER] = { .handler = ms_svskill, .args_min = 4 },
   .handlers[ENCAP_HANDLER] = { .handler = m_ignore },
   .handlers[OPER_HANDLER] = { .handler = m_ignore }
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&svskill_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&svskill_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
