@@ -2,7 +2,7 @@
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
  *  Copyright (c) 1999 Bahamut development team.
- *  Copyright (c) 2011-2022 ircd-hybrid development team
+ *  Copyright (c) 2011-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,15 +33,14 @@
 #include "channel.h"
 #include "channel_mode.h"
 #include "parse.h"
-#include "modules.h"
-#include "irc_string.h"
-#include "user.h"
+#include "module.h"
+#include "user_mode.h"
 #include "conf.h"
 
 
 /*! \brief SVSMODE command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -53,105 +52,49 @@
  *      - parv[3] = modes to be added or removed
  */
 static void
-ms_svsmode(struct Client *source_p, int parc, char *parv[])
+ms_svsmode(struct Client *source, int parc, char *parv[])
 {
-  const char *const modes = parv[3];
-  const struct user_modes *tab = NULL;
-  int what = MODE_ADD;
-
-  if (!HasFlag(source_p, FLAGS_SERVICE))
+  if (!HasFlag(source, FLAGS_SERVICE) && !IsServer(source))
     return;
 
-  struct Client *target_p = find_person(source_p, parv[1]);
-  if (target_p == NULL)
+  struct Client *target = find_person(source, parv[1]);
+  if (target == NULL)
     return;
 
   uintmax_t ts = strtoumax(parv[2], NULL, 10);
-  if (ts && (ts != target_p->tsinfo))
+  if (ts && (ts != target->tsinfo))
     return;
 
-  const unsigned int setmodes = target_p->umodes;
+  const uint64_t mode_flags_old = target->umodes;
+  user_mode_action_t action = USER_MODE_ACTION_ADD;
 
+  const char *const modes = parv[3];
   for (const char *m = modes; *m; ++m)
   {
     switch (*m)
     {
       case '+':
-        what = MODE_ADD;
+        action = USER_MODE_ACTION_ADD;
         break;
       case '-':
-        what = MODE_DEL;
+        action = USER_MODE_ACTION_DEL;
         break;
-
-      case 'o':
-        if (what == MODE_DEL && HasUMode(target_p, UMODE_OPER))
-        {
-          ClearOper(target_p);
-          --Count.oper;
-
-          if (MyConnect(target_p))
-          {
-            svstag_detach(&target_p->svstags, RPL_WHOISOPERATOR);
-            conf_detach(target_p, CONF_OPER);
-
-            ClrOFlag(target_p);
-            DelUMode(target_p, ConfigGeneral.oper_only_umodes);
-
-            dlink_node *node = dlinkFindDelete(&oper_list, target_p);
-            if (node)
-              free_dlink_node(node);
-          }
-        }
-
-        break;
-
-      case 'i':
-        if (what == MODE_ADD && !HasUMode(target_p, UMODE_INVISIBLE))
-        {
-          AddUMode(target_p, UMODE_INVISIBLE);
-          ++Count.invisi;
-        }
-        else if (what == MODE_DEL && HasUMode(target_p, UMODE_INVISIBLE))
-        {
-          DelUMode(target_p, UMODE_INVISIBLE);
-          --Count.invisi;
-        }
-
-        break;
-
-      case 'z':
-      case 'S':  /* Only servers may set +S in a burst */
-      case 'W':  /* Only servers may set +W in a burst */
-        break;
-
       default:
-        if ((tab = umode_map[(unsigned char)*m]))
-        {
-          if (what == MODE_ADD)
-            AddUMode(target_p, tab->flag);
-          else
-            DelUMode(target_p, tab->flag);
-        }
-
+        user_mode_change(target, *m, USER_MODE_SOURCE_SVSMODE, action);
         break;
     }
   }
 
-  sendto_server(source_p, 0, 0, ":%s SVSMODE %s %ju %s",
-                source_p->id,
-                target_p->id, target_p->tsinfo, modes);
+  sendto_servers(source, 0, 0, ":%s SVSMODE %s %ju %s",
+                 source->id, target->id, target->tsinfo, modes);
 
-  if (MyConnect(target_p) && (setmodes != target_p->umodes))
-  {
-    char buf[UMODE_MAX_STR] = "";
-
-    send_umode(target_p, true, setmodes, buf);
-  }
+  if (MyConnect(target))
+    user_mode_send(target, mode_flags_old, USER_MODE_SEND_CLIENT);
 }
 
-static struct Message svsmode_msgtab =
+static struct Command command_table =
 {
-  .cmd = "SVSMODE",
+  .name = "SVSMODE",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_ignore },
   .handlers[SERVER_HANDLER] = { .handler = ms_svsmode, .args_min = 4 },
@@ -160,19 +103,19 @@ static struct Message svsmode_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&svsmode_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&svsmode_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
