@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2014-2022 ircd-hybrid development team
+ *  Copyright (c) 2014-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,17 +27,16 @@
 #include "list.h"
 #include "channel.h"
 #include "client.h"
-#include "hash.h"
 #include "irc_string.h"
 #include "ircd.h"
 #include "send.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 
 
 /*! \brief SVSPART command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -49,42 +48,40 @@
  *      - parv[3] = part message
  */
 static void
-ms_svspart(struct Client *source_p, int parc, char *parv[])
+ms_svspart(struct Client *source, int parc, char *parv[])
 {
-  if (!HasFlag(source_p, FLAGS_SERVICE))
+  if (!HasFlag(source, FLAGS_SERVICE) && !IsServer(source))
     return;
 
-  struct Client *target_p = find_person(source_p, parv[1]);
-  if (target_p == NULL)
+  struct Client *target = find_person(source, parv[1]);
+  if (target == NULL)
     return;
 
-  if (MyConnect(target_p))
+  if (MyConnect(target))
   {
-    channel_do_part(target_p, parv[2], parv[3]);
+    channel_part_list(target, parv[2], parv[3]);
     return;
   }
 
-  if (target_p->from == source_p->from)
+  if (target->from == source->from)
   {
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "Received wrong-direction SVSPART "
-                         "for %s (behind %s) from %s",
-                         target_p->name, source_p->from->name,
-                         client_get_name(source_p, HIDE_IP));
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "Received wrong-direction SVSPART for %s (behind %s) from %s",
+                   target->name, source->from->name, client_get_name(source, HIDE_IP));
     return;
   }
 
   if (EmptyString(parv[3]))
-    sendto_one(target_p, ":%s SVSPART %s %s", source_p->id,
-               target_p->id, parv[2]);
+    sendto_one(target, ":%s SVSPART %s %s",
+               source->id, target->id, parv[2]);
   else
-    sendto_one(target_p, ":%s SVSPART %s %s :%s", source_p->id,
-               target_p->id, parv[2], parv[3]);
+    sendto_one(target, ":%s SVSPART %s %s :%s",
+               source->id, target->id, parv[2], parv[3]);
 }
 
-static struct Message svspart_msgtab =
+static struct Command command_table =
 {
-  .cmd = "SVSPART",
+  .name = "SVSPART",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_ignore },
   .handlers[SERVER_HANDLER] = { .handler = ms_svspart, .args_min = 3 },
@@ -93,19 +90,19 @@ static struct Message svspart_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&svspart_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&svspart_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
