@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2002-2022 ircd-hybrid development team
+ *  Copyright (c) 2002-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 #include "client.h"
 #include "ircd.h"
 #include "send.h"
-#include "modules.h"
+#include "module.h"
 #include "hash.h"
 #include "server_capab.h"
 #include "conf.h"
@@ -37,7 +37,7 @@
 
 /*! \brief TBURST command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -51,7 +51,7 @@
  *      - parv[5] = topic (can be an empty string)
  */
 static void
-ms_tburst(struct Client *source_p, int parc, char *parv[])
+ms_tburst(struct Client *source, int parc, char *parv[])
 {
   uintmax_t remote_channel_ts = strtoumax(parv[1], NULL, 10);
   uintmax_t remote_topic_ts = strtoumax(parv[3], NULL, 10);
@@ -84,7 +84,7 @@ ms_tburst(struct Client *source_p, int parc, char *parv[])
    *        the TS of the remote topic is newer than ours
    */
   bool accept_remote = false;
-  if (HasFlag(source_p, FLAGS_SERVICE))
+  if (HasFlag(source, FLAGS_SERVICE))
     accept_remote = true;
   else if (remote_channel_ts < channel->creation_time)
     accept_remote = true;
@@ -92,34 +92,32 @@ ms_tburst(struct Client *source_p, int parc, char *parv[])
     if (remote_topic_ts > channel->topic_time)
       accept_remote = true;
 
-  if (accept_remote == true)
+  if (accept_remote)
   {
     bool topic_differs = strncmp(channel->topic, topic, sizeof(channel->topic) - 1);
-    bool hidden_server = (ConfigServerHide.hide_servers || IsHidden(source_p));
+    bool hidden_server = (ConfigServerHide.hide_servers || IsHidden(source));
 
     channel_set_topic(channel, topic, setby, remote_topic_ts, false);
 
-    sendto_server(source_p, CAPAB_TBURST, 0, ":%s TBURST %s %s %s %s :%s",
-                  source_p->id, parv[1], parv[2], parv[3], setby, topic);
+    sendto_servers(source, CAPAB_TBURST, 0, ":%s TBURST %s %s %s %s :%s",
+                   source->id, parv[1], parv[2], parv[3], setby, topic);
 
     /* If it's a new topic, send it to clients, otherwise drop it to save bandwith. */
-    if (topic_differs == true)
+    if (topic_differs)
     {
-      if (IsClient(source_p))
+      if (IsClient(source))
         sendto_channel_local(NULL, channel, 0, 0, 0, ":%s!%s@%s TOPIC %s :%s",
-                             source_p->name, source_p->username, source_p->host,
-                             channel->name, channel->topic);
+                             source->name, source->username, source->host, channel->name, channel->topic);
       else
         sendto_channel_local(NULL, channel, 0, 0, 0, ":%s TOPIC %s :%s",
-                             hidden_server ? me.name : source_p->name,
-                             channel->name, channel->topic);
+                             hidden_server ? me.name : source->name, channel->name, channel->topic);
     }
   }
 }
 
-static struct Message tburst_msgtab =
+static struct Command command_table =
 {
-  .cmd = "TBURST",
+  .name = "TBURST",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_ignore },
   .handlers[SERVER_HANDLER] = { .handler = ms_tburst, .args_min = 6, .empty_last_arg = true },
@@ -128,21 +126,21 @@ static struct Message tburst_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&tburst_msgtab);
+  command_add(&command_table);
   capab_add("TBURST", CAPAB_TBURST, true);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&tburst_msgtab);
+  command_del(&command_table);
   capab_del("TBURST");
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
