@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,13 +32,12 @@
 #include "server.h"
 #include "send.h"
 #include "parse.h"
-#include "modules.h"
-#include "packet.h"
+#include "module.h"
 
 
 /*! \brief TIME command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -48,19 +47,19 @@
  *      - parv[1] = nickname/servername
  */
 static void
-m_time(struct Client *source_p, int parc, char *parv[])
+m_time(struct Client *source, int parc, char *parv[])
 {
   /* This is safe enough to use during non hidden server mode */
   if (ConfigServerHide.disable_remote_commands == 0)
-    if (server_hunt(source_p, ":%s TIME :%s", 1, parv)->ret != HUNTED_ISME)
+    if (server_route_command(source, ":%s TIME :%s", 1, parv)->result != SERVER_ROUTE_ISME)
       return;
 
-  sendto_one_numeric(source_p, &me, RPL_TIME, me.name, date(0));
+  sendto_one_numeric(source, &me, RPL_TIME, me.name, date(0));
 }
 
 /*! \brief TIME command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -70,17 +69,17 @@ m_time(struct Client *source_p, int parc, char *parv[])
  *      - parv[1] = nickname/servername
  */
 static void
-ms_time(struct Client *source_p, int parc, char *parv[])
+ms_time(struct Client *source, int parc, char *parv[])
 {
-  if (server_hunt(source_p, ":%s TIME :%s", 1, parv)->ret != HUNTED_ISME)
+  if (server_route_command(source, ":%s TIME :%s", 1, parv)->result != SERVER_ROUTE_ISME)
     return;
 
-  sendto_one_numeric(source_p, &me, RPL_TIME, me.name, date(0));
+  sendto_one_numeric(source, &me, RPL_TIME, me.name, date(0));
 }
 
-static struct Message time_msgtab =
+static struct Command command_table =
 {
-  .cmd = "TIME",
+  .name = "TIME",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_time },
   .handlers[SERVER_HANDLER] = { .handler = ms_time },
@@ -89,19 +88,19 @@ static struct Message time_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&time_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&time_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
