@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2003-2022 ircd-hybrid development team
+ *  Copyright (c) 2003-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "list.h"
 #include "client.h"
 #include "irc_string.h"
@@ -38,16 +39,17 @@
 #include "send.h"
 #include "server_capab.h"
 #include "parse.h"
-#include "modules.h"
+#include "module.h"
 #include "memory.h"
+#include "aline.h"
 
 
 static void
 xline_check(const struct GecosItem *gecos)
 {
-  dlink_node *node, *node_next;
+  list_node_t *node, *node_next;
 
-  DLINK_FOREACH_SAFE(node, node_next, local_client_list.head)
+  LIST_FOREACH_SAFE(node, node_next, local_client_list.head)
   {
     struct Client *client = node->data;
 
@@ -66,16 +68,16 @@ xline_check(const struct GecosItem *gecos)
  * side effects - when successful, adds an xline to the conf
  */
 static void
-xline_handle(struct Client *source_p, const struct aline_ctx *aline)
+xline_handle(struct Client *source, const struct aline_ctx *aline)
 {
   char buf[IRCD_BUFSIZE];
 
-  if (!HasFlag(source_p, FLAGS_SERVICE))
+  if (!HasFlag(source, FLAGS_SERVICE))
   {
-    if (valid_wild_card_simple(aline->mask) == false)
+    if (aline_valid_mask_simple(aline->mask) == false)
     {
-      if (IsClient(source_p))
-        sendto_one_notice(source_p, &me, ":Please include at least %u non-wildcard characters with the xline",
+      if (IsClient(source))
+        sendto_one_notice(source, &me, ":Please include at least %u non-wildcard characters with the xline",
                           ConfigGeneral.min_nonwildcard_simple);
       return;
     }
@@ -84,8 +86,8 @@ xline_handle(struct Client *source_p, const struct aline_ctx *aline)
   struct GecosItem *gecos = gecos_find(aline->mask, match);
   if (gecos)
   {
-    if (IsClient(source_p))
-      sendto_one_notice(source_p, &me, ":[%s] already X-Lined by [%s] - %s",
+    if (IsClient(source))
+      sendto_one_notice(source, &me, ":[%s] already X-Lined by [%s] - %s",
                         aline->mask, gecos->mask, gecos->reason);
     return;
   }
@@ -97,38 +99,35 @@ xline_handle(struct Client *source_p, const struct aline_ctx *aline)
     snprintf(buf, sizeof(buf), "%.*s (%s)", REASONLEN, aline->reason, date_iso8601(0));
 
   gecos = gecos_make();
-  gecos->mask = xstrdup(aline->mask);
-  gecos->reason = xstrdup(buf);
-  gecos->setat = event_base->time.sec_real;
+  gecos->mask = io_strdup(aline->mask);
+  gecos->reason = io_strdup(buf);
+  gecos->setat = io_time_get(IO_TIME_REALTIME_SEC);
   gecos->in_database = true;
 
   if (aline->duration)
   {
-    gecos->expire = event_base->time.sec_real + aline->duration;
+    gecos->expire = gecos->setat + aline->duration;
 
-    if (IsClient(source_p))
-      sendto_one_notice(source_p, &me, ":Added temporary %ju min. X-Line [%s]",
+    if (IsClient(source))
+      sendto_one_notice(source, &me, ":Added temporary %ju min. X-Line [%s]",
                         aline->duration / 60, gecos->mask);
 
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "%s added temporary %ju min. X-Line for [%s] [%s]",
-                         get_oper_name(source_p), aline->duration / 60,
-                         gecos->mask, gecos->reason);
-    ilog(LOG_TYPE_XLINE, "%s added temporary %ju min. X-Line for [%s] [%s]",
-         get_oper_name(source_p), aline->duration / 60, gecos->mask, gecos->reason);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                   "%s added temporary %ju min. X-Line for [%s] [%s]",
+                   get_oper_name(source), aline->duration / 60, gecos->mask, gecos->reason);
+    log_write(LOG_TYPE_XLINE, "%s added temporary %ju min. X-Line for [%s] [%s]",
+              get_oper_name(source), aline->duration / 60, gecos->mask, gecos->reason);
   }
   else
   {
-    if (IsClient(source_p))
-      sendto_one_notice(source_p, &me, ":Added X-Line [%s] [%s]",
+    if (IsClient(source))
+      sendto_one_notice(source, &me, ":Added X-Line [%s] [%s]",
                         gecos->mask, gecos->reason);
 
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                         "%s added X-Line for [%s] [%s]",
-                         get_oper_name(source_p), gecos->mask,
-                         gecos->reason);
-    ilog(LOG_TYPE_XLINE, "%s added X-Line for [%s] [%s]",
-         get_oper_name(source_p), gecos->mask, gecos->reason);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "%s added X-Line for [%s] [%s]",
+                   get_oper_name(source), gecos->mask, gecos->reason);
+    log_write(LOG_TYPE_XLINE, "%s added X-Line for [%s] [%s]",
+              get_oper_name(source), gecos->mask, gecos->reason);
   }
 
   xline_check(gecos);
@@ -145,22 +144,22 @@ xline_handle(struct Client *source_p, const struct aline_ctx *aline)
  *
  */
 static void
-mo_xline(struct Client *source_p, int parc, char *parv[])
+mo_xline(struct Client *source, int parc, char *parv[])
 {
   struct aline_ctx aline = { .add = true, .simple_mask = true };
 
-  if (!HasOFlag(source_p, OPER_FLAG_XLINE))
+  if (!HasOFlag(source, OPER_FLAG_XLINE))
   {
-    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "xline");
+    sendto_one_numeric(source, &me, ERR_NOPRIVS, "xline");
     return;
   }
 
-  if (parse_aline("XLINE", source_p, parc, parv, &aline) == false)
+  if (aline_parse("XLINE", source, parc, parv, &aline) == false)
     return;
 
   if (aline.server)
   {
-    sendto_match_servs(source_p, aline.server, CAPAB_CLUSTER, "XLINE %s %s %ju :%s",
+    sendto_match_servs(source, aline.server, CAPAB_CLUSTER, "XLINE %s %s %ju :%s",
                        aline.server, aline.mask, aline.duration, aline.reason);
 
     /* Allow ON to apply local xline as well if it matches */
@@ -168,15 +167,15 @@ mo_xline(struct Client *source_p, int parc, char *parv[])
       return;
   }
   else
-    cluster_distribute(source_p, "XLINE", CAPAB_CLUSTER, CLUSTER_XLINE, "%s %ju :%s",
-                       aline.mask, aline.mask, aline.reason);
+    cluster_distribute(source, "XLINE", CAPAB_CLUSTER, CLUSTER_XLINE, "%s %ju :%s",
+                       aline.mask, aline.duration, aline.reason);
 
-  xline_handle(source_p, &aline);
+  xline_handle(source, &aline);
 }
 
 /*! \brief XLINE command handler
  *
- * \param source_p Pointer to allocated Client struct from which the message
+ * \param source Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
@@ -189,7 +188,7 @@ mo_xline(struct Client *source_p, int parc, char *parv[])
  *      - parv[4] = reason
  */
 static void
-ms_xline(struct Client *source_p, int parc, char *parv[])
+ms_xline(struct Client *source, int parc, char *parv[])
 {
   struct aline_ctx aline =
   {
@@ -201,21 +200,20 @@ ms_xline(struct Client *source_p, int parc, char *parv[])
     .duration = strtoumax(parv[3], NULL, 10)
   };
 
-  sendto_match_servs(source_p, aline.server, CAPAB_CLUSTER, "XLINE %s %s %ju :%s",
+  sendto_match_servs(source, aline.server, CAPAB_CLUSTER, "XLINE %s %s %ju :%s",
                      aline.server, aline.mask, aline.duration, aline.reason);
 
   if (match(aline.server, me.name))
     return;
 
-  if (HasFlag(source_p, FLAGS_SERVICE) ||
-      shared_find(SHARED_XLINE, source_p->servptr->name,
-                  source_p->username, source_p->host))
-    xline_handle(source_p, &aline);
+  if (HasFlag(source, FLAGS_SERVICE) ||
+      shared_find(SHARED_XLINE, source->servptr->name, source->username, source->host))
+    xline_handle(source, &aline);
 }
 
-static struct Message xline_msgtab =
+static struct Command command_table =
 {
-  .cmd = "XLINE",
+  .name = "XLINE",
   .handlers[UNREGISTERED_HANDLER] = { .handler = m_unregistered },
   .handlers[CLIENT_HANDLER] = { .handler = m_not_oper },
   .handlers[SERVER_HANDLER] = { .handler = ms_xline, .args_min = 5 },
@@ -224,19 +222,19 @@ static struct Message xline_msgtab =
 };
 
 static void
-module_init(void)
+init_handler(void)
 {
-  mod_add_cmd(&xline_msgtab);
+  command_add(&command_table);
 }
 
 static void
-module_exit(void)
+exit_handler(void)
 {
-  mod_del_cmd(&xline_msgtab);
+  command_del(&command_table);
 }
 
-struct module module_entry =
+struct Module module_entry =
 {
-  .modinit = module_init,
-  .modexit = module_exit,
+  .init_handler = init_handler,
+  .exit_handler = exit_handler,
 };
