@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,35 +24,36 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
+#include "defaults.h"
 #include "list.h"
 #include "channel.h"
 #include "channel_invite.h"
 #include "channel_mode.h"
 #include "client.h"
 #include "hash.h"
+#include "cap.h"
 #include "conf.h"
 #include "conf_resv.h"
-#include "hostmask.h"
+#include "address.h"
 #include "irc_string.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "server.h"
 #include "server_capab.h"
 #include "send.h"
-#include "event.h"
+#include "user_mode.h"
 #include "memory.h"
 #include "misc.h"
 #include "extban.h"
 
-
 /** Doubly linked list containing a list of all channels. */
-static dlink_list channel_list;
-
+static list_t channel_list;
 
 /*! \brief Returns the channel_list as constant
  * \return channel_list
  */
-const dlink_list *
+const list_t *
 channel_get_list(void)
 {
   return &channel_list;
@@ -66,53 +67,49 @@ channel_get_list(void)
  * \param flood_ctrl Whether to count this join in flood calculations
  */
 void
-channel_add_user(struct Channel *channel, struct Client *client,
-                 unsigned int flags, bool flood_ctrl)
+channel_add_user(struct Channel *channel, struct Client *client, unsigned int flags, bool flood_ctrl)
 {
   assert(IsClient(client));
 
   if (GlobalSetOptions.joinfloodtime)
   {
-    if (flood_ctrl == true)
+    if (flood_ctrl)
       ++channel->number_joined;
 
-    channel->number_joined -= (event_base->time.sec_monotonic - channel->last_join_time) *
+    channel->number_joined -= (io_time_get(IO_TIME_MONOTONIC_SEC) - channel->last_join_time) *
       (((float)GlobalSetOptions.joinfloodcount) /
        (float)GlobalSetOptions.joinfloodtime);
 
     if (channel->number_joined <= 0)
     {
       channel->number_joined = 0;
-      ClearJoinFloodNoticed(channel);
+      channel->sent_join_flood_notice = false;
     }
     else if (channel->number_joined >= GlobalSetOptions.joinfloodcount)
     {
       channel->number_joined = GlobalSetOptions.joinfloodcount;
 
-      if (!IsSetJoinFloodNoticed(channel))
+      if (channel->sent_join_flood_notice == false)
       {
-        SetJoinFloodNoticed(channel);
-        sendto_realops_flags(UMODE_FLOOD, L_ALL, SEND_NOTICE,
-                             "Possible Join Flooder %s on %s target: %s",
-                             client_get_name(client, HIDE_IP),
-                             client->servptr->name, channel->name);
+        channel->sent_join_flood_notice = true;
+        sendto_clients(UMODE_FLOOD, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Possible Join Flooder %s on %s target: %s",
+                       client_get_name(client, HIDE_IP), client->servptr->name, channel->name);
       }
     }
 
-    channel->last_join_time = event_base->time.sec_monotonic;
+    channel->last_join_time = io_time_get(IO_TIME_MONOTONIC_SEC);
   }
 
-  struct ChannelMember *member = xcalloc(sizeof(*member));
+  struct ChannelMember *member = io_calloc(sizeof(*member));
   member->client = client;
   member->channel = channel;
   member->flags = flags;
-
-  dlinkAdd(member, &member->channode, &channel->members);
+  list_add(member, &member->channode, &channel->members);
 
   if (MyConnect(client))
-    dlinkAdd(member, &member->locchannode, &channel->members_local);
+    list_add(member, &member->locchannode, &channel->members_local);
 
-  dlinkAdd(member, &member->usernode, &client->channel);
+  list_add(member, &member->usernode, &client->channel);
 }
 
 /*! \brief Deletes an user from a channel by removing a link in the
@@ -125,16 +122,16 @@ channel_remove_user(struct ChannelMember *member)
   struct Client *const client = member->client;
   struct Channel *const channel = member->channel;
 
-  dlinkDelete(&member->channode, &channel->members);
+  list_remove(&member->channode, &channel->members);
 
   if (MyConnect(client))
-    dlinkDelete(&member->locchannode, &channel->members_local);
+    list_remove(&member->locchannode, &channel->members_local);
 
-  dlinkDelete(&member->usernode, &client->channel);
+  list_remove(&member->usernode, &client->channel);
 
-  xfree(member);
+  io_free(member);
 
-  if (channel->members.head == NULL)
+  if (list_is_empty(&channel->members))
     channel_free(channel);
 }
 
@@ -147,24 +144,25 @@ channel_remove_user(struct ChannelMember *member)
 void
 channel_demote_members(struct Channel *channel, const struct Client *client)
 {
-  dlink_node *node;
   char modebuf[MAXMODEPARAMS + 1];
   char parabuf[MAXMODEPARAMS * (NICKLEN + 1) + 1];
   char *mbuf = modebuf;
   char *pbuf = parabuf;
   unsigned int pargs = 0;
 
-  DLINK_FOREACH(node, channel->members.head)
+  list_node_t *node;
+  LIST_FOREACH(node, channel->members.head)
   {
     struct ChannelMember *member = node->data;
 
     for (const struct chan_mode *tab = cflag_tab; tab->letter; ++tab)
     {
-      if (member_has_flags(member, tab->flag) == true)
+      if (member_has_flags(member, tab->flag))
       {
         member->flags &= ~tab->flag;
         *mbuf++ = tab->letter;
-        pbuf += snprintf(pbuf, sizeof(parabuf) - (pbuf - parabuf), pbuf != parabuf ? " %s" : "%s", member->client->name);
+        pbuf += snprintf(pbuf, sizeof(parabuf) - (pbuf - parabuf), pbuf != parabuf ? " %s" : "%s",
+                         member->client->name);
 
         if (++pargs >= MAXMODEPARAMS)
         {
@@ -197,7 +195,6 @@ channel_demote_members(struct Channel *channel, const struct Client *client)
 static void
 channel_send_members(struct Client *client, const struct Channel *channel)
 {
-  dlink_node *node;
   size_t len;
   char buf[IRCD_BUFSIZE];
   char *bufptr = buf + snprintf(buf, sizeof(buf), ":%s SJOIN %ju %s %s :",
@@ -205,7 +202,8 @@ channel_send_members(struct Client *client, const struct Channel *channel)
                                 channel->name, channel_modes(channel, client, true));
   char *const bufptr_start = bufptr;
 
-  DLINK_FOREACH(node, channel->members.head)
+  list_node_t *node;
+  LIST_FOREACH(node, channel->members.head)
   {
     const struct ChannelMember *member = node->data;
 
@@ -218,7 +216,8 @@ channel_send_members(struct Client *client, const struct Channel *channel)
       bufptr = bufptr_start;
     }
 
-    bufptr += snprintf(bufptr, sizeof(buf) - (bufptr - buf), bufptr != bufptr_start ? " %s%s" : "%s%s", member_get_prefix(member, true), member->client->id);
+    bufptr += snprintf(bufptr, sizeof(buf) - (bufptr - buf), bufptr != bufptr_start ? " %s%s" : "%s%s",
+                       member_get_prefix(member, true), member->client->id);
   }
 
   sendto_one(client, "%s", buf);
@@ -231,25 +230,21 @@ channel_send_members(struct Client *client, const struct Channel *channel)
  * \param flag     Char flag flagging type of mode. Currently this can be 'b', e' or 'I'
  */
 static void
-channel_send_mask_list(struct Client *client, const struct Channel *channel,
-                       const dlink_list *list, const char flag)
+channel_send_mask_list(struct Client *client, const struct Channel *channel, const list_t *list, const char flag)
 {
-  dlink_node *node;
-  size_t len;
-  char buf[IRCD_BUFSIZE];
-
-  if (dlink_list_length(list) == 0)
+  if (list_is_empty(list))
     return;
 
-  char *bufptr = buf + snprintf(buf, sizeof(buf), ":%s BMASK %ju %s %c :", me.id,
-                                channel->creation_time, channel->name, flag);
+  char buf[IRCD_BUFSIZE];
+  char *bufptr = buf + snprintf(buf, sizeof(buf), ":%s BMASK %ju %s %c :",
+                                me.id, channel->creation_time, channel->name, flag);
   char *const bufptr_start = bufptr;
 
-  DLINK_FOREACH(node, list->head)
+  list_node_t *node;
+  LIST_FOREACH(node, list->head)
   {
     const struct Ban *ban = node->data;
-
-    len = ban->banstr_len + 1;  /* +1 for space */
+    size_t len = ban->banstr_len + 1;  /* +1 for space */
 
     /*
      * Send buffer and start over if we cannot fit another ban
@@ -294,8 +289,7 @@ channel_send_modes(struct Client *client, const struct Channel *channel)
    */
   if (channel->topic_time)
     sendto_one(client, ":%s TBURST %ju %s %ju %s :%s",
-               me.id, channel->creation_time,
-               channel->name, channel->topic_time,
+               me.id, channel->creation_time, channel->name, channel->topic_time,
                channel->topic_info, channel->topic);
 
   if (IsCapable(client, CAPAB_MLOCK))
@@ -336,20 +330,20 @@ channel_check_name(const char *name, bool local)
 }
 
 void
-remove_ban(struct Ban *ban, dlink_list *list)
+remove_ban(struct Ban *ban, list_t *list)
 {
-  dlinkDelete(&ban->node, list);
-  xfree(ban);
+  list_remove(&ban->node, list);
+  io_free(ban);
 }
 
 /* channel_free_mask_list()
  *
- * inputs       - pointer to dlink_list
+ * inputs       - pointer to list_t
  * output       - NONE
  * side effects -
  */
 static void
-channel_free_mask_list(dlink_list *list)
+channel_free_mask_list(list_t *list)
 {
   while (list->head)
   {
@@ -368,18 +362,18 @@ channel_make(const char *name)
 {
   assert(!EmptyString(name));
 
-  struct Channel *channel = xcalloc(sizeof(*channel));
+  struct Channel *channel = io_calloc(sizeof(*channel));
   channel->hnextch = channel;
   /* Doesn't hurt to set it here */
-  channel->creation_time = event_base->time.sec_real;
-  channel->last_join_time = event_base->time.sec_monotonic;
+  channel->creation_time = io_time_get(IO_TIME_REALTIME_SEC);
+  channel->last_join_time = io_time_get(IO_TIME_MONOTONIC_SEC);
 
   /* Cache channel name length to avoid repetitive strlen() calls. */
   channel->name_len = strlcpy(channel->name, name, sizeof(channel->name));
   if (channel->name_len >= sizeof(channel->name))
     channel->name_len = sizeof(channel->name) - 1;
 
-  dlinkAdd(channel, &channel->node, &channel_list);
+  list_add(channel, &channel->node, &channel_list);
   hash_add_channel(channel);
 
   return channel;
@@ -398,7 +392,7 @@ channel_free(struct Channel *channel)
   channel_free_mask_list(&channel->exceptlist);
   channel_free_mask_list(&channel->invexlist);
 
-  dlinkDelete(&channel->node, &channel_list);
+  list_remove(&channel->node, &channel_list);
   hash_del_channel(channel);
 
   assert(channel->hnextch == channel);
@@ -406,32 +400,32 @@ channel_free(struct Channel *channel)
   assert(channel->node.prev == NULL);
   assert(channel->node.next == NULL);
 
-  assert(dlink_list_length(&channel->members_local) == 0);
+  assert(list_length(&channel->members_local) == 0);
   assert(channel->members_local.head == NULL);
   assert(channel->members_local.tail == NULL);
 
-  assert(dlink_list_length(&channel->members) == 0);
+  assert(list_length(&channel->members) == 0);
   assert(channel->members.head == NULL);
   assert(channel->members.tail == NULL);
 
-  assert(dlink_list_length(&channel->invites) == 0);
+  assert(list_length(&channel->invites) == 0);
   assert(channel->invites.head == NULL);
   assert(channel->invites.tail == NULL);
 
-  assert(dlink_list_length(&channel->banlist) == 0);
+  assert(list_length(&channel->banlist) == 0);
   assert(channel->banlist.head == NULL);
   assert(channel->banlist.tail == NULL);
 
-  assert(dlink_list_length(&channel->exceptlist) == 0);
+  assert(list_length(&channel->exceptlist) == 0);
   assert(channel->exceptlist.head == NULL);
   assert(channel->exceptlist.tail == NULL);
 
-  assert(dlink_list_length(&channel->invexlist) == 0);
+  assert(list_length(&channel->invexlist) == 0);
   assert(channel->invexlist.head == NULL);
   assert(channel->invexlist.tail == NULL);
 
-  xfree(channel->mode_lock);
-  xfree(channel);
+  io_free(channel->mode_lock);
+  io_free(channel);
 }
 
 /*!
@@ -455,30 +449,30 @@ channel_pub_or_secret(const struct Channel *channel)
 void
 channel_send_namereply(struct Client *client, struct Channel *channel)
 {
-  dlink_node *node;
-  char buf[IRCD_BUFSIZE];
-  char *bufptr = buf;
-  size_t masklen = 0;
   bool is_member = member_find_link(client, channel) != NULL;
   bool multi_prefix = HasCap(client, CAP_MULTI_PREFIX) != 0;
   bool uhnames = HasCap(client, CAP_UHNAMES) != 0;
 
   assert(IsClient(client));
 
-  if (PubChannel(channel) || is_member == true)
+  if (PubChannel(channel) || is_member)
   {
+    char buf[IRCD_BUFSIZE];
+    char *bufptr = buf;
     /* :me.name 353 client->name @ channel->name :+nick1 @nick2 %nick3 ...\r\n */
     /* 1       23456            789             01                        2 3  */
     size_t len = strlen(me.name) + strlen(client->name) + channel->name_len + 13;
 
-    DLINK_FOREACH(node, channel->members.head)
+    list_node_t *node;
+    LIST_FOREACH(node, channel->members.head)
     {
+      size_t masklen = 0;
       const struct ChannelMember *member = node->data;
 
-      if (HasUMode(member->client, UMODE_INVISIBLE) && is_member == false)
+      if (user_mode_has_flag(member->client, UMODE_INVISIBLE) && is_member == false)
         continue;
 
-      if (uhnames == true)
+      if (uhnames)
         masklen = strlen(member->client->name) + strlen(member->client->username) +
                   strlen(member->client->host) + 3;  /* +3 for ! + @ + space */
       else
@@ -493,15 +487,13 @@ channel_send_namereply(struct Client *client, struct Channel *channel)
         bufptr = buf;
       }
 
-      if (uhnames == true)
+      if (uhnames)
         bufptr += snprintf(bufptr, sizeof(buf) - (bufptr - buf), bufptr != buf ? " %s%s!%s@%s" : "%s%s!%s@%s",
                            member_get_prefix(member, multi_prefix),
-                           member->client->name, member->client->username,
-                           member->client->host);
+                           member->client->name, member->client->username, member->client->host);
       else
         bufptr += snprintf(bufptr, sizeof(buf) - (bufptr - buf), bufptr != buf ? " %s%s" : "%s%s",
-                           member_get_prefix(member, multi_prefix),
-                           member->client->name);
+                           member_get_prefix(member, multi_prefix), member->client->name);
     }
 
     if (bufptr != buf)
@@ -566,7 +558,7 @@ member_get_prefix(const struct ChannelMember *member, bool combine)
   char *bufptr = buf;
 
   for (const struct chan_mode *tab = cflag_tab; tab->letter; ++tab)
-    if (member_has_flags(member, tab->flag) == true)
+    if (member_has_flags(member, tab->flag))
       if (*bufptr++ = tab->prefix, combine == false)
         break;
 
@@ -580,7 +572,7 @@ member_get_prefix_len(const struct ChannelMember *member, bool combine)
   size_t len = 0;
 
   for (const struct chan_mode *tab = cflag_tab; tab->letter; ++tab)
-    if (member_has_flags(member, tab->flag) == true)
+    if (member_has_flags(member, tab->flag))
       if (++len, combine == false)
         break;
 
@@ -593,19 +585,19 @@ member_highest_rank(const struct ChannelMember *member)
   if (member == NULL)
     return CHACCESS_NOTONCHAN;
 
-  if (member_has_flags(member, CHFL_CHANOWNER) == true)
+  if (member_has_flags(member, CHFL_CHANOWNER))
     return CHACCESS_OWNER;
 
-  if (member_has_flags(member, CHFL_CHANADMIN) == true)
+  if (member_has_flags(member, CHFL_CHANADMIN))
     return CHACCESS_ADMIN;
 
-  if (member_has_flags(member, CHFL_CHANOP) == true)
+  if (member_has_flags(member, CHFL_CHANOP))
     return CHACCESS_OP;
 
-  if (member_has_flags(member, CHFL_HALFOP) == true)
+  if (member_has_flags(member, CHFL_HALFOP))
     return CHACCESS_HALFOP;
 
-  if (member_has_flags(member, CHFL_VOICE) == true)
+  if (member_has_flags(member, CHFL_VOICE))
     return CHACCESS_VOICE;
 
   return CHACCESS_PEON;
@@ -643,7 +635,7 @@ ban_matches(struct Client *client, struct Channel *channel, struct Ban *ban)
         break;
       case HM_IPV6:
       case HM_IPV4:
-        if (address_compare(&client->ip, &ban->addr, false, false, ban->bits) == true)
+        if (address_compare(&client->addr, &ban->addr, false, false, ban->bits))
           return true;
         break;
       default:
@@ -655,11 +647,11 @@ ban_matches(struct Client *client, struct Channel *channel, struct Ban *ban)
 }
 
 bool
-find_bmask(struct Client *client, struct Channel *channel, const dlink_list *list, struct Extban *extban)
+find_bmask(struct Client *client, struct Channel *channel, const list_t *list, struct Extban *extban)
 {
-  dlink_node *node;
+  list_node_t *node;
 
-  DLINK_FOREACH(node, list->head)
+  LIST_FOREACH(node, list->head)
   {
     struct Ban *ban = node->data;
 
@@ -697,7 +689,7 @@ find_bmask(struct Client *client, struct Channel *channel, const dlink_list *lis
 bool
 is_banned(struct Channel *channel, struct Client *client, struct Extban *extban)
 {
-  if (find_bmask(client, channel, &channel->banlist, extban) == true)
+  if (find_bmask(client, channel, &channel->banlist, extban))
     return find_bmask(client, channel, &channel->exceptlist, extban) == false;
   return false;
 }
@@ -712,13 +704,13 @@ is_banned(struct Channel *channel, struct Client *client, struct Extban *extban)
 static int
 can_join(struct Client *client, struct Channel *channel, const char *key)
 {
-  if (HasCMode(channel, MODE_SECUREONLY) && !HasUMode(client, UMODE_SECURE))
+  if (HasCMode(channel, MODE_SECUREONLY) && user_mode_has_flag(client, UMODE_SECURE) == false)
     return ERR_SECUREONLYCHAN;
 
-  if (HasCMode(channel, MODE_REGONLY) && !HasUMode(client, UMODE_REGISTERED))
+  if (HasCMode(channel, MODE_REGONLY) && user_mode_has_flag(client, UMODE_REGISTERED) == false)
     return ERR_NEEDREGGEDNICK;
 
-  if (HasCMode(channel, MODE_OPERONLY) && !HasUMode(client, UMODE_OPER))
+  if (HasCMode(channel, MODE_OPERONLY) && user_mode_has_flag(client, UMODE_OPER) == false)
     return ERR_OPERONLYCHAN;
 
   if (HasCMode(channel, MODE_INVITEONLY))
@@ -729,11 +721,11 @@ can_join(struct Client *client, struct Channel *channel, const char *key)
   if (channel->mode.key[0] && (key == NULL || strcmp(channel->mode.key, key)))
     return ERR_BADCHANNELKEY;
 
-  if (channel->mode.limit && dlink_list_length(&channel->members) >=
+  if (channel->mode.limit && list_length(&channel->members) >=
       channel->mode.limit)
     return ERR_CHANNELISFULL;
 
-  if (is_banned(channel, client, NULL) == true || is_banned(channel, client, &extban_join) == true)
+  if (is_banned(channel, client, NULL) || is_banned(channel, client, &extban_join))
     return ERR_BANNEDFROMCHAN;
 
   return 0;
@@ -750,21 +742,21 @@ member_has_flags(const struct ChannelMember *member, const unsigned int flags)
 struct ChannelMember *
 member_find_link(const struct Client *client, const struct Channel *channel)
 {
-  dlink_node *node;
-
   if (!IsClient(client))
     return NULL;
 
   /* Take the shortest of the two lists */
-  if (dlink_list_length(&channel->members) < dlink_list_length(&client->channel))
+  if (list_length(&channel->members) < list_length(&client->channel))
   {
-    DLINK_FOREACH(node, channel->members.head)
+    list_node_t *node;
+    LIST_FOREACH(node, channel->members.head)
       if (((struct ChannelMember *)node->data)->client == client)
         return node->data;
   }
   else
   {
-    DLINK_FOREACH(node, client->channel.head)
+    list_node_t *node;
+    LIST_FOREACH(node, client->channel.head)
       if (((struct ChannelMember *)node->data)->channel == channel)
         return node->data;
   }
@@ -812,30 +804,30 @@ msg_has_ctrls(const char *message)
  *         CAN_SEND_NONOP if can send to channel but is not an op\n
  *         CAN_SEND_NO if they cannot send to channel\n
  */
-int
-can_send(struct Channel *channel, struct Client *client,
-         struct ChannelMember *member, const char *message, bool notice, const char **error)
+channel_send_perm_t
+channel_send_qualifies(struct Channel *channel, struct Client *client, struct ChannelMember *member,
+                       unsigned int statusmsg, const char *message, bool notice, const char **error)
 {
   if (IsServer(client) || HasFlag(client, FLAGS_SERVICE))
-    return CAN_SEND_OPV;
+    return CHANNEL_SEND_PERM_ELEVATED;
 
   if (MyConnect(client) && !HasFlag(client, FLAGS_EXEMPTRESV))
   {
-    if (!(HasUMode(client, UMODE_OPER) && HasOFlag(client, OPER_FLAG_JOIN_RESV)))
+    if (!(user_mode_has_flag(client, UMODE_OPER) && HasOFlag(client, OPER_FLAG_JOIN_RESV)))
     {
       const struct ResvItem *const resv = resv_find(channel->name, match);
       if (resv && resv_exempt_find(client, resv) == false)
       {
         *error = "channel is reserved";
-        return CAN_SEND_NO;
+        return CHANNEL_SEND_PERM_FORBIDDEN;
       }
     }
   }
 
-  if (HasCMode(channel, MODE_NOCTRL) && msg_has_ctrls(message) == true)
+  if (HasCMode(channel, MODE_NOCTRL) && msg_has_ctrls(message))
   {
     *error = "control codes are not permitted";
-    return CAN_SEND_NO;
+    return CHANNEL_SEND_PERM_FORBIDDEN;
   }
 
   if (HasCMode(channel, MODE_NOCTCP))
@@ -843,36 +835,42 @@ can_send(struct Channel *channel, struct Client *client,
     if (*message == '\001' && strncmp(message + 1, "ACTION ", 7))
     {
       *error = "CTCPs are not permitted";
-      return CAN_SEND_NO;
+      return CHANNEL_SEND_PERM_FORBIDDEN;
     }
   }
 
   if (member || (member = member_find_link(client, channel)))
     if (member_highest_rank(member) > CHACCESS_PEON)
-      return CAN_SEND_OPV;
+      return CHANNEL_SEND_PERM_ELEVATED;
+
+  if (statusmsg)
+  {
+    *error = "STATUSMSG requires elevated privileges";
+    return CHANNEL_SEND_PERM_FORBIDDEN;
+  }
 
   if (member == NULL && HasCMode(channel, MODE_NOPRIVMSGS))
   {
     *error = "external messages are not permitted";
-    return CAN_SEND_NO;
+    return CHANNEL_SEND_PERM_FORBIDDEN;
   }
 
   if (HasCMode(channel, MODE_MODERATED))
   {
     *error = "channel is moderated (+m)";
-    return CAN_SEND_NO;
+    return CHANNEL_SEND_PERM_FORBIDDEN;
   }
 
-  if (HasCMode(channel, MODE_MODREG) && !HasUMode(client, UMODE_REGISTERED))
+  if (HasCMode(channel, MODE_MODREG) && user_mode_has_flag(client, UMODE_REGISTERED) == false)
   {
     *error = "you need to identify to a registered nick";
-    return CAN_SEND_NO;
+    return CHANNEL_SEND_PERM_FORBIDDEN;
   }
 
-  if (HasCMode(channel, MODE_NONOTICE) && notice == true)
+  if (HasCMode(channel, MODE_NONOTICE) && notice)
   {
     *error = "NOTICEs are not permitted";
-    return CAN_SEND_NO;
+    return CHANNEL_SEND_PERM_FORBIDDEN;
   }
 
   *error = "you are banned (+b)";
@@ -882,24 +880,24 @@ can_send(struct Channel *channel, struct Client *client,
     if (member)
     {
       if (member->flags & CHFL_BAN_SILENCED)
-        return CAN_SEND_NO;
+        return CHANNEL_SEND_PERM_FORBIDDEN;
 
       if (!(member->flags & CHFL_BAN_CHECKED))
       {
-        if (is_banned(channel, client, NULL) == true || is_banned(channel, client, &extban_mute) == true)
+        if (is_banned(channel, client, NULL) || is_banned(channel, client, &extban_mute))
         {
           member->flags |= (CHFL_BAN_CHECKED | CHFL_BAN_SILENCED);
-          return CAN_SEND_NO;
+          return CHANNEL_SEND_PERM_FORBIDDEN;
         }
 
         member->flags |= CHFL_BAN_CHECKED;
       }
     }
-    else if (is_banned(channel, client, NULL) == true || is_banned(channel, client, &extban_mute) == true)
-      return CAN_SEND_NO;
+    else if (is_banned(channel, client, NULL) || is_banned(channel, client, &extban_mute))
+      return CHANNEL_SEND_PERM_FORBIDDEN;
   }
 
-  return CAN_SEND_NONOP;
+  return CHANNEL_SEND_PERM_STANDARD;
 }
 
 /*! \brief Updates the client's oper_warn_count_down, warns the
@@ -922,15 +920,14 @@ channel_check_spambot_warning(struct Client *client, const char *name)
       client->connection->oper_warn_count_down = OPER_SPAM_COUNTDOWN;
 
       /* It's already known as a possible spambot */
-      sendto_realops_flags(UMODE_FLOOD, L_ALL, SEND_NOTICE,
-                           "User %s (%s@%s) trying to join %s is a possible spambot",
-                           client->name, client->username,
-                           client->host, name);
+      sendto_clients(UMODE_FLOOD, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                     "User %s (%s@%s) trying to join %s is a possible spambot",
+                     client->name, client->username, client->host, name);
     }
   }
   else
   {
-    unsigned int t_delta = event_base->time.sec_monotonic - client->connection->last_leave_time;
+    unsigned int t_delta = io_time_get(IO_TIME_MONOTONIC_SEC) - client->connection->last_leave_time;
     if (t_delta > JOIN_LEAVE_COUNT_EXPIRE_TIME)
     {
       unsigned int decrement_count = (t_delta / JOIN_LEAVE_COUNT_EXPIRE_TIME);
@@ -941,14 +938,14 @@ channel_check_spambot_warning(struct Client *client, const char *name)
     }
     else
     {
-      if ((event_base->time.sec_monotonic - client->connection->last_join_time) < GlobalSetOptions.spam_time)
+      if ((io_time_get(IO_TIME_MONOTONIC_SEC) - client->connection->last_join_time) < GlobalSetOptions.spam_time)
         ++client->connection->join_leave_count;  /* It's a possible spambot */
     }
 
     if (name)
-      client->connection->last_join_time = event_base->time.sec_monotonic;
+      client->connection->last_join_time = io_time_get(IO_TIME_MONOTONIC_SEC);
     else
-      client->connection->last_leave_time = event_base->time.sec_monotonic;
+      client->connection->last_leave_time = io_time_get(IO_TIME_MONOTONIC_SEC);
   }
 }
 
@@ -960,11 +957,10 @@ channel_check_spambot_warning(struct Client *client, const char *name)
  * \param local      Whether the topic is set by a local client
  */
 void
-channel_set_topic(struct Channel *channel, const char *topic,
-                  const char *topic_info, uintmax_t topicts, bool local)
+channel_set_topic(struct Channel *channel, const char *topic, const char *topic_info, uintmax_t topicts, bool local)
 {
-  if (local == true)
-    strlcpy(channel->topic, topic, IRCD_MIN(sizeof(channel->topic), ConfigServerInfo.max_topic_length + 1));
+  if (local)
+    strlcpy(channel->topic, topic, IO_MIN(sizeof(channel->topic), ConfigServerInfo.max_topic_length + 1));
   else
     strlcpy(channel->topic, topic, sizeof(channel->topic));
 
@@ -978,25 +974,24 @@ channel_set_topic(struct Channel *channel, const char *topic,
  * \param mode_lock  The modes to lock as a string. Can be NULL.
  */
 void
-channel_set_mode_lock(struct Client *client, struct Channel *channel,
-                      const char *mode_lock)
+channel_set_mode_lock(struct Client *client, struct Channel *channel, const char *mode_lock)
 {
-  xfree(channel->mode_lock);
+  io_free(channel->mode_lock);
   channel->mode_lock = NULL;
 
   if (!EmptyString(mode_lock))
-    channel->mode_lock = xstrdup(mode_lock);
+    channel->mode_lock = io_strdup(mode_lock);
 }
 
 void
-channel_do_join(struct Client *client, char *chan_list, char *key_list)
+channel_join_list(struct Client *client, char *chan_list, char *key_list)
 {
-  char *p = NULL;
   const struct ResvItem *resv = NULL;
   const struct ClassItem *const class = class_get_ptr(&client->connection->confs);
 
   assert(MyClient(client));
 
+  char *p = NULL;
   for (const char *name = strtok_r(chan_list, ",", &p); name;
                    name = strtok_r(NULL,      ",", &p))
   {
@@ -1018,17 +1013,16 @@ channel_do_join(struct Client *client, char *chan_list, char *key_list)
     }
 
     if (!HasFlag(client, FLAGS_EXEMPTRESV) &&
-        !(HasUMode(client, UMODE_OPER) && HasOFlag(client, OPER_FLAG_JOIN_RESV)) &&
+        !(user_mode_has_flag(client, UMODE_OPER) && HasOFlag(client, OPER_FLAG_JOIN_RESV)) &&
         ((resv = resv_find(name, match)) && resv_exempt_find(client, resv) == false))
     {
       sendto_one_numeric(client, &me, ERR_CHANBANREASON, name, resv->reason);
-      sendto_realops_flags(UMODE_REJ, L_ALL, SEND_NOTICE,
-                           "Forbidding reserved channel %s from user %s",
-                           name, client_get_name(client, HIDE_IP));
+      sendto_clients(UMODE_REJ, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Forbidding reserved channel %s from user %s",
+                     name, client_get_name(client, HIDE_IP));
       continue;
     }
 
-    if (dlink_list_length(&client->channel) >=
+    if (list_length(&client->channel) >=
         ((class->max_channels) ? class->max_channels : ConfigChannel.max_channels))
     {
       sendto_one_numeric(client, &me, ERR_TOOMANYCHANNELS, name);
@@ -1055,10 +1049,11 @@ channel_do_join(struct Client *client, char *chan_list, char *key_list)
       channel = channel_make(name);
     }
 
-    if (!HasUMode(client, UMODE_OPER))
+    if (user_mode_has_flag(client, UMODE_OPER) == false)
       channel_check_spambot_warning(client, channel->name);
 
     channel_add_user(channel, client, flags, true);
+    client->connection->last_join_time = io_time_get(IO_TIME_MONOTONIC_SEC);
 
     /*
      * Set channel modes if appropriate, and propagate
@@ -1068,41 +1063,33 @@ channel_do_join(struct Client *client, char *chan_list, char *key_list)
       AddCMode(channel, MODE_TOPICLIMIT);
       AddCMode(channel, MODE_NOPRIVMSGS);
 
-      sendto_server(NULL, 0, 0, ":%s SJOIN %ju %s +nt :@%s",
-                    me.id, channel->creation_time,
-                    channel->name, client->id);
+      sendto_servers(NULL, 0, 0, ":%s SJOIN %ju %s +nt :@%s",
+                     me.id, channel->creation_time, channel->name, client->id);
 
       /*
        * Notify all other users on the new channel
        */
       sendto_channel_local(NULL, channel, 0, CAP_EXTENDED_JOIN, 0, ":%s!%s@%s JOIN %s %s :%s",
-                           client->name, client->username,
-                           client->host, channel->name, client->account, client->info);
+                           client->name, client->username, client->host, channel->name, client->account, client->info);
       sendto_channel_local(NULL, channel, 0, 0, CAP_EXTENDED_JOIN, ":%s!%s@%s JOIN :%s",
-                           client->name, client->username,
-                           client->host, channel->name);
+                           client->name, client->username, client->host, channel->name);
       sendto_channel_local(NULL, channel, 0, 0, 0, ":%s MODE %s +nt",
                            me.name, channel->name);
     }
     else
     {
-      sendto_server(NULL, 0, 0, ":%s JOIN %ju %s +",
-                    client->id, channel->creation_time,
-                    channel->name);
+      sendto_servers(NULL, 0, 0, ":%s JOIN %ju %s +",
+                     client->id, channel->creation_time, channel->name);
 
       sendto_channel_local(NULL, channel, 0, CAP_EXTENDED_JOIN, 0, ":%s!%s@%s JOIN %s %s :%s",
-                           client->name, client->username,
-                           client->host, channel->name, client->account, client->info);
+                           client->name, client->username, client->host, channel->name, client->account, client->info);
       sendto_channel_local(NULL, channel, 0, 0, CAP_EXTENDED_JOIN, ":%s!%s@%s JOIN :%s",
-                           client->name, client->username,
-                           client->host, channel->name);
+                           client->name, client->username, client->host, channel->name);
     }
 
     if (client->away[0])
-      sendto_channel_local(client, channel, 0, CAP_AWAY_NOTIFY, 0,
-                           ":%s!%s@%s AWAY :%s",
-                           client->name, client->username,
-                           client->host, client->away);
+      sendto_channel_local(client, channel, 0, CAP_AWAY_NOTIFY, 0, ":%s!%s@%s AWAY :%s",
+                           client->name, client->username, client->host, client->away);
 
     struct Invite *invite = invite_find(channel, client);
     if (invite)
@@ -1111,13 +1098,11 @@ channel_do_join(struct Client *client, char *chan_list, char *key_list)
     if (channel->topic[0])
     {
       sendto_one_numeric(client, &me, RPL_TOPIC, channel->name, channel->topic);
-      sendto_one_numeric(client, &me, RPL_TOPICWHOTIME, channel->name,
-                         channel->topic_info, channel->topic_time);
+      sendto_one_numeric(client, &me, RPL_TOPICWHOTIME,
+                         channel->name, channel->topic_info, channel->topic_time);
     }
 
     channel_send_namereply(client, channel);
-
-    client->connection->last_join_time = event_base->time.sec_monotonic;
   }
 }
 
@@ -1127,7 +1112,7 @@ channel_do_join(struct Client *client, char *chan_list, char *key_list)
  * \param reason Part reason to show
  */
 static void
-channel_part_one_client(struct Client *client, const char *name, const char *reason)
+channel_part_one(struct Client *client, const char *name, const char *reason)
 {
   struct Channel *channel = hash_find_channel(name);
   if (channel == NULL)
@@ -1143,7 +1128,7 @@ channel_part_one_client(struct Client *client, const char *name, const char *rea
     return;
   }
 
-  if (MyConnect(client) && !HasUMode(client, UMODE_OPER))
+  if (MyConnect(client) && user_mode_has_flag(client, UMODE_OPER) == false)
     channel_check_spambot_warning(client, NULL);
 
   /*
@@ -1156,39 +1141,37 @@ channel_part_one_client(struct Client *client, const char *name, const char *rea
   {
     const char *error;
     if ((client->connection->created_monotonic + ConfigGeneral.anti_spam_exit_message_time)
-          >= event_base->time.sec_monotonic)
+          >= io_time_get(IO_TIME_MONOTONIC_SEC))
       show_reason = false;
-    else if (can_send(channel, client, member, reason, false, &error) == CAN_SEND_NO)
+    else if (channel_send_qualifies(channel, client, member, 0, reason, false, &error) == CHANNEL_SEND_PERM_FORBIDDEN)
       show_reason = false;
   }
 
-  if (show_reason == true)
+  if (show_reason)
   {
-    sendto_server(client, 0, 0, ":%s PART %s :%.*s",
-                  client->id, channel->name, KICKLEN, reason);
+    sendto_servers(client, 0, 0, ":%s PART %s :%.*s",
+                   client->id, channel->name, KICKLEN, reason);
     sendto_channel_local(NULL, channel, 0, 0, 0, ":%s!%s@%s PART %s :%.*s",
-                         client->name, client->username,
-                         client->host, channel->name, KICKLEN, reason);
+                         client->name, client->username, client->host, channel->name, KICKLEN, reason);
   }
   else
   {
-    sendto_server(client, 0, 0, ":%s PART %s",
-                  client->id, channel->name);
+    sendto_servers(client, 0, 0, ":%s PART %s",
+                   client->id, channel->name);
     sendto_channel_local(NULL, channel, 0, 0, 0, ":%s!%s@%s PART %s",
-                         client->name, client->username,
-                         client->host, channel->name);
+                         client->name, client->username, client->host, channel->name);
   }
 
   channel_remove_user(member);
 }
 
 void
-channel_do_part(struct Client *client, char *channel, const char *reason)
+channel_part_list(struct Client *client, char *list, const char *reason)
 {
-  char *p = NULL;
   assert(IsClient(client));
 
-  for (const char *name = strtok_r(channel, ",", &p); name;
-                   name = strtok_r(NULL,    ",", &p))
-    channel_part_one_client(client, name, reason);
+  char *p = NULL;
+  for (const char *name = strtok_r(list, ",", &p); name;
+                   name = strtok_r(NULL, ",", &p))
+    channel_part_one(client, name, reason);
 }
