@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,14 +24,13 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "memory.h"
 #include "list.h"
-#include "event.h"
 #include "channel.h"
 #include "channel_invite.h"
 #include "conf.h"
 
-
 /*! \brief Checks whether a client is invited to a certain channel. While walking the
  *         shortest invitation list, expired invitations are vaccumed automatically.
  * \param channel Pointer to channel block
@@ -41,21 +40,14 @@
 struct Invite *
 invite_find(struct Channel *channel, struct Client *client)
 {
-  dlink_node *node, *node_next;
-  dlink_list *list;
+  list_t *list = list_get_shorter(&client->connection->invited, &channel->invites);
 
-  /* Take the shortest of the two lists */
-  if (dlink_list_length(&client->connection->invited) < dlink_list_length(&channel->invites))
-    list = &client->connection->invited;
-  else
-    list = &channel->invites;
-
-  DLINK_FOREACH_SAFE(node, node_next, list->head)
+  list_node_t *node, *node_next;
+  LIST_FOREACH_SAFE(node, node_next, list->head)
   {
     struct Invite *invite = node->data;
-
     if (ConfigChannel.invite_expire_time &&
-        ConfigChannel.invite_expire_time + invite->when < event_base->time.sec_monotonic)
+        ConfigChannel.invite_expire_time + invite->when < io_time_get(IO_TIME_MONOTONIC_SEC))
       invite_del(invite);
     else if (invite->channel == channel && invite->client == client)
       return invite;
@@ -75,21 +67,21 @@ invite_add(struct Channel *channel, struct Client *client)
   if (invite)
     invite_del(invite);
 
-  invite = xcalloc(sizeof(*invite));
+  invite = io_calloc(sizeof(*invite));
   invite->client = client;
   invite->channel = channel;
-  invite->when = event_base->time.sec_monotonic;
+  invite->when = io_time_get(IO_TIME_MONOTONIC_SEC);
 
   /* Delete last link in chain if the list is max length */
-  while (dlink_list_length(&client->connection->invited) &&
-         dlink_list_length(&client->connection->invited) >= ConfigChannel.max_invites)
+  while (list_length(&client->connection->invited) &&
+         list_length(&client->connection->invited) >= ConfigChannel.max_invites)
     invite_del(client->connection->invited.tail->data);
 
   /* Add client to channel invite list */
-  dlinkAdd(invite, &invite->chan_node, &channel->invites);
+  list_add(invite, &invite->chan_node, &channel->invites);
 
   /* Add channel to the end of the client invite list */
-  dlinkAdd(invite, &invite->user_node, &client->connection->invited);
+  list_add(invite, &invite->user_node, &client->connection->invited);
 }
 
 /*! \brief Delete Invite block from channel invite list
@@ -99,18 +91,16 @@ invite_add(struct Channel *channel, struct Client *client)
 void
 invite_del(struct Invite *invite)
 {
-  dlinkDelete(&invite->user_node, &invite->client->connection->invited);
-  dlinkDelete(&invite->chan_node, &invite->channel->invites);
-
-  /* Release memory pointed to by 'invite' */
-  xfree(invite);
+  list_remove(&invite->user_node, &invite->client->connection->invited);
+  list_remove(&invite->chan_node, &invite->channel->invites);
+  io_free(invite);
 }
 
 /*! \brief Removes and frees all Invite blocks from a list
- * \param list Pointer to a dlink_list
+ * \param list Pointer to a list_t
  */
 void
-invite_clear_list(dlink_list *list)
+invite_clear_list(list_t *list)
 {
   while (list->head)
     invite_del(list->head->data);
