@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,24 +24,26 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "list.h"
 #include "channel.h"
 #include "channel_mode.h"
 #include "client.h"
 #include "conf.h"
-#include "hostmask.h"
+#include "address.h"
 #include "irc_string.h"
 #include "ircd.h"
+#include "nuh.h"
 #include "numeric.h"
 #include "server.h"
 #include "send.h"
+#include "user_mode.h"
 #include "memory.h"
 #include "parse.h"
 #include "extban.h"
 #include "isupport.h"
 #include "server_capab.h"
 
-
 /** Buffer holding a list of channel modes to be used for RPL_MYINFO */
 char cmode_rpl04[2][256];
 /** Buffer holding a list of channel modes to be used for RPL_ISUPPORT */
@@ -52,7 +54,6 @@ static unsigned int mode_count;
 static unsigned int mode_limit;  /* number of modes set other than simple */
 static unsigned int simple_modes_mask;  /* bit mask of simple modes already set */
 
-
 /* check_string()
  *
  * inputs       - string to check
@@ -107,7 +108,7 @@ get_mask(const struct Ban *ban)
  */
 
 const char *
-add_id(struct Client *client, struct Channel *channel, const char *banid, dlink_list *list, unsigned int type)
+add_id(struct Client *client, struct Channel *channel, const char *banid, list_t *list, unsigned int type)
 {
   char mask[MODEBUFLEN];
   char *maskptr = mask;
@@ -117,13 +118,12 @@ add_id(struct Client *client, struct Channel *channel, const char *banid, dlink_
 
   if (MyClient(client))
   {
-    unsigned int num_mask = dlink_list_length(&channel->banlist) +
-                            dlink_list_length(&channel->exceptlist) +
-                            dlink_list_length(&channel->invexlist);
+    unsigned int num_mask = list_length(&channel->banlist) +
+                            list_length(&channel->exceptlist) +
+                            list_length(&channel->invexlist);
 
     /* Don't let local clients overflow the b/e/I lists */
-    if (num_mask >= ((HasCMode(channel, MODE_EXTLIMIT)) ? ConfigChannel.max_bans_large :
-                                                          ConfigChannel.max_bans))
+    if (num_mask >= ((HasCMode(channel, MODE_EXTLIMIT)) ? ConfigChannel.max_bans_large : ConfigChannel.max_bans))
     {
       sendto_one_numeric(client, &me, ERR_BANLISTFULL, channel->name, banid);
       return NULL;
@@ -176,9 +176,9 @@ add_id(struct Client *client, struct Channel *channel, const char *banid, dlink_
   if (EmptyString(maskptr))
     return NULL;
 
-  struct Ban *ban = xcalloc(sizeof(*ban));
+  struct Ban *ban = io_calloc(sizeof(*ban));
   ban->extban = extbans;
-  ban->when = event_base->time.sec_real;
+  ban->when = io_time_get(IO_TIME_REALTIME_SEC);
 
   check_string(maskptr);
 
@@ -187,18 +187,18 @@ add_id(struct Client *client, struct Channel *channel, const char *banid, dlink_
     strlcpy(ban->host, maskptr, sizeof(ban->host));
   else
   {
-    struct split_nuh_item nuh;
+    struct nuh_split nuh =
+    {
+      .nuhmask = maskptr,
+      .nickptr = ban->name,
+      .userptr = ban->user,
+      .hostptr = ban->host,
+      .nicksize = sizeof(ban->name),
+      .usersize = sizeof(ban->user),
+      .hostsize = sizeof(ban->host)
+    };
 
-    nuh.nuhmask  = maskptr;
-    nuh.nickptr  = ban->name;
-    nuh.userptr  = ban->user;
-    nuh.hostptr  = ban->host;
-
-    nuh.nicksize = sizeof(ban->name);
-    nuh.usersize = sizeof(ban->user);
-    nuh.hostsize = sizeof(ban->host);
-
-    split_nuh(&nuh);
+    nuh_split(&nuh);
 
     ban->type = parse_netmask(ban->host, &ban->addr, &ban->bits);
   }
@@ -208,14 +208,14 @@ add_id(struct Client *client, struct Channel *channel, const char *banid, dlink_
   else
     ban->banstr_len = strlcpy(ban->banstr, banid, sizeof(ban->banstr));
 
-  dlink_node *node;
-  DLINK_FOREACH(node, list->head)
+  list_node_t *node;
+  LIST_FOREACH(node, list->head)
   {
     const struct Ban *tmp = node->data;
 
     if (irccmp(tmp->banstr, ban->banstr) == 0)
     {
-      xfree(ban);
+      io_free(ban);
       return NULL;
     }
   }
@@ -223,14 +223,14 @@ add_id(struct Client *client, struct Channel *channel, const char *banid, dlink_
   clear_ban_cache_list(&channel->members_local);
 
   if (IsClient(client))
-    snprintf(ban->who, sizeof(ban->who), "%s!%s@%s", client->name,
-             client->username, client->host);
+    snprintf(ban->who, sizeof(ban->who), "%s!%s@%s",
+             client->name, client->username, client->host);
   else if (IsHidden(client) || ConfigServerHide.hide_servers)
     strlcpy(ban->who, me.name, sizeof(ban->who));
   else
     strlcpy(ban->who, client->name, sizeof(ban->who));
 
-  dlinkAdd(ban, &ban->node, list);
+  list_add(ban, &ban->node, list);
 
   return ban->banstr;
 }
@@ -243,16 +243,16 @@ add_id(struct Client *client, struct Channel *channel, const char *banid, dlink_
  * side effects	-
  */
 static const char *
-del_id(struct Client *client, struct Channel *channel, const char *banid, dlink_list *list, unsigned int type)
+del_id(struct Client *client, struct Channel *channel, const char *banid, list_t *list, unsigned int type)
 {
   static char mask[MODEBUFLEN];
-  dlink_node *node;
+  list_node_t *node;
 
   assert(banid);
 
   /* TBD: n!u@h formatting fo local clients */
 
-  DLINK_FOREACH(node, list->head)
+  LIST_FOREACH(node, list->head)
   {
     struct Ban *ban = node->data;
 
@@ -296,7 +296,7 @@ channel_modes(const struct Channel *channel, const struct Client *client, bool p
   {
     *bufptr++ = 'l';
 
-    if (params == true)
+    if (params)
       server_or_member =
         (server_or_member == true || IsServer(client) || member_find_link(client, channel));
   }
@@ -305,12 +305,12 @@ channel_modes(const struct Channel *channel, const struct Client *client, bool p
   {
     *bufptr++ = 'k';
 
-    if (params == true)
+    if (params)
       server_or_member =
         (server_or_member == true || IsServer(client) || member_find_link(client, channel));
   }
 
-  if (server_or_member == true)
+  if (server_or_member)
   {
     if (channel->mode.limit)
       bufptr += snprintf(bufptr, sizeof(buf) - (bufptr - buf), " %u", channel->mode.limit);
@@ -354,11 +354,11 @@ fix_key(char *arg)
  * side effects - clear ban cache
  */
 void
-clear_ban_cache_list(dlink_list *list)
+clear_ban_cache_list(list_t *list)
 {
-  dlink_node *node;
+  list_node_t *node;
 
-  DLINK_FOREACH(node, list->head)
+  LIST_FOREACH(node, list->head)
   {
     struct ChannelMember *member = node->data;
     member->flags &= ~(CHFL_BAN_SILENCED | CHFL_BAN_CHECKED);
@@ -389,9 +389,9 @@ channel_mode_can_change(struct Client *client, struct Channel *channel, int *err
   if (!MyClient(client))
     return true;
 
-  if (mode->only_opers == true)
+  if (mode->only_opers)
   {
-    if (!HasUMode(client, UMODE_OPER))
+    if (user_mode_has_flag(client, UMODE_OPER) == false)
     {
       if (!(*errors & SM_ERR_NOTOPER))
         sendto_one_numeric(client, &me, ERR_NOPRIVILEGES);
@@ -401,14 +401,13 @@ channel_mode_can_change(struct Client *client, struct Channel *channel, int *err
     }
   }
 
-  if (mode->only_servers == true)
+  if (mode->only_servers)
   {
     if (!IsServer(client) && !HasFlag(client, FLAGS_SERVICE))
     {
       if (!(*errors & SM_ERR_ONLYSERVER))
         sendto_one_numeric(client, &me,
-                           rank == CHACCESS_NOTONCHAN ? ERR_NOTONCHANNEL :
-                           ERR_ONLYSERVERSCANCHANGE, channel->name);
+                           rank == CHACCESS_NOTONCHAN ? ERR_NOTONCHANNEL : ERR_ONLYSERVERSCANCHANGE, channel->name);
 
       *errors |= SM_ERR_ONLYSERVER;
       return false;
@@ -419,8 +418,7 @@ channel_mode_can_change(struct Client *client, struct Channel *channel, int *err
   {
     if (!(*errors & SM_ERR_NOOPS))
       sendto_one_numeric(client, &me,
-                         rank == CHACCESS_NOTONCHAN ? ERR_NOTONCHANNEL :
-                         ERR_CHANOPRIVSNEEDED, channel->name);
+                         rank == CHACCESS_NOTONCHAN ? ERR_NOTONCHANNEL : ERR_CHANOPRIVSNEEDED, channel->name);
 
     *errors |= SM_ERR_NOOPS;
     return false;
@@ -429,8 +427,7 @@ channel_mode_can_change(struct Client *client, struct Channel *channel, int *err
   if (channel->mode_lock && strchr(channel->mode_lock, c))
   {
     if (!(*errors & SM_ERR_MLOCK))
-      sendto_one_numeric(client, &me, ERR_MLOCKRESTRICTED,
-                         channel->name, c, channel->mode_lock);
+      sendto_one_numeric(client, &me, ERR_MLOCKRESTRICTED, channel->name, c, channel->mode_lock);
 
     *errors |= SM_ERR_MLOCK;
     return false;
@@ -508,7 +505,7 @@ chm_mask(struct Client *client, struct Channel *channel, int parc, int *parn, ch
          int *errors, int rank, int dir, const char c, const struct chan_mode *mode)
 {
   const char *ret = NULL;
-  dlink_list *list;
+  list_t *list;
   enum irc_numerics rpl_list = 0, rpl_endlist = 0;
   int errtype = 0;
 
@@ -536,19 +533,18 @@ chm_mask(struct Client *client, struct Channel *channel, int parc, int *parn, ch
       list = NULL;  /* Let it crash */
   }
 
-  if (dir == MODE_QUERY || parc <= *parn)
+  if (parc <= *parn)
   {
     if (*errors & errtype)
       return;
 
     *errors |= errtype;
 
-    dlink_node *node;
-    DLINK_FOREACH(node, list->head)
+    list_node_t *node;
+    LIST_FOREACH(node, list->head)
     {
       const struct Ban *ban = node->data;
-      sendto_one_numeric(client, &me, rpl_list, channel->name,
-                         ban->banstr, ban->who, ban->when);
+      sendto_one_numeric(client, &me, rpl_list, channel->name, ban->banstr, ban->who, ban->when);
     }
 
     sendto_one_numeric(client, &me, rpl_endlist, channel->name);
@@ -597,7 +593,7 @@ chm_flag(struct Client *client, struct Channel *channel, int parc, int *parn, ch
   if (channel_mode_can_change(client, channel, errors, rank, c, mode) == false)
     return;
 
-  if (dir == MODE_QUERY || parc <= *parn)
+  if (parc <= *parn)
     return;
 
   struct Client *client_target = find_chasing(client, parv[(*parn)++]);
@@ -619,7 +615,7 @@ chm_flag(struct Client *client, struct Channel *channel, int parc, int *parn, ch
 
   if (dir == MODE_ADD)  /* setting + */
   {
-    if (member_has_flags(member, mode->flag) == true)
+    if (member_has_flags(member, mode->flag))
       return;  /* No redundant mode changes */
 
     AddMemberFlag(member, mode->flag);
@@ -645,9 +641,6 @@ chm_limit(struct Client *client, struct Channel *channel, int parc, int *parn, c
   if (channel_mode_can_change(client, channel, errors, rank, c, mode) == false)
     return;
 
-  if (dir == MODE_QUERY)
-    return;
-
   if (dir == MODE_ADD && parc > *parn)
   {
     char *const lstr = parv[(*parn)++];
@@ -691,9 +684,6 @@ chm_key(struct Client *client, struct Channel *channel, int parc, int *parn, cha
   if (channel_mode_can_change(client, channel, errors, rank, c, mode) == false)
     return;
 
-  if (dir == MODE_QUERY)
-    return;
-
   if (dir == MODE_ADD && parc > *parn)
   {
     char *const key = fix_key(parv[(*parn)++]);
@@ -701,7 +691,7 @@ chm_key(struct Client *client, struct Channel *channel, int parc, int *parn, cha
     if (EmptyString(key))
       return;
 
-    assert(key[0] != ' ');
+    assert(*key != ' ');
     strlcpy(channel->mode.key, key, sizeof(channel->mode.key));
 
     /* If somebody does MODE #channel +kk a b, accept latter --fl */
@@ -745,7 +735,7 @@ send_mode_changes_server(struct Client *client, struct Channel *channel)
   char parabuf[IRCD_BUFSIZE] = "";  /* Essential that parabuf[0] = '\0' */
   char *parptr = parabuf;
   unsigned int mbl = 0, pbl = 0, arglen = 0, modecount = 0, paracount = 0;
-  unsigned int dir = MODE_QUERY;
+  unsigned int dir = MODE_NONE;
 
   mbl = snprintf(modebuf, sizeof(modebuf), ":%s TMODE %ju %s ",
                  client->id, channel->creation_time, channel->name);
@@ -771,11 +761,10 @@ send_mode_changes_server(struct Client *client, struct Channel *channel)
      * If we're creeping past the buf size, we need to send it and make
      * another line for the other modes
      */
-    if ((paracount == MAXMODEPARAMS) ||
-        ((arglen + mbl + pbl + 2 /* +2 for /r/n */ ) > sizeof(modebuf)))
+    if ((paracount == MAXMODEPARAMS) || ((arglen + mbl + pbl + 2 /* +2 for /r/n */ ) > sizeof(modebuf)))
     {
       if (modecount)
-        sendto_server(client, 0, 0, paracount == 0 ? "%s" : "%s %s", modebuf, parabuf);
+        sendto_servers(client, 0, 0, paracount == 0 ? "%s" : "%s %s", modebuf, parabuf);
 
       modecount = 0;
       paracount = 0;
@@ -786,12 +775,12 @@ send_mode_changes_server(struct Client *client, struct Channel *channel)
       pbl = 0;
       parabuf[0] = '\0';
       parptr = parabuf;
-      dir = MODE_QUERY;
+      dir = MODE_NONE;
     }
 
     if (dir != mode_changes[i].dir)
     {
-      modebuf[mbl++] = (mode_changes[i].dir == MODE_ADD) ? '+' : '-';
+      modebuf[mbl++] = mode_changes[i].dir == MODE_ADD ? '+' : '-';
       dir = mode_changes[i].dir;
     }
 
@@ -801,7 +790,7 @@ send_mode_changes_server(struct Client *client, struct Channel *channel)
 
     if (arg)
     {
-      int len = sprintf(parptr, (pbl == 0) ? "%s" : " %s", arg);
+      int len = sprintf(parptr, pbl == 0 ? "%s" : " %s", arg);
       pbl += len;
       parptr += len;
       ++paracount;
@@ -809,7 +798,7 @@ send_mode_changes_server(struct Client *client, struct Channel *channel)
   }
 
   if (modecount)
-    sendto_server(client, 0, 0, paracount == 0 ? "%s" : "%s %s", modebuf, parabuf);
+    sendto_servers(client, 0, 0, paracount == 0 ? "%s" : "%s %s", modebuf, parabuf);
 }
 
 /* void send_mode_changes(struct Client *client,
@@ -829,15 +818,14 @@ send_mode_changes_client(struct Client *client, struct Channel *channel)
   char parabuf[IRCD_BUFSIZE] = "";  /* Essential that parabuf[0] = '\0' */
   char *parptr = parabuf;
   unsigned int mbl = 0, pbl = 0, arglen = 0, modecount = 0, paracount = 0;
-  unsigned int dir = MODE_QUERY;
+  unsigned int dir = MODE_NONE;
 
   if (IsClient(client))
-    mbl = snprintf(modebuf, sizeof(modebuf), ":%s!%s@%s MODE %s ", client->name,
-                   client->username, client->host, channel->name);
+    mbl = snprintf(modebuf, sizeof(modebuf), ":%s!%s@%s MODE %s ",
+                   client->name, client->username, client->host, channel->name);
   else
-    mbl = snprintf(modebuf, sizeof(modebuf), ":%s MODE %s ", (IsHidden(client) ||
-                   ConfigServerHide.hide_servers) ?
-                   me.name : client->name, channel->name);
+    mbl = snprintf(modebuf, sizeof(modebuf), ":%s MODE %s ",
+                   (IsHidden(client) || ConfigServerHide.hide_servers) ? me.name : client->name, channel->name);
 
   for (unsigned int i = 0; i < mode_count; ++i)
   {
@@ -850,8 +838,7 @@ send_mode_changes_client(struct Client *client, struct Channel *channel)
     else
       arglen = 0;
 
-    if ((paracount == MAXMODEPARAMS) ||
-        ((arglen + mbl + pbl + 2 /* +2 for /r/n */ ) > sizeof(modebuf)))
+    if ((paracount == MAXMODEPARAMS) || ((arglen + mbl + pbl + 2 /* +2 for /r/n */ ) > sizeof(modebuf)))
     {
       if (modecount)
         sendto_channel_local(NULL, channel, 0, 0, 0, paracount == 0 ? "%s" : "%s %s", modebuf, parabuf);
@@ -860,22 +847,21 @@ send_mode_changes_client(struct Client *client, struct Channel *channel)
       paracount = 0;
 
       if (IsClient(client))
-        mbl = snprintf(modebuf, sizeof(modebuf), ":%s!%s@%s MODE %s ", client->name,
-                       client->username, client->host, channel->name);
+        mbl = snprintf(modebuf, sizeof(modebuf), ":%s!%s@%s MODE %s ",
+                       client->name, client->username, client->host, channel->name);
       else
-        mbl = snprintf(modebuf, sizeof(modebuf), ":%s MODE %s ", (IsHidden(client) ||
-                       ConfigServerHide.hide_servers) ?
-                       me.name : client->name, channel->name);
+        mbl = snprintf(modebuf, sizeof(modebuf), ":%s MODE %s ",
+                       (IsHidden(client) || ConfigServerHide.hide_servers) ? me.name : client->name, channel->name);
 
       pbl = 0;
       parabuf[0] = '\0';
       parptr = parabuf;
-      dir = MODE_QUERY;
+      dir = MODE_NONE;
     }
 
     if (dir != mode_changes[i].dir)
     {
-      modebuf[mbl++] = (mode_changes[i].dir == MODE_ADD) ? '+' : '-';
+      modebuf[mbl++] = mode_changes[i].dir == MODE_ADD ? '+' : '-';
       dir = mode_changes[i].dir;
     }
 
@@ -885,7 +871,7 @@ send_mode_changes_client(struct Client *client, struct Channel *channel)
 
     if (arg)
     {
-      int len = sprintf(parptr, (pbl == 0) ? "%s" : " %s", arg);
+      int len = sprintf(parptr, pbl == 0 ? "%s" : " %s", arg);
       pbl += len;
       parptr += len;
       ++paracount;
@@ -986,17 +972,15 @@ channel_mode_init(void)
       strlcat(cmode_class[cmode->class], str, sizeof(cmode_class[cmode->class]));
   }
 
-  char buf[IRCD_BUFSIZE];
-  snprintf(buf, sizeof(buf), "%s,%s,%s,%s", cmode_class[MODE_CLASS_A],
-           cmode_class[MODE_CLASS_B],
-           cmode_class[MODE_CLASS_C],
-           cmode_class[MODE_CLASS_D]);
-  isupport_add("CHANMODES", buf, -1);
+  isupport_add("CHANMODES", "%s,%s,%s,%s",
+               cmode_class[MODE_CLASS_A], cmode_class[MODE_CLASS_B],
+               cmode_class[MODE_CLASS_C], cmode_class[MODE_CLASS_D]);
 
-
-  snprintf(buf, sizeof(buf), "(%s)%s", letter, prefix);
-  isupport_add("PREFIX", buf, -1);
-  isupport_add("STATUSMSG", prefix, -1);
+  isupport_add("PREFIX", "(%s)%s", letter, prefix);
+  isupport_add("STATUSMSG", "%s", prefix);
+  isupport_add("MODES", "%d", MAXMODEPARAMS);
+  isupport_add("EXCEPTS", NULL);
+  isupport_add("INVEX", NULL);
 
   capab_add("QOP", CAPAB_QOP, ConfigChannel.enable_owner != 0);
   capab_add("AOP", CAPAB_AOP, ConfigChannel.enable_admin != 0);
@@ -1037,9 +1021,6 @@ channel_mode_set(struct Client *client, struct Channel *channel, int parc, char
       case '-':
         dir = MODE_DEL;
         break;
-      case '=':
-        dir = MODE_QUERY;
-        break;
       default:
       {
         const struct chan_mode *mode = cmode_map[(unsigned char)*ml];
