@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2015-2022 ircd-hybrid development team
+ *  Copyright (c) 2015-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,56 +28,54 @@
 #include "client_svstag.h"
 #include "memory.h"
 #include "numeric.h"
-#include "user.h"
-
+#include "user_mode.h"
 
 static void
-svstag_free(struct ServicesTag *svstag, dlink_list *list)
+svstag_free(struct ServicesTag *svstag, list_t *list)
 {
-  dlinkDelete(&svstag->node, list);
-  xfree(svstag->tag);
-  xfree(svstag);
+  list_remove(&svstag->node, list);
+  io_free(svstag->tag);
+  io_free(svstag);
 }
 
 void
-svstag_detach(dlink_list *list, unsigned int numeric)
+svstag_detach(list_t *list, unsigned int numeric)
 {
-  dlink_node *node, *node_next;
+  list_node_t *node, *node_next;
 
-  DLINK_FOREACH_SAFE(node, node_next, list->head)
+  LIST_FOREACH_SAFE(node, node_next, list->head)
   {
     struct ServicesTag *svstag = node->data;
-
     if (svstag->numeric == numeric)
       svstag_free(svstag, list);
   }
 }
 
 void
-svstag_attach(dlink_list *list, unsigned int numeric,
-              const char *umodes, const char *tag)
+svstag_attach(list_t *list, unsigned int numeric, const char *umodes, const char *tag)
 {
-  const struct user_modes *tab = NULL;
-
   if (numeric >= ERR_LAST_ERR_MSG || *umodes != '+')
     return;
 
-  struct ServicesTag *svstag = xcalloc(sizeof(*svstag));
+  struct ServicesTag *svstag = io_calloc(sizeof(*svstag));
   svstag->numeric = numeric;
-  svstag->tag = xstrdup(tag);
+  svstag->tag = io_strdup(tag);
 
   for (const char *m = umodes + 1  /* + 1 to skip the '+' */; *m; ++m)
-    if ((tab = umode_map[(unsigned char)*m]))
-      svstag->umodes |= tab->flag;
+  {
+    const struct UserMode *mode = user_mode_find(*m);
+    if (mode)
+      svstag->umodes |= mode->mode_bit;
+  }
 
   if (numeric != RPL_WHOISOPERATOR)
-    dlinkAddTail(svstag, &svstag->node, list);
+    list_add_tail(svstag, &svstag->node, list);
   else
-    dlinkAdd(svstag, &svstag->node, list);
+    list_add(svstag, &svstag->node, list);
 }
 
 void
-svstag_clear_list(dlink_list *list)
+svstag_clear_list(list_t *list)
 {
   while (list->head)
     svstag_free(list->head->data, list);
