@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,21 +24,18 @@
  */
 
 #include "stdinc.h"
+#include "defaults.h"
 #include "list.h"
-#include "ircd.h"
 #include "conf.h"
-#include "hostmask.h"
 #include "irc_string.h"
 #include "memory.h"
 #include "patricia.h"
 
+static list_t class_list;
 
 struct ClassItem *class_default;
 
-static dlink_list class_list;
-
-
-const dlink_list *
+const list_t *
 class_get_list(void)
 {
   return &class_list;
@@ -47,8 +44,7 @@ class_get_list(void)
 struct ClassItem *
 class_make(void)
 {
-  struct ClassItem *class = xcalloc(sizeof(*class));
-
+  struct ClassItem *class = io_calloc(sizeof(*class));
   class->active = true;
   class->con_freq = DEFAULT_CONNECTFREQUENCY;
   class->ping_freq = DEFAULT_PINGFREQUENCY;
@@ -56,8 +52,7 @@ class_make(void)
   class->max_recvq = DEFAULT_RECVQ;
   class->ip_tree_v6 = patricia_new(128);
   class->ip_tree_v4 = patricia_new( 32);
-
-  dlinkAdd(class, &class->node, &class_list);
+  list_add(class, &class->node, &class_list);
 
   return class;
 }
@@ -74,22 +69,22 @@ class_free(struct ClassItem *const class)
   if (class->ip_tree_v4)
     patricia_destroy(class->ip_tree_v4, NULL);
 
-  dlinkDelete(&class->node, &class_list);
-  xfree(class->name);
-  xfree(class);
+  list_remove(&class->node, &class_list);
+  io_free(class->name);
+  io_free(class);
 }
 
 void
 class_init(void)
 {
   class_default = class_make();
-  class_default->name = xstrdup("default");
+  class_default->name = io_strdup("default");
 }
 
 const struct ClassItem *
-class_get_ptr(const dlink_list *const list)
+class_get_ptr(const list_t *const list)
 {
-  const dlink_node *const node = list->head;
+  const list_node_t *const node = list->head;
 
   if (node)
   {
@@ -105,25 +100,25 @@ class_get_ptr(const dlink_list *const list)
 }
 
 const char *
-class_get_name(const dlink_list *const list)
+class_get_name(const list_t *const list)
 {
   return class_get_ptr(list)->name;
 }
 
 unsigned int
-class_get_ping_freq(const dlink_list *const list)
+class_get_ping_freq(const list_t *const list)
 {
   return class_get_ptr(list)->ping_freq;
 }
 
 unsigned int
-class_get_sendq(const dlink_list *const list)
+class_get_sendq(const list_t *const list)
 {
   return class_get_ptr(list)->max_sendq;
 }
 
 unsigned int
-class_get_recvq(const dlink_list *const list)
+class_get_recvq(const list_t *const list)
 {
   return class_get_ptr(list)->max_recvq;
 }
@@ -136,15 +131,14 @@ class_get_recvq(const dlink_list *const list)
 struct ClassItem *
 class_find(const char *name, bool active)
 {
-  dlink_node *node;
+  list_node_t *node;
 
-  DLINK_FOREACH(node, class_list.head)
+  LIST_FOREACH(node, class_list.head)
   {
     struct ClassItem *class = node->data;
-
     if (irccmp(class->name, name) == 0)
     {
-      if (active == true && class->active == false)
+      if (active && class->active == false)
         return NULL;
 
       return class;
@@ -161,9 +155,9 @@ class_find(const char *name, bool active)
 void
 class_mark_for_deletion(void)
 {
-  dlink_node *node;
+  list_node_t *node;
 
-  DLINK_FOREACH_PREV(node, class_list.tail->prev)
+  LIST_FOREACH_PREV(node, class_list.tail->prev)
   {
     struct ClassItem *class = node->data;
     class->active = false;
@@ -173,12 +167,11 @@ class_mark_for_deletion(void)
 void
 class_delete_marked(void)
 {
-  dlink_node *node, *node_next;
+  list_node_t *node, *node_next;
 
-  DLINK_FOREACH_SAFE(node, node_next, class_list.head)
+  LIST_FOREACH_SAFE(node, node_next, class_list.head)
   {
     struct ClassItem *class = node->data;
-
     if (class->active == false && class->ref_count == 0)
       class_free(class);
   }
@@ -189,8 +182,8 @@ class_ip_limit_trie(struct ClassItem *class, void *addr)
 {
   if (((struct sockaddr *)addr)->sa_family == AF_INET6)
     return class->ip_tree_v6;
-  else
-    return class->ip_tree_v4;
+
+  return class->ip_tree_v4;
 }
 
 bool
@@ -209,7 +202,7 @@ class_ip_limit_add(struct ClassItem *class, void *addr, bool over_rule)
   patricia_node_t *pnode = patricia_make_and_lookup_addr(class_ip_limit_trie(class, addr), addr, bitlen);
   if (((uintptr_t)pnode->data) >= class->number_per_cidr)
   {
-    if (over_rule == true)
+    if (over_rule)
       /*
        * In case of overruling, we continue with the client registration process
        * which means we expect a class_ip_limit_remove() call when detaching the
@@ -256,18 +249,17 @@ class_ip_limit_remove(struct ClassItem *class, void *addr)
 void
 class_ip_limit_rebuild(struct ClassItem *class)
 {
-  dlink_node *node;
-
   patricia_clear(class->ip_tree_v6, NULL);
   patricia_clear(class->ip_tree_v4, NULL);
 
-  DLINK_FOREACH(node, local_client_list.head)
+  list_node_t *node;
+  LIST_FOREACH(node, local_client_list.head)
   {
     struct Client *client = node->data;
     struct MaskItem *conf = client->connection->confs.tail->data;
 
     if (conf->type == CONF_CLIENT)
       if (conf->class == class)
-        class_ip_limit_add(class, &client->ip, true);
+        class_ip_limit_add(class, &client->addr, true);
   }
 }
