@@ -2,7 +2,7 @@
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
  *  Copyright (C) 1996-2009 by Andrew Church <achurch@achurch.org>
- *  Copyright (c) 2012-2022 ircd-hybrid development team
+ *  Copyright (c) 2012-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
  * \brief Includes file utilities for database handling
  */
 
-
 #include "stdinc.h"
 #include "conf.h"
 #include "conf_db.h"
@@ -34,8 +33,6 @@
 #include "log.h"
 #include "send.h"
 #include "irc_string.h"
-#include "hostmask.h"
-
 
 /*! \brief Return the version number on the file.  Return 0 if there is no version
  * number or the number doesn't make sense (i.e. less than 1 or greater
@@ -51,15 +48,15 @@ get_file_version(struct dbFILE *f)
 
   if (read_uint32(&version, f) == false)
   {
-    ilog(LOG_TYPE_IRCD, "Error reading version number on %s: %s",
-         f->filename, strerror(errno));
+    log_write(LOG_TYPE_IRCD, "Error reading version number on %s: %s",
+              f->filename, strerror(errno));
     return 0;
   }
 
   if (version < 1)
   {
-    ilog(LOG_TYPE_IRCD, "Invalid version number (%u) on %s",
-         version, f->filename);
+    log_write(LOG_TYPE_IRCD, "Invalid version number (%u) on %s",
+              version, f->filename);
     return 0;
   }
 
@@ -76,8 +73,8 @@ write_file_version(struct dbFILE *f, uint32_t version)
 {
   if (write_uint32(version, f) == false)
   {
-    ilog(LOG_TYPE_IRCD, "Error writing version number on %s",
-         f->filename);
+    log_write(LOG_TYPE_IRCD, "Error writing version number on %s",
+              f->filename);
     return false;
   }
 
@@ -91,7 +88,7 @@ write_file_version(struct dbFILE *f, uint32_t version)
 static struct dbFILE *
 open_db_read(const char *filename)
 {
-  struct dbFILE *f = xcalloc(sizeof(*f));
+  struct dbFILE *f = io_calloc(sizeof(*f));
 
   strlcpy(f->filename, filename, sizeof(f->filename));
 
@@ -103,9 +100,9 @@ open_db_read(const char *filename)
     int errno_save = errno;
 
     if (errno != ENOENT)
-      ilog(LOG_TYPE_IRCD, "Cannot read database file %s", f->filename);
+      log_write(LOG_TYPE_IRCD, "Cannot read database file %s", f->filename);
 
-    xfree(f);
+    io_free(f);
     errno = errno_save;
     return NULL;
   }
@@ -121,7 +118,7 @@ open_db_read(const char *filename)
 static struct dbFILE *
 open_db_write(const char *filename, uint32_t version)
 {
-  struct dbFILE *f = xcalloc(sizeof(*f));
+  struct dbFILE *f = io_calloc(sizeof(*f));
 
   strlcpy(f->filename, filename, sizeof(f->filename));
 
@@ -132,9 +129,9 @@ open_db_write(const char *filename, uint32_t version)
 
   if (f->tempname[0] == '\0' || strcmp(f->tempname, filename) == 0)
   {
-    ilog(LOG_TYPE_IRCD, "Opening database file %s for write: Filename too long",
-         filename);
-    xfree(f);
+    log_write(LOG_TYPE_IRCD, "Opening database file %s for write: Filename too long",
+              filename);
+    io_free(f);
     errno = ENAMETOOLONG;
     return NULL;
   }
@@ -157,20 +154,20 @@ open_db_write(const char *filename, uint32_t version)
     if (walloped == false)
     {
       walloped = true;
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                           "Cannot create temporary database file %s",
-                           f->tempname);
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                     "Cannot create temporary database file %s",
+                     f->tempname);
     }
 
     errno = errno_save;
-    ilog(LOG_TYPE_IRCD, "Cannot create temporary database file %s",
-        f->tempname);
+    log_write(LOG_TYPE_IRCD, "Cannot create temporary database file %s",
+              f->tempname);
 
     if (f->fp)
       fclose(f->fp);
 
     remove(f->tempname);
-    xfree(f);
+    io_free(f);
 
     errno = errno_save;
     return NULL;
@@ -199,10 +196,8 @@ open_db(const char *filename, const char *mode, uint32_t version)
   {
     case 'r':
       return open_db_read(filename);
-      break;
     case 'w':
       return open_db_write(filename, version);
-      break;
     default:
       errno = EINVAL;
       return NULL;
@@ -226,7 +221,7 @@ restore_db(struct dbFILE *f)
   if (f->mode == 'w' && f->tempname[0])
     remove(f->tempname);
 
-  xfree(f);
+  io_free(f);
   errno = errno_save;
 }
 
@@ -258,17 +253,17 @@ close_db(struct dbFILE *f)
     {
       int errno_save = errno;
 
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE, "Unable to move new "
-                           "data to database file %s; new data NOT saved.",
-                           f->filename);
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Unable to move new "
+                     "data to database file %s; new data NOT saved.",
+                     f->filename);
       errno = errno_save;
-      ilog(LOG_TYPE_IRCD, "Unable to move new data to database file %s; new "
-           "data NOT saved.", f->filename);
+      log_write(LOG_TYPE_IRCD, "Unable to move new data to database file %s; new data NOT saved.",
+                f->filename);
       remove(f->tempname);
     }
   }
 
-  xfree(f);
+  io_free(f);
   return true;
 }
 
@@ -281,6 +276,27 @@ close_db(struct dbFILE *f)
  * null-terminated.
  */
 
+bool
+read_bool(bool *ret, struct dbFILE *f)
+{
+  int c = fgetc(f->fp);
+
+  if (c == EOF)
+    return false;
+
+  *ret = (c != 0);
+  return true;
+}
+
+bool
+write_bool(bool val, struct dbFILE *f)
+{
+  if (fputc(val ? 1 : 0, f->fp) == EOF)
+    return false;
+
+  return true;
+}
+
 /*! \brief Read a unsigned 8bit integer
  *
  * \param ret 16bit integer to read
@@ -385,7 +401,6 @@ read_uint64(uint64_t *ret, struct dbFILE *f)
   return true;
 }
 
-
 /*! \brief Write a unsigned 64bit integer
  *
  * \param val unsigned 64bit integer to write
@@ -434,10 +449,10 @@ read_string(char **ret, struct dbFILE *f)
     return true;
   }
 
-  char *s = xcalloc(len);
+  char *s = io_calloc(len);
   if (len != fread(s, 1, len, f->fp))
   {
-    xfree(s);
+    io_free(s);
     return false;
   }
 
@@ -479,7 +494,7 @@ write_string(const char *s, struct dbFILE *f)
 #define SAFE_WRITE(x,db) do {                         \
     if ((x) == false) {                               \
         restore_db(f);                                \
-        ilog(LOG_TYPE_IRCD, "Write error on %s", db); \
+        log_write(LOG_TYPE_IRCD, "Write error on %s", db); \
         return;                                       \
     }                                                 \
 } while (false)
@@ -490,14 +505,14 @@ save_kline_database(const char *filename)
   uint32_t i = 0;
   uint32_t records = 0;
   struct dbFILE *f = NULL;
-  dlink_node *ptr = NULL;
+  list_node_t *ptr = NULL;
 
   if ((f = open_db(filename, "w", KLINE_DB_VERSION)) == NULL)
     return;
 
-  for (i = 0; i < ATABLE_SIZE; ++i)
+  for (i = 0; i < ADDRESS_HASHSIZE; ++i)
   {
-    DLINK_FOREACH(ptr, atable[i].head)
+    LIST_FOREACH(ptr, atable[i].head)
     {
       struct AddressRec *arec = ptr->data;
 
@@ -508,9 +523,9 @@ save_kline_database(const char *filename)
 
   SAFE_WRITE(write_uint32(records, f), filename);
 
-  for (i = 0; i < ATABLE_SIZE; ++i)
+  for (i = 0; i < ADDRESS_HASHSIZE; ++i)
   {
-    DLINK_FOREACH(ptr, atable[i].head)
+    LIST_FOREACH(ptr, atable[i].head)
     {
       struct AddressRec *arec = ptr->data;
 
@@ -580,14 +595,14 @@ save_dline_database(const char *filename)
   uint32_t i = 0;
   uint32_t records = 0;
   struct dbFILE *f = NULL;
-  dlink_node *ptr = NULL;
+  list_node_t *ptr = NULL;
 
   if ((f = open_db(filename, "w", KLINE_DB_VERSION)) == NULL)
     return;
 
-  for (i = 0; i < ATABLE_SIZE; ++i)
+  for (i = 0; i < ADDRESS_HASHSIZE; ++i)
   {
-    DLINK_FOREACH(ptr, atable[i].head)
+    LIST_FOREACH(ptr, atable[i].head)
     {
       struct AddressRec *arec = ptr->data;
 
@@ -598,9 +613,9 @@ save_dline_database(const char *filename)
 
   SAFE_WRITE(write_uint32(records, f), filename);
 
-  for (i = 0; i < ATABLE_SIZE; ++i)
+  for (i = 0; i < ADDRESS_HASHSIZE; ++i)
   {
-    DLINK_FOREACH(ptr, atable[i].head)
+    LIST_FOREACH(ptr, atable[i].head)
     {
       struct AddressRec *arec = ptr->data;
 
@@ -665,31 +680,31 @@ save_resv_database(const char *filename)
 {
   uint32_t records = 0;
   struct dbFILE *f = NULL;
-  dlink_node *node = NULL;
+  list_node_t *node = NULL;
   const struct ResvItem *resv = NULL;
 
   if ((f = open_db(filename, "w", KLINE_DB_VERSION)) == NULL)
     return;
 
-  DLINK_FOREACH(node, resv_chan_get_list()->head)
+  LIST_FOREACH(node, resv_chan_get_list()->head)
   {
     resv = node->data;
 
-    if (resv->in_database == true)
+    if (resv->in_database)
       ++records;
   }
 
-  DLINK_FOREACH(node, resv_nick_get_list()->head)
+  LIST_FOREACH(node, resv_nick_get_list()->head)
   {
     resv = node->data;
 
-    if (resv->in_database == true)
+    if (resv->in_database)
       ++records;
   }
 
   SAFE_WRITE(write_uint32(records, f), filename);
 
-  DLINK_FOREACH(node, resv_chan_get_list()->head)
+  LIST_FOREACH(node, resv_chan_get_list()->head)
   {
     resv = node->data;
 
@@ -702,7 +717,7 @@ save_resv_database(const char *filename)
     SAFE_WRITE(write_uint64(resv->expire, f), filename);
   }
 
-  DLINK_FOREACH(node, resv_nick_get_list()->head)
+  LIST_FOREACH(node, resv_nick_get_list()->head)
   {
     resv = node->data;
 
@@ -752,8 +767,8 @@ load_resv_database(const char *filename)
     resv->expire = tmp64_hold;
     resv->in_database = true;
 
-    xfree(name);
-    xfree(reason);
+    io_free(name);
+    io_free(reason);
   }
 
   close_db(f);
@@ -764,23 +779,23 @@ save_xline_database(const char *filename)
 {
   uint32_t records = 0;
   struct dbFILE *f = NULL;
-  dlink_node *ptr = NULL;
+  list_node_t *ptr = NULL;
   struct GecosItem *gecos = NULL;
 
   if ((f = open_db(filename, "w", KLINE_DB_VERSION)) == NULL)
     return;
 
-  DLINK_FOREACH(ptr, gecos_get_list()->head)
+  LIST_FOREACH(ptr, gecos_get_list()->head)
   {
     gecos = ptr->data;
 
-    if (gecos->in_database == true)
+    if (gecos->in_database)
       ++records;
   }
 
   SAFE_WRITE(write_uint32(records, f), filename);
 
-  DLINK_FOREACH(ptr, gecos_get_list()->head)
+  LIST_FOREACH(ptr, gecos_get_list()->head)
   {
     gecos = ptr->data;
 
