@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1998-2022 ircd-hybrid development team
+ *  Copyright (c) 1998-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,19 +24,16 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "list.h"
 #include "send.h"
 #include "client.h"
-#include "ircd.h"
 #include "memory.h"
-#include "conf.h"
 #include "conf_gecos.h"
 
+static list_t gecos_list;
 
-static dlink_list gecos_list;
-
-
-const dlink_list *
+const list_t *
 gecos_get_list(void)
 {
   return &gecos_list;
@@ -45,12 +42,11 @@ gecos_get_list(void)
 void
 gecos_clear(void)
 {
-  dlink_node *node, *node_next;
+  list_node_t *node, *node_next;
 
-  DLINK_FOREACH_SAFE(node, node_next, gecos_list.head)
+  LIST_FOREACH_SAFE(node, node_next, gecos_list.head)
   {
     struct GecosItem *gecos = node->data;
-
     if (gecos->in_database == false)
       gecos_delete(gecos, false);
   }
@@ -59,21 +55,21 @@ gecos_clear(void)
 void
 gecos_delete(struct GecosItem *gecos, bool expired)
 {
-  if (expired == true)
-    sendto_realops_flags(UMODE_EXPIRATION, L_ALL, SEND_NOTICE, "Temporary X-line for [%s] expired",
-                         gecos->mask);
+  if (expired)
+    sendto_clients(UMODE_EXPIRATION, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Temporary X-line for [%s] expired",
+                   gecos->mask);
 
-  dlinkDelete(&gecos->node, &gecos_list);
-  xfree(gecos->mask);
-  xfree(gecos->reason);
-  xfree(gecos);
+  list_remove(&gecos->node, &gecos_list);
+  io_free(gecos->mask);
+  io_free(gecos->reason);
+  io_free(gecos);
 }
 
 struct GecosItem *
 gecos_make(void)
 {
-  struct GecosItem *gecos = xcalloc(sizeof(*gecos));
-  dlinkAdd(gecos, &gecos->node, &gecos_list);
+  struct GecosItem *gecos = io_calloc(sizeof(*gecos));
+  list_add(gecos, &gecos->node, &gecos_list);
 
   return gecos;
 }
@@ -81,14 +77,13 @@ gecos_make(void)
 struct GecosItem *
 gecos_find(const char *name, int (*compare)(const char *, const char *))
 {
-  dlink_node *node, *node_next;
+  list_node_t *node, *node_next;
 
-  DLINK_FOREACH_SAFE(node, node_next, gecos_list.head)
+  LIST_FOREACH_SAFE(node, node_next, gecos_list.head)
   {
     struct GecosItem *gecos = node->data;
-
     if (gecos->expire &&
-        (gecos->expire <= event_base->time.sec_real))
+        (gecos->expire <= io_time_get(IO_TIME_REALTIME_SEC)))
       gecos_delete(gecos, true);
     else if (compare(gecos->mask, name) == 0)
       return gecos;
@@ -100,14 +95,13 @@ gecos_find(const char *name, int (*compare)(const char *, const char *))
 void
 gecos_expire(void)
 {
-  dlink_node *node, *node_next;
+  list_node_t *node, *node_next;
 
-  DLINK_FOREACH_SAFE(node, node_next, gecos_list.head)
+  LIST_FOREACH_SAFE(node, node_next, gecos_list.head)
   {
     struct GecosItem *gecos = node->data;
-
     if (gecos->expire &&
-        (gecos->expire <= event_base->time.sec_real))
+        (gecos->expire <= io_time_get(IO_TIME_REALTIME_SEC)))
       gecos_delete(gecos, true);
   }
 }
