@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2001-2022 ircd-hybrid development team
+ *  Copyright (c) 2001-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,30 +24,30 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
+#include "irc_string.h"
 #include "list.h"
 #include "send.h"
 #include "client.h"
 #include "memory.h"
 #include "ircd.h"
-#include "irc_string.h"
 #include "ircd_defs.h"
 #include "misc.h"
+#include "nuh.h"
 #include "conf.h"
 #include "conf_resv.h"
-#include "hostmask.h"
+#include "address.h"
 
+static list_t resv_chan_list;
+static list_t resv_nick_list;
 
-static dlink_list resv_chan_list;
-static dlink_list resv_nick_list;
-
-
-const dlink_list *
+const list_t *
 resv_chan_get_list(void)
 {
   return &resv_chan_list;
 }
 
-const dlink_list *
+const list_t *
 resv_nick_get_list(void)
 {
   return &resv_nick_list;
@@ -56,25 +56,24 @@ resv_nick_get_list(void)
 void
 resv_delete(struct ResvItem *resv, bool expired)
 {
-  if (expired == true)
-    sendto_realops_flags(UMODE_EXPIRATION, L_ALL, SEND_NOTICE, "Temporary RESV for [%s] expired",
-                         resv->mask);
+  if (expired)
+    sendto_clients(UMODE_EXPIRATION, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Temporary RESV for [%s] expired",
+                   resv->mask);
 
   while (resv->exempt_list.head)
   {
     struct ResvExemptItem *exempt = resv->exempt_list.head->data;
-
-    dlinkDelete(&exempt->node, &resv->exempt_list);
-    xfree(exempt->name);
-    xfree(exempt->user);
-    xfree(exempt->host);
-    xfree(exempt);
+    list_remove(&exempt->node, &resv->exempt_list);
+    io_free(exempt->name);
+    io_free(exempt->user);
+    io_free(exempt->host);
+    io_free(exempt);
   }
 
-  dlinkDelete(&resv->node, resv->list);
-  xfree(resv->mask);
-  xfree(resv->reason);
-  xfree(resv);
+  list_remove(&resv->node, resv->list);
+  io_free(resv->mask);
+  io_free(resv->reason);
+  io_free(resv);
 }
 
 /* create_resv()
@@ -86,50 +85,50 @@ resv_delete(struct ResvItem *resv, bool expired)
  * side effects	-
  */
 struct ResvItem *
-resv_make(const char *mask, const char *reason, const dlink_list *elist)
+resv_make(const char *mask, const char *reason, const list_t *elist)
 {
-  dlink_list *list;
+  list_t *list;
 
   if (IsChanPrefix(*mask))
     list = &resv_chan_list;
   else
     list = &resv_nick_list;
 
-  struct ResvItem *resv = xcalloc(sizeof(*resv));
+  struct ResvItem *resv = io_calloc(sizeof(*resv));
   resv->list = list;
-  resv->mask = xstrdup(mask);
-  resv->reason = xstrndup(reason, IRCD_MIN(strlen(reason), REASONLEN));
-  dlinkAdd(resv, &resv->node, resv->list);
+  resv->mask = io_strdup(mask);
+  resv->reason = io_strndup(reason, IO_MIN(strlen(reason), REASONLEN));
+  list_add(resv, &resv->node, resv->list);
 
   if (elist)
   {
-    dlink_node *node;
+    list_node_t *node;
 
-    DLINK_FOREACH(node, elist->head)
+    LIST_FOREACH(node, elist->head)
     {
+      char *s = node->data;
       char nick[NICKLEN + 1];
       char user[USERLEN + 1];
       char host[HOSTLEN + 1];
-      struct split_nuh_item nuh;
-      char *s = node->data;
+      struct nuh_split nuh =
+      {
+        .nuhmask = s,
+        .nickptr = nick,
+        .userptr = user,
+        .hostptr = host,
+        .nicksize = sizeof(nick),
+        .usersize = sizeof(user),
+        .hostsize = sizeof(host)
+      };
 
-      nuh.nuhmask  = s;
-      nuh.nickptr  = nick;
-      nuh.userptr  = user;
-      nuh.hostptr  = host;
+      nuh_split(&nuh);
 
-      nuh.nicksize = sizeof(nick);
-      nuh.usersize = sizeof(user);
-      nuh.hostsize = sizeof(host);
-
-      split_nuh(&nuh);
-
-      struct ResvExemptItem *exempt = xcalloc(sizeof(*exempt));
-      exempt->name = xstrdup(nick);
-      exempt->user = xstrdup(user);
-      exempt->host = xstrdup(host);
+      struct ResvExemptItem *exempt = io_calloc(sizeof(*exempt));
+      exempt->name = io_strdup(nick);
+      exempt->user = io_strdup(user);
+      exempt->host = io_strdup(host);
       exempt->type = parse_netmask(host, &exempt->addr, &exempt->bits);
-      dlinkAdd(exempt, &exempt->node, &resv->exempt_list);
+      list_add(exempt, &exempt->node, &resv->exempt_list);
     }
   }
 
@@ -139,20 +138,19 @@ resv_make(const char *mask, const char *reason, const dlink_list *elist)
 struct ResvItem *
 resv_find(const char *name, int (*compare)(const char *, const char *))
 {
-  dlink_node *node, *node_next;
-  dlink_list *list;
+  list_t *list;
 
   if (IsChanPrefix(*name))
     list = &resv_chan_list;
   else
     list = &resv_nick_list;
 
-  DLINK_FOREACH_SAFE(node, node_next, list->head)
+  list_node_t *node, *node_next;
+  LIST_FOREACH_SAFE(node, node_next, list->head)
   {
     struct ResvItem *resv = node->data;
-
     if (resv->expire &&
-        (resv->expire <= event_base->time.sec_real))
+        (resv->expire <= io_time_get(IO_TIME_REALTIME_SEC)))
       resv_delete(resv, true);
     else if (compare(resv->mask, name) == 0)
       return resv;
@@ -164,12 +162,11 @@ resv_find(const char *name, int (*compare)(const char *, const char *))
 bool
 resv_exempt_find(const struct Client *client, const struct ResvItem *resv)
 {
-  dlink_node *node;
+  list_node_t *node;
 
-  DLINK_FOREACH(node, resv->exempt_list.head)
+  LIST_FOREACH(node, resv->exempt_list.head)
   {
     const struct ResvExemptItem *exempt = node->data;
-
     if (match(exempt->name, client->name) == 0 && match(exempt->user, client->username) == 0)
     {
       switch (exempt->type)
@@ -181,7 +178,7 @@ resv_exempt_find(const struct Client *client, const struct ResvItem *resv)
           break;
         case HM_IPV6:
         case HM_IPV4:
-          if (address_compare(&client->ip, &exempt->addr, false, false, exempt->bits) == true)
+          if (address_compare(&client->addr, &exempt->addr, false, false, exempt->bits))
             return true;
           break;
         default:
@@ -196,16 +193,15 @@ resv_exempt_find(const struct Client *client, const struct ResvItem *resv)
 void
 resv_clear(void)
 {
-  dlink_list *tab[] = { &resv_chan_list, &resv_nick_list, NULL };
+  list_t *tab[] = { &resv_chan_list, &resv_nick_list, NULL };
 
-  for (dlink_list **list = tab; *list; ++list)
+  for (list_t **list = tab; *list; ++list)
   {
-    dlink_node *node, *node_next;
+    list_node_t *node, *node_next;
 
-    DLINK_FOREACH_SAFE(node, node_next, (*list)->head)
+    LIST_FOREACH_SAFE(node, node_next, (*list)->head)
     {
       struct ResvItem *resv = node->data;
-
       if (resv->in_database == false)
         resv_delete(resv, false);
     }
@@ -215,18 +211,17 @@ resv_clear(void)
 void
 resv_expire(void)
 {
-  dlink_list *tab[] = { &resv_chan_list, &resv_nick_list, NULL };
+  list_t *tab[] = { &resv_chan_list, &resv_nick_list, NULL };
 
-  for (dlink_list **list = tab; *list; ++list)
+  for (list_t **list = tab; *list; ++list)
   {
-    dlink_node *node, *node_next;
+    list_node_t *node, *node_next;
 
-    DLINK_FOREACH_SAFE(node, node_next, (*list)->head)
+    LIST_FOREACH_SAFE(node, node_next, (*list)->head)
     {
       struct ResvItem *resv = node->data;
-
       if (resv->expire &&
-          (resv->expire <= event_base->time.sec_real))
+          (resv->expire <= io_time_get(IO_TIME_REALTIME_SEC)))
         resv_delete(resv, true);
     }
   }
