@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,11 +29,9 @@
 #include "memory.h"
 #include "conf_shared.h"
 
+static list_t shared_list;
 
-static dlink_list shared_list;
-
-
-const dlink_list *
+const list_t *
 shared_get_list(void)
 {
   return &shared_list;
@@ -45,38 +43,34 @@ shared_clear(void)
   while (shared_list.head)
   {
     struct SharedItem *shared = shared_list.head->data;
-
-    dlinkDelete(&shared->node, &shared_list);
-    xfree(shared->server);
-    xfree(shared->user);
-    xfree(shared->host);
-    xfree(shared);
+    list_remove(&shared->node, &shared_list);
+    io_free(shared->server);
+    io_free(shared->user);
+    io_free(shared->host);
+    io_free(shared);
   }
 }
 
 struct SharedItem *
 shared_make(void)
 {
-  struct SharedItem *shared = xcalloc(sizeof(*shared));
-  dlinkAdd(shared, &shared->node, &shared_list);
+  struct SharedItem *shared = io_calloc(sizeof(*shared));
+  list_add(shared, &shared->node, &shared_list);
 
   return shared;
 }
 
 const struct SharedItem *
-shared_find(unsigned int type, const char *server,
-            const char *user, const char *host)
+shared_find(unsigned int type, const char *server, const char *user, const char *host)
 {
-  dlink_node *node;
+  list_node_t *node;
 
-  DLINK_FOREACH(node, shared_list.head)
+  LIST_FOREACH(node, shared_list.head)
   {
     const struct SharedItem *shared = node->data;
-
     if (shared->type & type)
       if (match(shared->server, server) == 0)
-        if (match(shared->user, user) == 0 &&
-            match(shared->host, host) == 0)
+        if (match(shared->user, user) == 0 && match(shared->host, host) == 0)
           return shared;
   }
 
