@@ -2,7 +2,7 @@
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
  *  Copyright (c) 2015-2016 plexus development team
- *  Copyright (c) 2019-2022 ircd-hybrid development team
+ *  Copyright (c) 2019-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,26 +25,23 @@
  */
 
 #include "stdinc.h"
-#include "ircd_defs.h"
 #include "list.h"
 #include "irc_string.h"
-#include "user.h"
+#include "client.h"
 #include "isupport.h"
 #include "extban.h"
 
-
-static dlink_list extban_list;
+static list_t extban_list;
 static unsigned int matching_mask, acting_mask;
 
-
 static unsigned int
 extban_find_mask(void)
 {
-  dlink_node *node;
+  list_node_t *node;
   unsigned int used = 0;
   unsigned int i;
 
-  DLINK_FOREACH(node, extban_list.head)
+  LIST_FOREACH(node, extban_list.head)
   {
     const struct Extban *extban = node->data;
     used |= extban->flag;
@@ -74,19 +71,18 @@ extban_init(void)
 
   const char *ptr = extban_get_isupport();
   if (ptr)
-    isupport_add("EXTBAN", ptr, -1);
+    isupport_add("EXTBAN", "%s", ptr);
 }
 
 void
 extban_add(struct Extban *extban)
 {
   unsigned int mask = extban_find_mask();
-
   if (mask == 0)
     return;
 
   extban->flag = mask;
-  dlinkAdd(extban, &extban->node, &extban_list);
+  list_add(extban, &extban->node, &extban_list);
 
   if (extban->type & EXTBAN_MATCHING)
     matching_mask |= mask;
@@ -100,7 +96,7 @@ extban_del(struct Extban *extban)
   if (extban->flag == 0)
     return;
 
-  dlinkDelete(&extban->node, &extban_list);
+  list_remove(&extban->node, &extban_list);
 
   matching_mask &= ~extban->flag;
   acting_mask &= ~extban->flag;
@@ -109,12 +105,11 @@ extban_del(struct Extban *extban)
 struct Extban *
 extban_find(unsigned char c)
 {
-  dlink_node *node;
+  list_node_t *node;
 
-  DLINK_FOREACH(node, extban_list.head)
+  LIST_FOREACH(node, extban_list.head)
   {
     struct Extban *extban = node->data;
-
     if (extban->character == c)
       return extban;
   }
@@ -125,12 +120,11 @@ extban_find(unsigned char c)
 struct Extban *
 extban_find_flag(unsigned int flag)
 {
-  dlink_node *node;
+  list_node_t *node;
 
-  DLINK_FOREACH(node, extban_list.head)
+  LIST_FOREACH(node, extban_list.head)
   {
     struct Extban *extban = node->data;
-
     if (extban->flag == flag)
       return extban;
   }
@@ -188,13 +182,12 @@ extban_parse(const char *mask, unsigned int *input_extbans, unsigned int *offset
 size_t
 extban_format(unsigned int e, char *buf)
 {
-  dlink_node *node;
+  list_node_t *node;
   size_t written = 0;
 
-  DLINK_FOREACH(node, extban_list.head)
+  LIST_FOREACH(node, extban_list.head)
   {
     struct Extban *extban = node->data;
-
     if (extban->type != EXTBAN_ACTING)
       continue;
 
@@ -214,10 +207,9 @@ extban_format(unsigned int e, char *buf)
     }
   }
 
-  DLINK_FOREACH(node, extban_list.head)
+  LIST_FOREACH(node, extban_list.head)
   {
     struct Extban *extban = node->data;
-
     if (extban->type != EXTBAN_MATCHING)
       continue;
 
@@ -255,14 +247,14 @@ extban_acting_mask(void)
 const char *
 extban_get_isupport(void)
 {
-  dlink_node *node;
   char extban_chars[256] = { 0 };
   static char buf[sizeof(extban_chars) + 3 /* +3 = $,\0 */ ];
 
-  if (dlink_list_length(&extban_list) == 0)
+  if (list_is_empty(&extban_list))
     return NULL;
 
-  DLINK_FOREACH(node, extban_list.head)
+  list_node_t *node;
+  LIST_FOREACH(node, extban_list.head)
   {
     struct Extban *extban = node->data;
     extban_chars[extban->character] = extban->character;
