@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 2019-2022 ircd-hybrid development team
+ *  Copyright (c) 2019-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,44 +29,44 @@
 #include "channel_mode.h"
 #include "client.h"
 #include "extban.h"
-#include "user.h"
-
+#include "user_mode.h"
 
 static enum extban_match
 extban_usermode_matches(struct Client *client, struct Channel *channel, struct Ban *ban)
 {
-  int what = MODE_ADD;
-  unsigned int modes_del = 0, modes_add = 0;
+  user_mode_action_t action = USER_MODE_ACTION_ADD;
+  uint64_t modes_del = 0, modes_add = 0;
 
   for (const char *m = ban->host; *m; ++m)
   {
     switch (*m)
     {
       case '+':
-        what = MODE_ADD;
+        action = USER_MODE_ACTION_ADD;
         break;
       case '-':
-        what = MODE_DEL;
+        action = USER_MODE_ACTION_DEL;
         break;
       default:
       {
-        const struct user_modes *tab = umode_map[(unsigned char)*m];
-
-        switch (what)
+        const struct UserMode *mode = user_mode_find(*m);
+        if (mode)
         {
-          case MODE_DEL:
-            if (tab)
-              modes_del |= tab->flag;
-            break;
-          case MODE_ADD:
-            if (tab)
-              modes_add |= tab->flag;
-            break;
+          switch (action)
+          {
+            case USER_MODE_ACTION_ADD:
+              modes_add |= mode->mode_bit;
+              break;
+            case USER_MODE_ACTION_DEL:
+              modes_del |= mode->mode_bit;
+              break;
+          }
         }
       }
     }
   }
 
+
   if ((client->umodes & modes_add) == modes_add &&
       (client->umodes & modes_del) == 0)
     return EXTBAN_MATCH;
