@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,20 +24,18 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "list.h"
 #include "ipcache.h"
 #include "event.h"
 #include "memory.h"
 #include "conf.h"
-#include "ircd.h"
 #include "patricia.h"
 
-
-static dlink_list ipcache_list;
+static list_t ipcache_list;
 static patricia_tree_t *ipcache_trie_v6;
 static patricia_tree_t *ipcache_trie_v4;
 
-
 static void *
 ipcache_get_trie(void *addr)
 {
@@ -49,7 +47,7 @@ ipcache_get_trie(void *addr)
 
 /* ipcache_find_or_add_address()
  *
- * inputs       - pointer to struct irc_ssaddr
+ * inputs       - pointer to struct io_addr
  * output       - pointer to a struct ip_entry
  * side effects -
  *
@@ -65,9 +63,9 @@ ipcache_record_find_or_add(void *addr)
   if (pnode->data)  /* Deliberate crash if 'pnode' is NULL */
     return pnode->data;  /* Already added to the trie */
 
-  struct ip_entry *iptr = xcalloc(sizeof(*iptr));
+  struct ip_entry *iptr = io_calloc(sizeof(*iptr));
   iptr->trie_pointer = ptrie;
-  dlinkAdd(pnode, &iptr->node, &ipcache_list);
+  list_add(pnode, &iptr->node, &ipcache_list);
 
   PATRICIA_DATA_SET(pnode, iptr);
 
@@ -80,12 +78,12 @@ ipcache_record_delete(patricia_node_t *pnode)
   struct ip_entry *iptr = PATRICIA_DATA_GET(pnode, struct ip_entry);
 
   if (iptr->count_local == 0 && iptr->count_remote == 0 &&
-      (event_base->time.sec_monotonic - iptr->last_attempt) >= ConfigGeneral.throttle_time)
+      (io_time_get(IO_TIME_MONOTONIC_SEC) - iptr->last_attempt) >= ConfigGeneral.throttle_time)
   {
     patricia_remove(iptr->trie_pointer, pnode);
 
-    dlinkDelete(&iptr->node, &ipcache_list);
-    xfree(iptr);
+    list_remove(&iptr->node, &ipcache_list);
+    io_free(iptr);
   }
 }
 
@@ -102,14 +100,13 @@ void
 ipcache_record_remove(void *addr, bool local)
 {
   patricia_node_t *pnode = patricia_try_search_exact_addr(ipcache_get_trie(addr), addr, 0);
-
   if (pnode == NULL)
     return;
 
   struct ip_entry *iptr = PATRICIA_DATA_GET(pnode, struct ip_entry);
   assert(iptr->count_local > 0 || iptr->count_remote > 0);
 
-  if (local == true)
+  if (local)
     --iptr->count_local;
   else
     --iptr->count_remote;
@@ -126,9 +123,9 @@ ipcache_record_remove(void *addr, bool local)
 static void
 ipcache_remove_expired_records(void *unused)
 {
-  dlink_node *node, *node_next;
+  list_node_t *node, *node_next;
 
-  DLINK_FOREACH_SAFE(node, node_next, ipcache_list.head)
+  LIST_FOREACH_SAFE(node, node_next, ipcache_list.head)
     ipcache_record_delete(node->data);
 }
 
@@ -146,8 +143,8 @@ void
 ipcache_get_stats(unsigned int *const number_ips_stored, size_t *const mem_ips_stored)
 {
   /* TBD: inaccurate for now as it does only count the amount of memory for struct ip_entry items */
-  (*number_ips_stored) = dlink_list_length(&ipcache_list);
-  (*mem_ips_stored) = dlink_list_length(&ipcache_list) * sizeof(struct ip_entry);
+  (*number_ips_stored) = list_length(&ipcache_list);
+  (*mem_ips_stored) = list_length(&ipcache_list) * sizeof(struct ip_entry);
 }
 
 void
