@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1999-2022 ircd-hybrid development team
+ *  Copyright (c) 1999-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,14 +27,16 @@
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
 #include <netinet/tcp.h>
+#include "io_time.h"
 #include "list.h"
+#include "log.h"
 #include "listener.h"
 #include "client.h"
 #include "fdlist.h"
 #include "irc_string.h"
 #include "ircd.h"
 #include "ircd_defs.h"
-#include "s_bsd.h"
+#include "comm.h"
 #include "conf.h"
 #include "send.h"
 #include "memory.h"
@@ -43,22 +45,21 @@
 #define DLINE_WARNING "ERROR :You have been D-lined.\r\n"
 #define ALLINUSE_WARNING "ERROR :All connections in use\r\n"
 
-static dlink_list listener_list;
+static list_t listener_list;
 
-const dlink_list *
+const list_t *
 listener_get_list(void)
 {
   return &listener_list;
 }
 
 static struct Listener *
-listener_make(const int port, const struct irc_ssaddr *addr)
+listener_make(const int port, const struct io_addr *addr)
 {
-  struct Listener *listener = xcalloc(sizeof(*listener));
-
+  struct Listener *listener = io_calloc(sizeof(*listener));
   listener->port = port;
   listener->addr = *addr;
-  dlinkAdd(listener, &listener->node, &listener_list);
+  list_add(listener, &listener->node, &listener_list);
 
   return listener;
 }
@@ -66,9 +67,9 @@ listener_make(const int port, const struct irc_ssaddr *addr)
 static void
 listener_free(struct Listener *listener)
 {
-  dlinkDelete(&listener->node, &listener_list);
-  xfree(listener->name);
-  xfree(listener);
+  list_remove(&listener->node, &listener_list);
+  io_free(listener->name);
+  io_free(listener);
 }
 
 /*
@@ -96,18 +97,116 @@ listener_has_flag(const struct Listener *listener, unsigned int flags)
 void
 listener_count_memory(unsigned int *count, size_t *bytes)
 {
-  (*count) = dlink_list_length(&listener_list);
-  (*bytes) = *count * sizeof(struct Listener);
+  (*count) = list_length(&listener_list);
+  (*bytes) = list_length(&listener_list) * sizeof(struct Listener);
+}
+
+/*
+ * ssl_handshake - let OpenSSL initialize the protocol. Register for
+ * read/write events if necessary.
+ */
+static void
+ssl_handshake(fde_t *F, void *data_)
+{
+  struct Client *client = data_;
+
+  assert(client);
+  assert(client->connection);
+  assert(client->connection->fd);
+  assert(client->connection->fd == F);
+
+  tls_handshake_status_t ret = tls_handshake(&F->tls, TLS_ROLE_SERVER, NULL);
+  if (ret != TLS_HANDSHAKE_DONE)
+  {
+    if ((io_time_get(IO_TIME_MONOTONIC_SEC) - client->connection->created_monotonic) > TLS_HANDSHAKE_TIMEOUT)
+    {
+      client_exit(client, "Timeout during TLS handshake");
+      return;
+    }
+
+    switch (ret)
+    {
+      case TLS_HANDSHAKE_WANT_WRITE:
+        comm_setselect(F, COMM_SELECT_WRITE, ssl_handshake, client, TLS_HANDSHAKE_TIMEOUT);
+        return;
+      case TLS_HANDSHAKE_WANT_READ:
+        comm_setselect(F, COMM_SELECT_READ, ssl_handshake, client, TLS_HANDSHAKE_TIMEOUT);
+        return;
+      default:
+        client_exit(client, "Error during TLS handshake");
+        return;
+    }
+  }
+
+  comm_settimeout(F, 0, NULL, NULL);
+  comm_setselect(F, COMM_SELECT_WRITE | COMM_SELECT_READ, NULL, NULL, 0);
+
+  if (tls_verify_certificate(&F->tls, &client->tls_certfp) == false)
+    log_write(LOG_TYPE_IRCD, "Client %s gave bad TLS client certificate",
+              client_get_name(client, MASK_IP));
+
+  auth_start(client);
+}
+
+/*
+ * add_connection - creates a client which has just connected to us on
+ * the given fd. The sockhost field is initialized with the ip# of the host.
+ * An unique id is calculated now, in case it is needed for auth.
+ * The client is sent to the auth module for verification, and not put in
+ * any client list yet.
+ */
+static void
+add_connection(struct Listener *listener, struct io_addr *addr, int fd)
+{
+  struct Client *client = client_make(NULL);
+
+  client->connection->fd = fd_open(fd, true, listener_has_flag(listener, LISTENER_TLS) ?
+                                   "Incoming TLS connection" : "Incoming connection");
+
+  /*
+   * copy address to 'sockhost' as a string, copy it to host too
+   * so we have something valid to put into error messages...
+   */
+  client->addr = *addr;
+
+  getnameinfo((const struct sockaddr *)&client->addr,
+              client->addr.ss_len, client->sockhost,
+              sizeof(client->sockhost), NULL, 0, NI_NUMERICHOST);
+
+  if (client->sockhost[0] == ':' &&
+      client->sockhost[1] == ':')
+  {
+    memmove(client->sockhost + 1, client->sockhost, sizeof(client->sockhost) - 1);
+    client->sockhost[0] = '0';
+  }
+
+  strlcpy(client->host, client->sockhost, sizeof(client->host));
+
+  client->connection->listener = listener;
+  ++listener->ref_count;
+
+  if (listener_has_flag(listener, LISTENER_TLS))
+  {
+    if (tls_new(&client->connection->fd->tls, fd, TLS_ROLE_SERVER) == false)
+    {
+      SetDead(client);
+      client_exit(client, "TLS context initialization failed");
+      return;
+    }
+
+    AddFlag(client, FLAGS_TLS);
+    ssl_handshake(client->connection->fd, client);
+  }
+  else
+    auth_start(client);
 }
 
 static void
-listener_accept_connection(fde_t *F, void *data)
+listener_accept_connection(fde_t *F, void *data_)
 {
-  static uintmax_t rate = 0;
-  struct irc_ssaddr addr;
+  struct Listener *const listener = data_;
+  struct io_addr addr;
   int fd;
-  int pe;
-  struct Listener *const listener = data;
 
   assert(listener);
   assert(listener->fd == F);
@@ -132,21 +231,24 @@ listener_accept_connection(fde_t *F, void *data)
     if (number_fd > hard_fdlimit - 10)
     {
       ++ServerStats.is_ref;
-      sendto_realops_flags_ratelimited(&rate, "All connections in use. (%s)",
-                                       listener_get_name(listener));
+
+      static uintmax_t rate = 0;
+      sendto_clients_ratelimited(&rate, "All connections in use. (%s)",
+                                 listener_get_name(listener));
 
       if (!(listener->flags & LISTENER_TLS))
         send(fd, ALLINUSE_WARNING, sizeof(ALLINUSE_WARNING) - 1, 0);
 
       close(fd);
-      break;    /* jump out and re-register a new io request */
+      continue;  /* drop the one and keep on clearing the queue */
     }
 
     /*
      * Do an initial check we aren't connecting too fast or with too many
      * from this IP...
      */
-    if ((pe = conf_connect_allowed(&addr)))
+    int pe = conf_connect_allowed(&addr);
+    if (pe)
     {
       ++ServerStats.is_ref;
 
@@ -155,10 +257,10 @@ listener_accept_connection(fde_t *F, void *data)
         switch (pe)
         {
           case BANNED_CLIENT:
-            send(fd, DLINE_WARNING, sizeof(DLINE_WARNING)-1, 0);
+            send(fd, DLINE_WARNING, sizeof(DLINE_WARNING) - 1, 0);
             break;
           case TOO_FAST:
-            send(fd, TOOFAST_WARNING, sizeof(TOOFAST_WARNING)-1, 0);
+            send(fd, TOOFAST_WARNING, sizeof(TOOFAST_WARNING) - 1, 0);
             break;
         }
       }
@@ -172,29 +274,24 @@ listener_accept_connection(fde_t *F, void *data)
   }
 
   /* Re-register a new IO request for the next accept .. */
-  comm_setselect(listener->fd, COMM_SELECT_READ, listener_accept_connection,
-                 listener, 0);
+  comm_setselect(listener->fd, COMM_SELECT_READ, listener_accept_connection, listener, 0);
 }
 
-
-/*
- * inetport - create a listener socket in the AF_INET or AF_INET6 domain,
- * bind it to the port given in 'port' and listen to it
- * returns true (1) if successful false (0) on error.
+/**
+ * @var LISTEN_BACKLOG
+ * @brief Enumeration for the backlog argument in the listen() function.
  *
- * If the operating system has a define for SOMAXCONN, use it, otherwise
- * use HYBRID_SOMAXCONN
+ * This enum specifies the maximum length of the queue of pending connections
+ * that the listen() function can handle. When a connection request arrives
+ * and the queue is full, the request may be refused, depending on the system's
+ * behavior.
  */
-#ifdef SOMAXCONN
-#undef HYBRID_SOMAXCONN
-#define HYBRID_SOMAXCONN SOMAXCONN
-#endif
+enum { LISTEN_BACKLOG = 128 };
 
 static bool
 listener_finalize(struct Listener *listener)
 {
   char buf[HOSTIPLEN + 1];
-  const socklen_t opt = 1;
 
   getnameinfo((const struct sockaddr *)&listener->addr, listener->addr.ss_len,
               buf, sizeof(buf), NULL, 0, NI_NUMERICHOST);
@@ -205,7 +302,7 @@ listener_finalize(struct Listener *listener)
     buf[0] = '0';
   }
 
-  listener->name = xstrdup(buf);
+  listener->name = io_strdup(buf);
 
   /*
    * At first, open a new socket
@@ -213,19 +310,20 @@ listener_finalize(struct Listener *listener)
   int fd = comm_socket(listener->addr.ss.ss_family, SOCK_STREAM, 0);
   if (fd == -1)
   {
-    report_error(L_ALL, "opening listener socket %s:%s",
-                 listener_get_name(listener), errno);
+    log_write(LOG_TYPE_IRCD, "opening listener socket %s: %s",
+              listener_get_name(listener), strerror(errno));
     return false;
   }
 
+  const socklen_t opt = 1;
 #ifdef IPV6_V6ONLY
   if (listener->addr.ss.ss_family == AF_INET6 &&
       IN6_IS_ADDR_UNSPECIFIED(&((struct sockaddr_in6 *)&listener->addr)->sin6_addr))
   {
     if (setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, &opt, sizeof(opt)))
     {
-      report_error(L_ALL, "setting IPV6_V6ONLY for listener %s:%s",
-                   listener_get_name(listener), errno);
+      log_write(LOG_TYPE_IRCD, "setting IPV6_V6ONLY for listener %s: %s",
+                listener_get_name(listener), strerror(errno));
       close(fd);
       return false;
     }
@@ -234,8 +332,8 @@ listener_finalize(struct Listener *listener)
 
   if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt)))
   {
-    report_error(L_ALL, "setting SO_REUSEADDR for listener %s:%s",
-                 listener_get_name(listener), errno);
+    log_write(LOG_TYPE_IRCD, "setting SO_REUSEADDR for listener %s: %s",
+              listener_get_name(listener), strerror(errno));
     close(fd);
     return false;
   }
@@ -246,16 +344,16 @@ listener_finalize(struct Listener *listener)
    */
   if (bind(fd, (const struct sockaddr *)&listener->addr, listener->addr.ss_len))
   {
-    report_error(L_ALL, "binding listener socket %s:%s",
-                 listener_get_name(listener), errno);
+    log_write(LOG_TYPE_IRCD, "binding listener socket %s: %s",
+              listener_get_name(listener), strerror(errno));
     close(fd);
     return false;
   }
 
-  if (listen(fd, HYBRID_SOMAXCONN))
+  if (listen(fd, LISTEN_BACKLOG))
   {
-    report_error(L_ALL, "listen failed for %s:%s",
-                 listener_get_name(listener), errno);
+    log_write(LOG_TYPE_IRCD, "listen failed for %s: %s",
+              listener_get_name(listener), strerror(errno));
     close(fd);
     return false;
   }
@@ -264,10 +362,10 @@ listener_finalize(struct Listener *listener)
   if (listener_has_flag(listener, LISTENER_DEFER))
   {
     int timeout = 1;
-
     setsockopt(fd, IPPROTO_TCP, TCP_DEFER_ACCEPT, &timeout, sizeof(timeout));
   }
 #endif
+
 #ifdef SO_ACCEPTFILTER
   if (listener_has_flag(listener, LISTENER_DEFER))
   {
@@ -282,24 +380,20 @@ listener_finalize(struct Listener *listener)
   listener->fd = fd_open(fd, true, "Listener socket");
 
   /* Listen completion events are READ events .. */
-
   listener_accept_connection(listener->fd, listener);
   return true;
 }
 
 static struct Listener *
-listener_find(int port, struct irc_ssaddr *addr)
+listener_find(int port, struct io_addr *addr)
 {
-  dlink_node *node;
-  struct Listener *listener    = NULL;
   struct Listener *last_closed = NULL;
 
-  DLINK_FOREACH(node, listener_list.head)
+  list_node_t *node;
+  LIST_FOREACH(node, listener_list.head)
   {
-    listener = node->data;
-
-    if ((port == listener->port) &&
-        (!memcmp(addr, &listener->addr, sizeof(*addr))))
+    struct Listener *listener = node->data;
+    if ((port == listener->port) && memcmp(addr, &listener->addr, sizeof(*addr)) == 0)
     {
       /* Try to return an open listener, otherwise reuse a closed one */
       if (listener->fd)
@@ -308,11 +402,11 @@ listener_find(int port, struct irc_ssaddr *addr)
         last_closed = listener;
       }
       else
-        return (listener);
+        return listener;
     }
   }
 
-  return (last_closed);
+  return last_closed;
 }
 
 /*
@@ -343,10 +437,10 @@ listener_close(struct Listener *listener)
 void
 listener_close_marked(void)
 {
-  dlink_node *node, *node_next;
+  list_node_t *node, *node_next;
 
   /* close all 'extra' listening ports we have */
-  DLINK_FOREACH_SAFE(node, node_next, listener_list.head)
+  LIST_FOREACH_SAFE(node, node_next, listener_list.head)
     listener_close(node->data);
 }
 
@@ -368,7 +462,6 @@ listener_release(struct Listener *listener)
 void
 listener_add(int port, const char *vhost_ip, unsigned int flags)
 {
-  struct irc_ssaddr vaddr;
   struct addrinfo hints, *res;
   char portname[PORTNAMELEN + 1];
   static short int pass = 0; /* if ipv6 and no address specified we need to
@@ -392,6 +485,7 @@ listener_add(int port, const char *vhost_ip, unsigned int flags)
   getaddrinfo("::", portname, &hints, &res);
   assert(res);
 
+  struct io_addr vaddr;
   memcpy((struct sockaddr *)&vaddr, res->ai_addr, res->ai_addrlen);
   vaddr.ss_len = res->ai_addrlen;
   freeaddrinfo(res);
@@ -433,7 +527,7 @@ listener_add(int port, const char *vhost_ip, unsigned int flags)
     listener->flags = flags;
   }
 
-  if (listener_finalize(listener) == true)
+  if (listener_finalize(listener))
     listener->active = true;
   else
     listener_close(listener);
