@@ -2,7 +2,7 @@
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
  *  Copyright (c) 2000 Kevin L. Mitchell <klmitch@mit.edu>
- *  Copyright (c) 2013-2022 ircd-hybrid development team
+ *  Copyright (c) 2013-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
  */
 
 #include "stdinc.h"
+#include "defaults.h"
 #include "list.h"
 #include "ircd.h"
 #include "conf.h"
@@ -35,17 +36,16 @@
 #include "memory.h"
 #include "log.h"
 #include "motd.h"
-#include "hostmask.h"
+#include "address.h"
 #include "misc.h"
 
-
 /** Global list of messages of the day. */
 static struct
 {
   struct Motd *local;    /**< Local MOTD. */
   struct Motd *remote;   /**< Remote MOTD. */
-  dlink_list other;      /**< MOTDs specified in configuration file. */
-  dlink_list cachelist;  /**< List of MotdCache entries. */
+  list_t other;      /**< MOTDs specified in configuration file. */
+  list_t cachelist;  /**< List of MotdCache entries. */
 } MotdList;
 
 
@@ -56,7 +56,7 @@ static struct
 static struct Motd *
 motd_create(const char *mask, const char *path)
 {
-  struct Motd *motd = xcalloc(sizeof(*motd));
+  struct Motd *motd = io_calloc(sizeof(*motd));
 
   if (EmptyString(mask))
     motd->type = MOTD_UNIVERSAL;
@@ -78,10 +78,8 @@ motd_create(const char *mask, const char *path)
     }
   }
 
-  if (mask)
-    motd->mask = xstrdup(mask);
-
-  motd->path = xstrdup(path);
+  motd->mask = mask ? io_strdup(mask) : NULL;
+  motd->path = io_strdup(path);
   motd->maxcount = MOTD_MAXLINES;
 
   return motd;
@@ -96,20 +94,15 @@ motd_create(const char *mask, const char *path)
 static struct MotdCache *
 motd_cache(struct Motd *motd)
 {
-  struct stat sb;
-  char line[MOTD_LINESIZE + 2];  /* +2 for \r\n */
-  char *tmp = NULL;
-  unsigned int i = 0;
-  dlink_node *node;
-
-  assert(motd);
-  assert(motd->path);
+  if (motd == NULL)
+    return NULL;
 
   if (motd->cache)
     return motd->cache;
 
   /* Try to find it in the list of cached files */
-  DLINK_FOREACH(node, MotdList.cachelist.head)
+  list_node_t *node;
+  LIST_FOREACH(node, MotdList.cachelist.head)
   {
     struct MotdCache *cache = node->data;
 
@@ -121,49 +114,59 @@ motd_cache(struct Motd *motd)
     }
   }
 
-  /* Need the file's modification time */
-  if (stat(motd->path, &sb) == -1)
+  int fd = open(motd->path, O_RDONLY);
+  if (fd == -1)
   {
-    ilog(LOG_TYPE_IRCD, "Couldn't stat \"%s\": %s", motd->path,
-         strerror(errno));
-    return 0;
+    log_write(LOG_TYPE_IRCD, "Couldn't open \"%s\": %s", motd->path, strerror(errno));
+    return NULL;
+  }
+
+  /* Need the file's modification time */
+  struct stat sb;
+  if (fstat(fd, &sb))
+  {
+    log_write(LOG_TYPE_IRCD, "Couldn't fstat \"%s\": %s", motd->path, strerror(errno));
+    close(fd);
+    return NULL;
   }
 
   /* Gotta read in the file, now */
-  FILE *file = fopen(motd->path, "r");
+  FILE *file = fdopen(fd, "r");
   if (file == NULL)
   {
-    ilog(LOG_TYPE_IRCD, "Couldn't open \"%s\": %s", motd->path,
-         strerror(errno));
-    return 0;
+    log_write(LOG_TYPE_IRCD, "Couldn't fdopen \"%s\": %s", motd->path, strerror(errno));
+    close(fd);
+    return NULL;
   }
 
   /* Ok, allocate a structure; we'll realloc later to trim memory */
-  struct MotdCache *cache = xcalloc(sizeof(*cache) + (MOTD_LINESIZE * MOTD_MAXLINES));
+  struct MotdCache *cache = io_calloc(sizeof(*cache) + (MOTD_LINESIZE * MOTD_MAXLINES));
   cache->ref = 1;
-  cache->path = xstrdup(motd->path);
+  cache->path = io_strdup(motd->path);
   cache->maxcount = motd->maxcount;
   cache->modtime = sb.st_mtime;  /* Store modtime */
 
+  char line[MOTD_LINESIZE + 2];  /* +2 for \r\n */
   while (cache->count < cache->maxcount && fgets(line, sizeof(line), file))
   {
+    unsigned int i = 0;
     /* Copy over line, stopping when we overflow or hit line end */
-    for (tmp = line, i = 0; i < (MOTD_LINESIZE - 1) && *tmp && *tmp != '\r' && *tmp != '\n'; ++tmp, ++i)
+    for (char *tmp = line; i < (MOTD_LINESIZE - 1) && *tmp && *tmp != '\r' && *tmp != '\n'; ++tmp, ++i)
       cache->motd[cache->count][i] = *tmp;
-    cache->motd[cache->count][i] = '\0';
 
+    cache->motd[cache->count][i] = '\0';
     cache->count++;
   }
 
   fclose(file);  /* Close the file */
 
   /* Trim memory usage a little */
-  motd->cache = xcalloc(sizeof(*motd->cache) + (MOTD_LINESIZE * cache->count));
+  motd->cache = io_calloc(sizeof(*motd->cache) + (MOTD_LINESIZE * cache->count));
   memcpy(motd->cache, cache, sizeof(*motd->cache) + (MOTD_LINESIZE * cache->count));
-  xfree(cache);
+  io_free(cache);
 
   /* Now link it in */
-  dlinkAdd(motd->cache, &motd->cache->node, &MotdList.cachelist);
+  list_add(motd->cache, &motd->cache->node, &MotdList.cachelist);
 
   return motd->cache;
 }
@@ -175,7 +178,8 @@ motd_cache(struct Motd *motd)
 static void
 motd_decache(struct Motd *motd)
 {
-  assert(motd);
+  if (motd == NULL)
+    return;
 
   struct MotdCache *cache = motd->cache;
   if (cache == NULL)  /* We can be called for records with no cache */
@@ -185,9 +189,9 @@ motd_decache(struct Motd *motd)
 
   if (--cache->ref == 0)  /* Reduce reference count */
   {
-    dlinkDelete(&cache->node, &MotdList.cachelist);
-    xfree(cache->path);  /* Free path info */
-    xfree(cache);  /* Very simple for a reason */
+    list_remove(&cache->node, &MotdList.cachelist);
+    io_free(cache->path);  /* Free path info */
+    io_free(cache);  /* Very simple for a reason */
   }
 }
 
@@ -198,14 +202,15 @@ motd_decache(struct Motd *motd)
 static void
 motd_destroy(struct Motd *motd)
 {
-  assert(motd);
+  if (motd == NULL)
+    return;
 
   if (motd->cache)  /* Drop the cache */
     motd_decache(motd);
 
-  xfree(motd->path);  /* We always must have a path */
-  xfree(motd->mask);
-  xfree(motd);
+  io_free(motd->path);  /* We always must have a path */
+  io_free(motd->mask);
+  io_free(motd);
 }
 
 /*! \brief Find the first matching MOTD block for a user.
@@ -219,15 +224,14 @@ motd_destroy(struct Motd *motd)
 static struct Motd *
 motd_lookup(const struct Client *client)
 {
-  dlink_node *node;
-
   assert(client);
 
   if (!MyConnect(client))  /* Not my user, always return remote motd */
     return MotdList.remote;
 
   /* Check the motd blocks first */
-  DLINK_FOREACH(node, MotdList.other.head)
+  list_node_t *node;
+  LIST_FOREACH(node, MotdList.other.head)
   {
     struct Motd *motd = node->data;
 
@@ -248,7 +252,7 @@ motd_lookup(const struct Client *client)
         break;
       case MOTD_IPMASKV6:
       case MOTD_IPMASKV4:
-        if (address_compare(&client->ip, &motd->address, false, false, motd->addrbits) == true)
+        if (address_compare(&client->addr, &motd->address, false, false, motd->addrbits))
           return motd;
         break;
       default: break;
@@ -322,12 +326,14 @@ motd_signon(struct Client *client)
 void
 motd_recache(void)
 {
-  dlink_node *node;
+  if (MotdList.local)
+    motd_decache(MotdList.local);  /* Decache local and remote MOTDs */
 
-  motd_decache(MotdList.local);  /* Decache local and remote MOTDs */
-  motd_decache(MotdList.remote);
+  if (MotdList.remote)
+    motd_decache(MotdList.remote);
 
-  DLINK_FOREACH(node, MotdList.other.head)  /* Now all the others */
+  list_node_t *node;
+  LIST_FOREACH(node, MotdList.other.head)  /* Now all the others */
     motd_decache(node->data);
 
   /* Now recache local and remote MOTDs */
@@ -342,15 +348,25 @@ void
 motd_init(void)
 {
   if (MotdList.local)  /* Destroy old local MOTD */
+  {
     motd_destroy(MotdList.local);
-
-  MotdList.local = motd_create(NULL, MPATH);
-  motd_cache(MotdList.local);  /* Initialize local MOTD and cache it */
+    MotdList.local = NULL;
+  }
 
   if (MotdList.remote)  /* Destroy old remote MOTD */
+  {
     motd_destroy(MotdList.remote);
+    MotdList.remote = NULL;
+  }
 
-  MotdList.remote = motd_create(NULL, MPATH);
+  const char *const path = ConfigServerInfo.motd_file;
+  if (EmptyString(path))
+    return;
+
+  MotdList.local = motd_create(NULL, path);
+  motd_cache(MotdList.local);  /* Initialize local MOTD and cache it */
+
+  MotdList.remote = motd_create(NULL, path);
   motd_cache(MotdList.remote);  /* Initialize remote MOTD and cache it */
 }
 
@@ -362,8 +378,7 @@ void
 motd_add(const char *mask, const char *path)
 {
   struct Motd *motd = motd_create(mask, path);  /* Create the motd */
-
-  dlinkAdd(motd, &motd->node, &MotdList.other);
+  list_add(motd, &motd->node, &MotdList.other);
 }
 
 /*! \brief Clear out all MOTDs.
@@ -380,7 +395,7 @@ motd_clear(void)
   while (MotdList.other.head)  /* Destroy other MOTDs */
   {
     struct Motd *motd = MotdList.other.head->data;
-    dlinkDelete(&motd->node, &MotdList.other);
+    list_remove(&motd->node, &MotdList.other);
     motd_destroy(motd);
   }
 
@@ -395,13 +410,12 @@ motd_clear(void)
 void
 motd_report(struct Client *client, int parc, char *parv[])
 {
-  dlink_node *node;
+  list_node_t *node;
 
-  DLINK_FOREACH(node, MotdList.other.head)
+  LIST_FOREACH(node, MotdList.other.head)
   {
     const struct Motd *motd = node->data;
-    sendto_one_numeric(client, &me, RPL_STATSTLINE,
-                       motd->mask, motd->path);
+    sendto_one_numeric(client, &me, RPL_STATSTLINE, motd->mask, motd->path);
   }
 }
 
@@ -411,46 +425,44 @@ motd_report(struct Client *client, int parc, char *parv[])
 void
 motd_memory_count(struct Client *client)
 {
-  dlink_node *node;
-  unsigned int mt  = 0;  /* Motd count */
-  unsigned int mtc = 0;  /* Motd cache count */
-  size_t mtm  = 0;  /* Memory consumed by motd */
-  size_t mtcm = 0;  /* Memory consumed by motd cache */
+  unsigned int motd_count = 0;
+  size_t motd_bytes = 0;
 
   if (MotdList.local)
   {
-    ++mt;
-    mtm += sizeof(struct Motd);
-    mtm += MotdList.local->path ? (strlen(MotdList.local->path) + 1) : 0;
+    ++motd_count;
+    motd_bytes += sizeof(struct Motd);
+    motd_bytes += MotdList.local->path ? strlen(MotdList.local->path) + 1 : 0;
   }
 
   if (MotdList.remote)
   {
-    ++mt;
-    mtm += sizeof(struct Motd);
-    mtm += MotdList.remote->path ? (strlen(MotdList.remote->path) + 1) : 0;
+    ++motd_count;
+    motd_bytes += sizeof(struct Motd);
+    motd_bytes += MotdList.remote->path ? strlen(MotdList.remote->path) + 1 : 0;
   }
 
-  DLINK_FOREACH(node, MotdList.other.head)
+  list_node_t *node;
+  LIST_FOREACH(node, MotdList.other.head)
   {
     const struct Motd *motd = node->data;
-
-    ++mt;
-    mtm += sizeof(struct Motd);
-    mtm += motd->path ? (strlen(motd->path) + 1) : 0;
-    mtm += motd->mask ? (strlen(motd->mask) + 1) : 0;
+    ++motd_count;
+    motd_bytes += sizeof(struct Motd);
+    motd_bytes += motd->path ? strlen(motd->path) + 1 : 0;
+    motd_bytes += motd->mask ? strlen(motd->mask) + 1 : 0;
   }
 
-  DLINK_FOREACH(node, MotdList.cachelist.head)
+  unsigned int motd_cache_count = 0;
+  size_t motd_cache_bytes = 0;
+
+  LIST_FOREACH(node, MotdList.cachelist.head)
   {
     const struct MotdCache *cache = node->data;
-
-    ++mtc;
-    mtcm += sizeof(struct MotdCache) + (MOTD_LINESIZE * (cache->count - 1));
-    mtcm += cache->path ? (strlen(cache->path) + 1) : 0;
+    ++motd_cache_count;
+    motd_cache_bytes += sizeof(struct MotdCache) + (MOTD_LINESIZE * (cache->count - 1));
+    motd_cache_bytes += cache->path ? strlen(cache->path) + 1 : 0;
   }
 
-  sendto_one_numeric(client, &me, RPL_STATSDEBUG | SND_EXPLICIT,
-                     "z :Motds %u(%zu) Cache %u(%zu)",
-                     mt, mtm, mtc, mtcm);
+  sendto_one_numeric(client, &me, RPL_STATSDEBUG | SND_EXPLICIT, "z :Motds %u(%zu) Cache %u(%zu)",
+                     motd_count, motd_bytes, motd_cache_count, motd_cache_bytes);
 }
