@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,33 +24,35 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
 #include "list.h"
 #include "send.h"
+#include "cap.h"
 #include "channel.h"
 #include "client.h"
 #include "dbuf.h"
 #include "irc_string.h"
-#include "ircd.h"
-#include "s_bsd.h"
+#include "comm.h"
 #include "server_capab.h"
+#include "user_mode.h"
 #include "conf_class.h"
 #include "log.h"
-
+#include "conf.h"
+#include "address.h"
 
 static uintmax_t send_marker;
 
-
 /* send_format()
  *
  * inputs
  *		- buffer
- *		- format pattern to use
+ *		- format format to use
  *		- var args
  * output	- number of bytes formatted output
  * side effects	- modifies sendbuf
  */
 static void
-send_format(struct dbuf_block *buffer, const char *pattern, va_list args)
+send_format(struct dbuf_block *buffer, const char *format, va_list args)
 {
   /*
    * from rfc1459
@@ -64,7 +66,7 @@ send_format(struct dbuf_block *buffer, const char *pattern, va_list args)
    * continuation message lines.  See section 7 for more details about
    * current implementations.
    */
-  dbuf_put_args(buffer, pattern, args);
+  dbuf_put_args(buffer, format, args);
 
   if (buffer->size > IRCD_BUFSIZE - 2)
     buffer->size = IRCD_BUFSIZE - 2;
@@ -79,7 +81,7 @@ send_format(struct dbuf_block *buffer, const char *pattern, va_list args)
  **      sendq.
  */
 static void
-send_message(struct Client *to, struct dbuf_block *buffer)
+sendto_one_buffer(struct Client *to, struct dbuf_block *buffer)
 {
   assert(!IsMe(to));
   assert(to != &me);
@@ -88,11 +90,11 @@ send_message(struct Client *to, struct dbuf_block *buffer)
   if (dbuf_length(&to->connection->buf_sendq) + buffer->size > class_get_sendq(&to->connection->confs))
   {
     if (IsServer(to))
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                           "Max SendQ limit exceeded for %s: %zu > %u",
-                           client_get_name(to, HIDE_IP),
-                           (dbuf_length(&to->connection->buf_sendq) + buffer->size),
-                           class_get_sendq(&to->connection->confs));
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                     "Max SendQ limit exceeded for %s: %zu > %u",
+                     client_get_name(to, HIDE_IP),
+                     (dbuf_length(&to->connection->buf_sendq) + buffer->size),
+                     class_get_sendq(&to->connection->confs));
 
     if (IsClient(to))
       AddFlag(to, FLAGS_SENDQEX);
@@ -125,7 +127,7 @@ send_message(struct Client *to, struct dbuf_block *buffer)
  *
  */
 static void
-send_message_remote(struct Client *to, const struct Client *from, struct dbuf_block *buffer)
+sendto_one_buffer_remote(struct Client *to, const struct Client *from, struct dbuf_block *buffer)
 {
   assert(MyConnect(to));
   assert(IsServer(to));
@@ -133,10 +135,10 @@ send_message_remote(struct Client *to, const struct Client *from, struct dbuf_bl
   assert(to->from == to);
 
   if (to == from->from)
-    sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE, "Send message to %s dropped from %s (Fake Dir)",
-                         to->name, from->name);
+    sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Send message to %s dropped from %s (Fake Dir)",
+                   to->name, from->name);
   else
-    send_message(to, buffer);
+    sendto_one_buffer(to, buffer);
 }
 
 /*
@@ -166,8 +168,6 @@ sendq_unblocked(fde_t *F, void *data)
 void
 send_queued_write(struct Client *to)
 {
-  ssize_t retlen;
-
   /*
    * Once socket is marked dead, we cannot start writing to it,
    * even if the error is removed...
@@ -178,6 +178,7 @@ send_queued_write(struct Client *to)
   /* Next, lets try to write some data */
   while (dbuf_length(&to->connection->buf_sendq))
   {
+    ssize_t retlen;
     bool want_read = false;
     const struct dbuf_block *first = to->connection->buf_sendq.blocks.head->data;
 
@@ -186,7 +187,7 @@ send_queued_write(struct Client *to)
       retlen = tls_write(&to->connection->fd->tls, first->data + to->connection->buf_sendq.pos,
                                                    first->size - to->connection->buf_sendq.pos, &want_read);
 
-      if (want_read == true)
+      if (want_read)
         return;  /* Retry later, don't register for write events */
     }
     else
@@ -195,7 +196,7 @@ send_queued_write(struct Client *to)
 
     if (retlen <= 0)
     {
-      if (retlen < 0 && comm_ignore_errno(errno) == true)
+      if (retlen < 0 && comm_ignore_errno(errno))
       {
         AddFlag(to, FLAGS_BLOCKED);
         /* We have a non-fatal error, reschedule a write */
@@ -222,21 +223,19 @@ send_queued_write(struct Client *to)
  * side effects	- send message to single client
  */
 void
-sendto_one(struct Client *to, const char *pattern, ...)
+sendto_one(struct Client *to, const char *format, ...)
 {
-  va_list args;
-
   if (IsDead(to->from))
     return;  /* This socket has already been marked as dead */
 
-  va_start(args, pattern);
-
   struct dbuf_block *buffer = dbuf_alloc();
-  send_format(buffer, pattern, args);
 
+  va_list args;
+  va_start(args, format);
+  send_format(buffer, format, args);
   va_end(args);
 
-  send_message(to->from, buffer);
+  sendto_one_buffer(to->from, buffer);
 
   dbuf_ref_free(buffer);
 }
@@ -244,8 +243,6 @@ sendto_one(struct Client *to, const char *pattern, ...)
 void
 sendto_one_numeric(struct Client *to, const struct Client *from, enum irc_numerics numeric, ...)
 {
-  va_list args;
-
   if (IsDead(to->from))
     return;  /* This socket has already been marked as dead */
 
@@ -256,6 +253,7 @@ sendto_one_numeric(struct Client *to, const struct Client *from, enum irc_numeri
   struct dbuf_block *buffer = dbuf_alloc();
   dbuf_put_fmt(buffer, ":%s %03d %s ", ID_or_name(from, to), numeric & ~SND_EXPLICIT, dest);
 
+  va_list args;
   va_start(args, numeric);
 
   const char *numstr;
@@ -267,16 +265,14 @@ sendto_one_numeric(struct Client *to, const struct Client *from, enum irc_numeri
   send_format(buffer, numstr, args);
   va_end(args);
 
-  send_message(to->from, buffer);
+  sendto_one_buffer(to->from, buffer);
 
   dbuf_ref_free(buffer);
 }
 
 void
-sendto_one_notice(struct Client *to, const struct Client *from, const char *pattern, ...)
+sendto_one_notice(struct Client *to, const struct Client *from, const char *format, ...)
 {
-  va_list args;
-
   if (IsDead(to->from))
     return;  /* This socket has already been marked as dead */
 
@@ -287,245 +283,149 @@ sendto_one_notice(struct Client *to, const struct Client *from, const char *patt
   struct dbuf_block *buffer = dbuf_alloc();
   dbuf_put_fmt(buffer, ":%s NOTICE %s ", ID_or_name(from, to), dest);
 
-  va_start(args, pattern);
-  send_format(buffer, pattern, args);
-  va_end(args);
-
-  send_message(to->from, buffer);
-
-  dbuf_ref_free(buffer);
-}
-
-/* sendto_channel_butone()
- *
- * inputs	- pointer to client(server) to NOT send message to
- *		- pointer to client that is sending this message
- *		- pointer to channel being sent to
- *		- vargs message
- * output	- NONE
- * side effects	- message as given is sent to given channel members.
- *
- * WARNING - +D clients are ignored
- */
-void
-sendto_channel_butone(struct Client *one, const struct Client *from,
-                      struct Channel *channel, int rank,
-                      const char *pattern, ...)
-{
-  va_list args_l, args_r;
-  dlink_node *node;
-  struct dbuf_block *buffer_l = dbuf_alloc();
-  struct dbuf_block *buffer_r = dbuf_alloc();
-
-  if (IsClient(from))
-    dbuf_put_fmt(buffer_l, ":%s!%s@%s ", from->name, from->username, from->host);
-  else
-    dbuf_put_fmt(buffer_l, ":%s ", from->name);
-
-  dbuf_put_fmt(buffer_r, ":%s ", from->id);
-
-  va_start(args_l, pattern);
-  va_start(args_r, pattern);
-  send_format(buffer_l, pattern, args_l);
-  send_format(buffer_r, pattern, args_r);
-  va_end(args_l);
-  va_end(args_r);
-
-  ++send_marker;
-
-  DLINK_FOREACH(node, channel->members.head)
-  {
-    struct ChannelMember *member = node->data;
-    struct Client *target = member->client;
-
-    assert(IsClient(target));
-
-    if (IsDead(target->from))
-      continue;
-
-    if (one && (target->from == one->from))
-      continue;
-
-    if (rank && member_highest_rank(member) < rank)
-      continue;
-
-    if (HasUMode(target, UMODE_DEAF))
-      continue;
-
-    if (MyConnect(target))
-      send_message(target, buffer_l);
-    else if (target->from->connection->send_marker != send_marker)
-      send_message_remote(target->from, from, buffer_r);
-
-    target->from->connection->send_marker = send_marker;
-  }
-
-  dbuf_ref_free(buffer_l);
-  dbuf_ref_free(buffer_r);
-}
-
-/* sendto_server()
- *
- * inputs       - pointer to client to NOT send to
- *              - pointer to channel
- *              - capabs or'd together which must ALL be present
- *              - capabs or'd together which must ALL NOT be present
- *              - printf style format string
- *              - args to format string
- * output       - NONE
- * side effects - Send a message to all connected servers, except the
- *                client 'one' (if non-NULL), as long as the servers
- *                support ALL capabs in 'capab', and NO capabs in 'nocapab'.
- *
- * This function was written in an attempt to merge together the other
- * billion sendto_*serv*() functions, which sprung up with capabs,
- * lazylinks, uids, etc.
- * -davidt
- */
-void
-sendto_server(const struct Client *one,
-              const unsigned int capab,
-              const unsigned int nocapab,
-              const char *format, ...)
-{
   va_list args;
-  dlink_node *node;
-  struct dbuf_block *buffer = dbuf_alloc();
-
   va_start(args, format);
   send_format(buffer, format, args);
   va_end(args);
 
-  DLINK_FOREACH(node, local_server_list.head)
+  sendto_one_buffer(to->from, buffer);
+
+  dbuf_ref_free(buffer);
+}
+
+/* sendto_anywhere()
+ *
+ * inputs	- pointer to dest client
+ * 		- pointer to from client
+ * 		- varags
+ * output	- NONE
+ * side effects	- less efficient than sendto_remote and sendto_one
+ * 		  but useful when one does not know where target "lives"
+ */
+void
+sendto_one_anywhere(struct Client *to, const struct Client *from, const char *command, const char *format, ...)
+{
+  if (IsDead(to->from))
+    return;
+
+  struct dbuf_block *buffer = dbuf_alloc();
+  if (MyClient(to) && IsClient(from))
+    dbuf_put_fmt(buffer, ":%s!%s@%s %s %s ",
+                 from->name, from->username, from->host, command, to->name);
+  else
+    dbuf_put_fmt(buffer, ":%s %s %s ",
+                 ID_or_name(from, to), command, ID_or_name(to, to));
+
+  va_list args;
+  va_start(args, format);
+  send_format(buffer, format, args);
+  va_end(args);
+
+  if (MyConnect(to))
+    sendto_one_buffer(to, buffer);
+  else
+    sendto_one_buffer_remote(to->from, from, buffer);
+
+  dbuf_ref_free(buffer);
+}
+
+static bool
+sendto_clients_qualifies(const struct Client *client, uint64_t flags, send_recipient_t recipient)
+{
+  if (flags && user_mode_has_flag(client, flags) == false)
+    return false;
+
+  switch (recipient)
+  {
+    case SEND_RECIPIENT_ADMIN:
+      return user_mode_has_flag(client, UMODE_ADMIN);
+    case SEND_RECIPIENT_OPER:
+      return user_mode_has_flag(client, UMODE_OPER) && user_mode_has_flag(client, UMODE_ADMIN) == false;
+    case SEND_RECIPIENT_OPER_ALL:
+      return user_mode_has_flag(client, UMODE_OPER);
+    case SEND_RECIPIENT_CLIENT:
+      return true;
+    default:
+      return false;
+  }
+}
+
+/* sendto_clients()
+ *
+ * inputs	- flag types of messages to show to real opers
+ *		- flag indicating opers/admins
+ *		- var args input message
+ * output	- NONE
+ * side effects	- Send to *local* ops only but NOT +s nonopers.
+ */
+void
+sendto_clients(uint64_t flags, send_recipient_t recipient, send_type_t type, const char *format, ...)
+{
+  const char *ntype = "???";
+
+  switch (type)
+  {
+    case SEND_TYPE_NOTICE:
+      ntype = "Notice";
+      break;
+    case SEND_TYPE_GLOBAL:
+      ntype = "Global";
+      break;
+    case SEND_TYPE_LOCOPS:
+      ntype = "LocOps";
+      break;
+    default:
+      assert(0);
+  }
+
+  struct dbuf_block *buffer = dbuf_alloc();
+  dbuf_put_fmt(buffer, ":%s NOTICE * :*** %s -- ", me.name, ntype);
+
+  va_list args;
+  va_start(args, format);
+  send_format(buffer, format, args);
+  va_end(args);
+
+  list_t *list = (recipient == SEND_RECIPIENT_CLIENT) ? &local_client_list : &oper_list;
+  list_node_t *node;
+  LIST_FOREACH(node, list->head)
   {
     struct Client *client = node->data;
-
-    /* If dead already skip */
     if (IsDead(client))
       continue;
 
-    /* check against 'one' */
-    if (one && (client == one->from))
+    if (sendto_clients_qualifies(client, flags, recipient) == false)
       continue;
 
-    /* check we have required capabs */
-    if (IsCapable(client, capab) != capab)
-      continue;
-
-    /* check we don't have any forbidden capabs */
-    if (IsCapable(client, nocapab))
-      continue;
-
-    send_message(client, buffer);
+    sendto_one_buffer(client, buffer);
   }
 
   dbuf_ref_free(buffer);
 }
 
-/* sendto_common_channels_local()
+/* ts_warn()
  *
- * inputs	- pointer to client
- *		- pattern to send
- * output	- NONE
- * side effects	- Sends a message to all people on local server who are
- * 		  in same channel with user.
- *		  used by m_nick.c and exit_one_client.
+ * inputs       - var args message
+ * output       - NONE
+ * side effects - Call sendto_clients, with some flood checking
+ *                (at most 5 warnings every 5 seconds)
  */
 void
-sendto_common_channels_local(struct Client *user, bool touser, unsigned int poscap,
-                             unsigned int negcap, const char *pattern, ...)
+sendto_clients_ratelimited(uintmax_t *rate, const char *format, ...)
 {
-  va_list args;
-  dlink_node *node, *node2;
-  struct dbuf_block *buffer = dbuf_alloc();
+  if ((io_time_get(IO_TIME_MONOTONIC_SEC) - *rate) < 20)
+    return;
 
-  va_start(args, pattern);
-  send_format(buffer, pattern, args);
+  *rate = io_time_get(IO_TIME_MONOTONIC_SEC);
+
+  char buffer[IRCD_BUFSIZE];
+  va_list args;
+  va_start(args, format);
+  vsnprintf(buffer, sizeof(buffer), format, args);
   va_end(args);
 
-  ++send_marker;
-
-  DLINK_FOREACH(node, user->channel.head)
-  {
-    struct ChannelMember *member = node->data;
-    struct Channel *channel = member->channel;
-
-    DLINK_FOREACH(node2, channel->members_local.head)
-    {
-      struct ChannelMember *member2 = node2->data;
-      struct Client *target = member2->client;
-
-      if (IsDead(target))
-        continue;
-
-      if (target == user)
-        continue;
-
-      if (target->connection->send_marker == send_marker)
-        continue;
-
-      if (poscap && HasCap(target, poscap) != poscap)
-        continue;
-
-      if (negcap && HasCap(target, negcap))
-        continue;
-
-      target->connection->send_marker = send_marker;
-      send_message(target, buffer);
-    }
-  }
-
-  if (touser == true && MyConnect(user) && !IsDead(user))
-    if (HasCap(user, poscap) == poscap)
-      send_message(user, buffer);
-
-  dbuf_ref_free(buffer);
-}
-
-/*! \brief Send a message to members of a channel that are locally connected to this server.
- * \param one      Client to skip; can be NULL
- * \param channel    Destination channel
- * \param rank     Minimum channel member rank clients must have
- * \param poscap   Positive client capabilities flags (CAP)
- * \param negcap   Negative client capabilities flags (CAP)
- * \param pattern  Format string for command arguments
- */
-void
-sendto_channel_local(const struct Client *one, struct Channel *channel, int rank,
-                     unsigned int poscap, unsigned int negcap, const char *pattern, ...)
-{
-  va_list args;
-  dlink_node *node;
-  struct dbuf_block *buffer = dbuf_alloc();
-
-  va_start(args, pattern);
-  send_format(buffer, pattern, args);
-  va_end(args);
-
-  DLINK_FOREACH(node, channel->members_local.head)
-  {
-    struct ChannelMember *member = node->data;
-    struct Client *target = member->client;
-
-    if (IsDead(target))
-      continue;
-
-    if (one && (target == one->from))
-      continue;
-
-    if (rank && member_highest_rank(member) < rank)
-      continue;
-
-    if (poscap && HasCap(target, poscap) != poscap)
-      continue;
-
-    if (negcap && HasCap(target, negcap))
-      continue;
-
-    send_message(target, buffer);
-  }
-
-  dbuf_ref_free(buffer);
+  sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "%s", buffer);
+  log_write(LOG_TYPE_IRCD, "%s", buffer);
 }
 
 /*
@@ -546,10 +446,19 @@ sendto_channel_local(const struct Client *one, struct Channel *channel, int rank
  * side effects	- NONE
  */
 static bool
-match_it(const struct Client *one, const char *mask, bool host)
+sendto_match_butone_qualifies(const struct Client *one, const char *mask, send_match_type_t type)
 {
-  if (host == true)
-    return match(mask, one->host) == 0;
+  if (type == SEND_MATCH_HOST)
+  {
+    struct io_addr addr;
+    int bits = 0;
+    const int ret = parse_netmask(mask, &addr, &bits);
+
+    if (ret == HM_IPV4 || ret == HM_IPV6)
+      if (address_compare(&one->addr, &addr, false, false, bits))
+        return true;
+    return match(mask, one->realhost) == 0;
+  }
 
   return match(mask, one->servptr->name) == 0;
 }
@@ -562,26 +471,26 @@ match_it(const struct Client *one, const char *mask, bool host)
  * ugh. ONLY used by m_message.c to send an "oper magic" message. ugh.
  */
 void
-sendto_match_butone(const struct Client *one, const struct Client *from,
-                    const char *mask, bool host, const char *pattern, ...)
+sendto_match_butone(const struct Client *one, const struct Client *from, const char *mask,
+                    send_match_type_t type, const char *format, ...)
 {
-  va_list args_l, args_r;
-  dlink_node *node;
   struct dbuf_block *buffer_l = dbuf_alloc();
   struct dbuf_block *buffer_r = dbuf_alloc();
 
   dbuf_put_fmt(buffer_l, ":%s!%s@%s ", from->name, from->username, from->host);
   dbuf_put_fmt(buffer_r, ":%s ", from->id);
 
-  va_start(args_l, pattern);
-  va_start(args_r, pattern);
-  send_format(buffer_l, pattern, args_l);
-  send_format(buffer_r, pattern, args_r);
+  va_list args_l, args_r;
+  va_start(args_l, format);
+  va_start(args_r, format);
+  send_format(buffer_l, format, args_l);
+  send_format(buffer_r, format, args_r);
   va_end(args_l);
   va_end(args_r);
 
   /* Scan the local clients */
-  DLINK_FOREACH(node, local_client_list.head)
+  list_node_t *node;
+  LIST_FOREACH(node, local_client_list.head)
   {
     struct Client *client = node->data;
 
@@ -591,14 +500,14 @@ sendto_match_butone(const struct Client *one, const struct Client *from,
     if (one && (client == one->from))
       continue;
 
-    if (match_it(client, mask, host) == false)
+    if (sendto_match_butone_qualifies(client, mask, type) == false)
       continue;
 
-    send_message(client, buffer_l);
+    sendto_one_buffer(client, buffer_l);
   }
 
   /* Now scan servers */
-  DLINK_FOREACH(node, local_server_list.head)
+  LIST_FOREACH(node, local_server_list.head)
   {
     struct Client *client = node->data;
 
@@ -611,7 +520,7 @@ sendto_match_butone(const struct Client *one, const struct Client *from,
      *        if (IsRegisteredUser(target) &&
      *                        match_it(target, mask, what) &&
      *                        (target->from == client))
-     *   vsendto_prefix_one(client, from, pattern, args);
+     *   vsendto_prefix_one(client, from, format, args);
      *
      * That way, we wouldn't send the message to a server who didn't have
      * a matching client. However, on a network such as EFNet, that code
@@ -627,13 +536,69 @@ sendto_match_butone(const struct Client *one, const struct Client *from,
     if (one && (client == one->from))
       continue;
 
-    send_message_remote(client, from, buffer_r);
+    sendto_one_buffer_remote(client, from, buffer_r);
   }
 
   dbuf_ref_free(buffer_l);
   dbuf_ref_free(buffer_r);
 }
 
+/* sendto_servers()
+ *
+ * inputs       - pointer to client to NOT send to
+ *              - pointer to channel
+ *              - capabs or'd together which must ALL be present
+ *              - capabs or'd together which must ALL NOT be present
+ *              - printf style format string
+ *              - args to format string
+ * output       - NONE
+ * side effects - Send a message to all connected servers, except the
+ *                client 'one' (if non-NULL), as long as the servers
+ *                support ALL capabs in 'capab', and NO capabs in 'nocapab'.
+ *
+ * This function was written in an attempt to merge together the other
+ * billion sendto_*serv*() functions, which sprung up with capabs,
+ * lazylinks, uids, etc.
+ * -davidt
+ */
+void
+sendto_servers(const struct Client *one, const unsigned int capab,
+               const unsigned int nocapab, const char *format, ...)
+{
+  struct dbuf_block *buffer = dbuf_alloc();
+
+  va_list args;
+  va_start(args, format);
+  send_format(buffer, format, args);
+  va_end(args);
+
+  list_node_t *node;
+  LIST_FOREACH(node, local_server_list.head)
+  {
+    struct Client *client = node->data;
+
+    /* If dead already skip */
+    if (IsDead(client))
+      continue;
+
+    /* check against 'one' */
+    if (one && (client == one->from))
+      continue;
+
+    /* check we have required capabs */
+    if (IsCapable(client, capab) != capab)
+      continue;
+
+    /* check we don't have any forbidden capabs */
+    if (IsCapable(client, nocapab))
+      continue;
+
+    sendto_one_buffer(client, buffer);
+  }
+
+  dbuf_ref_free(buffer);
+}
+
 /* sendto_match_servs()
  *
  * inputs       - source client
@@ -644,22 +609,21 @@ sendto_match_butone(const struct Client *one, const struct Client *from,
  * side effects	- data sent to servers matching with capab
  */
 void
-sendto_match_servs(const struct Client *source_p, const char *mask, unsigned int capab,
-                   const char *pattern, ...)
+sendto_match_servs(const struct Client *source, const char *mask, unsigned int capab, const char *format, ...)
 {
-  va_list args;
-  dlink_node *node;
   struct dbuf_block *buffer = dbuf_alloc();
 
-  dbuf_put_fmt(buffer, ":%s ", source_p->id);
+  dbuf_put_fmt(buffer, ":%s ", source->id);
 
-  va_start(args, pattern);
-  send_format(buffer, pattern, args);
+  va_list args;
+  va_start(args, format);
+  send_format(buffer, format, args);
   va_end(args);
 
   ++send_marker;
 
-  DLINK_FOREACH(node, global_server_list.head)
+  list_node_t *node;
+  LIST_FOREACH(node, global_server_list.head)
   {
     struct Client *target = node->data;
 
@@ -671,7 +635,7 @@ sendto_match_servs(const struct Client *source_p, const char *mask, unsigned int
       continue;
 
     /* ... or the source */
-    if (target->from == source_p->from)
+    if (target->from == source->from)
       continue;
 
     if (target->from->connection->send_marker == send_marker)
@@ -684,177 +648,181 @@ sendto_match_servs(const struct Client *source_p, const char *mask, unsigned int
       continue;
 
     target->from->connection->send_marker = send_marker;
-    send_message_remote(target->from, source_p, buffer);
+    sendto_one_buffer_remote(target->from, source, buffer);
   }
 
   dbuf_ref_free(buffer);
 }
 
-/* sendto_anywhere()
+/* sendto_common_channels_local()
  *
- * inputs	- pointer to dest client
- * 		- pointer to from client
- * 		- varags
+ * inputs	- pointer to client
+ *		- format to send
  * output	- NONE
- * side effects	- less efficient than sendto_remote and sendto_one
- * 		  but useful when one does not know where target "lives"
+ * side effects	- Sends a message to all people on local server who are
+ * 		  in same channel with user.
+ *		  used by m_nick.c and exit_one_client.
  */
 void
-sendto_anywhere(struct Client *to, const struct Client *from,
-                const char *command,
-                const char *pattern, ...)
+sendto_common_channels_local(struct Client *user, bool touser, unsigned int poscap,
+                             unsigned int negcap, const char *format, ...)
 {
-  va_list args;
-
-  if (IsDead(to->from))
-    return;
-
   struct dbuf_block *buffer = dbuf_alloc();
-  if (MyClient(to) && IsClient(from))
-    dbuf_put_fmt(buffer, ":%s!%s@%s %s %s ", from->name, from->username,
-                 from->host, command, to->name);
-  else
-    dbuf_put_fmt(buffer, ":%s %s %s ", ID_or_name(from, to),
-                 command, ID_or_name(to, to));
 
-  va_start(args, pattern);
-  send_format(buffer, pattern, args);
+  va_list args;
+  va_start(args, format);
+  send_format(buffer, format, args);
   va_end(args);
 
-  if (MyConnect(to))
-    send_message(to, buffer);
-  else
-    send_message_remote(to->from, from, buffer);
+  ++send_marker;
+
+  list_node_t *node, *node2;
+  LIST_FOREACH(node, user->channel.head)
+  {
+    struct ChannelMember *member = node->data;
+    struct Channel *channel = member->channel;
+
+    LIST_FOREACH(node2, channel->members_local.head)
+    {
+      struct ChannelMember *member2 = node2->data;
+      struct Client *target = member2->client;
+
+      if (IsDead(target))
+        continue;
+
+      if (target == user)
+        continue;
+
+      if (target->connection->send_marker == send_marker)
+        continue;
+
+      if (poscap && HasCap(target, poscap) != poscap)
+        continue;
+
+      if (negcap && HasCap(target, negcap))
+        continue;
+
+      target->connection->send_marker = send_marker;
+      sendto_one_buffer(target, buffer);
+    }
+  }
+
+  if (touser && MyConnect(user) && !IsDead(user))
+    if (HasCap(user, poscap) == poscap)
+      sendto_one_buffer(user, buffer);
 
   dbuf_ref_free(buffer);
 }
 
-/* sendto_realops_flags()
+/*! \brief Send a message to members of a channel that are locally connected to this server.
+ * \param one      Client to skip; can be NULL
+ * \param channel    Destination channel
+ * \param rank     Minimum channel member rank clients must have
+ * \param poscap   Positive client capabilities flags (CAP)
+ * \param negcap   Negative client capabilities flags (CAP)
+ * \param format  Format string for command arguments
+ */
+void
+sendto_channel_local(const struct Client *one, struct Channel *channel, int rank, unsigned int poscap,
+                     unsigned int negcap, const char *format, ...)
+{
+  struct dbuf_block *buffer = dbuf_alloc();
+
+  va_list args;
+  va_start(args, format);
+  send_format(buffer, format, args);
+  va_end(args);
+
+  list_node_t *node;
+  LIST_FOREACH(node, channel->members_local.head)
+  {
+    struct ChannelMember *member = node->data;
+    struct Client *target = member->client;
+
+    if (IsDead(target))
+      continue;
+
+    if (one && (target == one->from))
+      continue;
+
+    if (rank && member_highest_rank(member) < rank)
+      continue;
+
+    if (poscap && HasCap(target, poscap) != poscap)
+      continue;
+
+    if (negcap && HasCap(target, negcap))
+      continue;
+
+    sendto_one_buffer(target, buffer);
+  }
+
+  dbuf_ref_free(buffer);
+}
+
+/* sendto_channel_butone()
  *
- * inputs	- flag types of messages to show to real opers
- *		- flag indicating opers/admins
- *		- var args input message
+ * inputs	- pointer to client(server) to NOT send message to
+ *		- pointer to client that is sending this message
+ *		- pointer to channel being sent to
+ *		- vargs message
  * output	- NONE
- * side effects	- Send to *local* ops only but NOT +s nonopers.
- */
-void
-sendto_realops_flags(unsigned int flags, int level, int type, const char *pattern, ...)
-{
-  va_list args;
-  dlink_node *node;
-  const char *ntype = "???";
-
-  switch (type)
-  {
-    case SEND_NOTICE:
-      ntype = "Notice";
-      break;
-    case SEND_GLOBAL:
-      ntype = "Global";
-      break;
-    case SEND_LOCOPS:
-      ntype = "LocOps";
-      break;
-    default:
-      assert(0);
-  }
-
-  struct dbuf_block *buffer = dbuf_alloc();
-  dbuf_put_fmt(buffer, ":%s NOTICE * :*** %s -- ", me.name, ntype); 
-
-  va_start(args, pattern);
-  send_format(buffer, pattern, args);
-  va_end(args);
-
-  DLINK_FOREACH(node, oper_list.head)
-  {
-    struct Client *client = node->data;
-    assert(HasUMode(client, UMODE_OPER));
-
-    if (IsDead(client))
-      continue;
-
-    /*
-     * If we're sending it to opers and they're an admin, skip.
-     * If we're sending it to admins, and they're not, skip.
-     */
-    if (((level == L_ADMIN) && !HasUMode(client, UMODE_ADMIN)) ||
-        ((level == L_OPER) && HasUMode(client, UMODE_ADMIN)))
-      continue;
-
-    if (!HasUMode(client, flags))
-      continue;
-
-    send_message(client, buffer);
-  }
-
-  dbuf_ref_free(buffer);
-}
-
-/* ts_warn()
+ * side effects	- message as given is sent to given channel members.
  *
- * inputs       - var args message
- * output       - NONE
- * side effects - Call sendto_realops_flags, with some flood checking
- *                (at most 5 warnings every 5 seconds)
+ * WARNING - +D clients are ignored
  */
 void
-sendto_realops_flags_ratelimited(uintmax_t *rate, const char *pattern, ...)
+sendto_channel_butone(struct Client *one, const struct Client *from, struct Channel *channel,
+                      int rank, const char *format, ...)
 {
-  va_list args;
-  char buffer[IRCD_BUFSIZE];
+  struct dbuf_block *buffer_l = dbuf_alloc();
+  struct dbuf_block *buffer_r = dbuf_alloc();
 
-  if ((event_base->time.sec_monotonic - *rate) < 20)
-    return;
-
-  *rate = event_base->time.sec_monotonic;
-
-  va_start(args, pattern);
-  vsnprintf(buffer, sizeof(buffer), pattern, args);
-  va_end(args);
-
-  sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE, "%s", buffer);
-  ilog(LOG_TYPE_IRCD, "%s", buffer);
-}
-
-/* sendto_wallops_flags()
- *
- * inputs       - flag types of messages to show to real opers
- *              - client sending request
- *              - var args input message
- * output       - NONE
- * side effects - Send a wallops to local opers
- */
-void
-sendto_wallops_flags(unsigned int flags, const struct Client *source_p,
-                     const char *pattern, ...)
-{
-  va_list args;
-  dlink_node *node;
-  struct dbuf_block *buffer = dbuf_alloc();
-
-  if (IsClient(source_p))
-    dbuf_put_fmt(buffer, ":%s!%s@%s WALLOPS :", source_p->name, source_p->username, source_p->host);
+  if (IsClient(from))
+    dbuf_put_fmt(buffer_l, ":%s!%s@%s ", from->name, from->username, from->host);
   else
-    dbuf_put_fmt(buffer, ":%s WALLOPS :", source_p->name);
+    dbuf_put_fmt(buffer_l, ":%s ", from->name);
 
-  va_start(args, pattern);
-  send_format(buffer, pattern, args);
-  va_end(args);
+  dbuf_put_fmt(buffer_r, ":%s ", from->id);
 
-  DLINK_FOREACH(node, oper_list.head)
+  va_list args_l, args_r;
+  va_start(args_l, format);
+  va_start(args_r, format);
+  send_format(buffer_l, format, args_l);
+  send_format(buffer_r, format, args_r);
+  va_end(args_l);
+  va_end(args_r);
+
+  ++send_marker;
+
+  list_node_t *node;
+  LIST_FOREACH(node, channel->members.head)
   {
-    struct Client *client = node->data;
-    assert(client->umodes & UMODE_OPER);
+    struct ChannelMember *member = node->data;
+    struct Client *target = member->client;
 
-    if (IsDead(client))
+    assert(IsClient(target));
+
+    if (IsDead(target->from))
       continue;
 
-    if (!HasUMode(client, flags))
+    if (one && (target->from == one->from))
       continue;
 
-    send_message(client, buffer);
+    if (rank && member_highest_rank(member) < rank)
+      continue;
+
+    if (user_mode_has_flag(target, UMODE_DEAF))
+      continue;
+
+    if (MyConnect(target))
+      sendto_one_buffer(target, buffer_l);
+    else if (target->from->connection->send_marker != send_marker)
+      sendto_one_buffer_remote(target->from, from, buffer_r);
+
+    target->from->connection->send_marker = send_marker;
   }
 
-  dbuf_ref_free(buffer);
+  dbuf_ref_free(buffer_l);
+  dbuf_ref_free(buffer_r);
 }
