@@ -1,7 +1,7 @@
 /*
  *  ircd-hybrid: an advanced, lightweight Internet Relay Chat Daemon (ircd)
  *
- *  Copyright (c) 1997-2022 ircd-hybrid development team
+ *  Copyright (c) 1997-2025 ircd-hybrid development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,15 +24,19 @@
  */
 
 #include "stdinc.h"
+#include "io_time.h"
+#include "defaults.h"
 #include "list.h"
-#include "user.h"
+#include "cap.h"
 #include "channel.h"
 #include "channel_mode.h"
 #include "client.h"
+#include "cloak.h"
 #include "hash.h"
 #include "id.h"
 #include "irc_string.h"
 #include "ircd.h"
+#include "ircd_hook.h"
 #include "listener.h"
 #include "motd.h"
 #include "numeric.h"
@@ -42,63 +46,14 @@
 #include "server.h"
 #include "send.h"
 #include "memory.h"
-#include "packet.h"
 #include "rng_mt.h"
 #include "misc.h"
 #include "parse.h"
 #include "monitor.h"
 #include "isupport.h"
 #include "tls.h"
-
-static char umode_buffer[UMODE_MAX_STR];
-
-const struct user_modes *umode_map[256];
-const struct user_modes  umode_tab[] =
-{
-  { 'B', UMODE_BOT          },
-  { 'D', UMODE_DEAF         },
-  { 'F', UMODE_FARCONNECT   },
-  { 'G', UMODE_SOFTCALLERID },
-  { 'H', UMODE_HIDDEN       },
-  { 'X', UMODE_EXPIRATION   },
-  { 'R', UMODE_REGONLY      },
-  { 'S', UMODE_SECURE       },
-  { 'W', UMODE_WEBIRC       },
-  { 'Z', UMODE_SECUREONLY   },
-  { 'a', UMODE_ADMIN        },
-  { 'c', UMODE_CCONN        },
-  { 'e', UMODE_EXTERNAL     },
-  { 'f', UMODE_FLOOD        },
-  { 'g', UMODE_CALLERID     },
-  { 'i', UMODE_INVISIBLE    },
-  { 'j', UMODE_REJ          },
-  { 'k', UMODE_SKILL        },
-  { 'l', UMODE_LOCOPS       },
-  { 'n', UMODE_NCHANGE      },
-  { 'o', UMODE_OPER         },
-  { 'p', UMODE_HIDECHANS    },
-  { 'q', UMODE_HIDEIDLE     },
-  { 'r', UMODE_REGISTERED   },
-  { 's', UMODE_SERVNOTICE   },
-  { 'w', UMODE_WALLOP       },
-  { 'y', UMODE_SPY          },
-  { 'z', UMODE_SECURE       },
-  { '\0', 0 }
-};
-
-void
-user_modes_init(void)
-{
-  char *umode_buffer_ptr = umode_buffer;
-
-  for (const struct user_modes *tab = umode_tab; tab->c; ++tab)
-  {
-    umode_map[tab->c] = tab;
-    *umode_buffer_ptr++ = tab->c;
-  }
-
-  *umode_buffer_ptr = '\0';
-}
+#include "user.h"
+#include "user_mode.h"
 
 /* show_lusers()
  *
@@ -109,34 +64,34 @@ user_modes_init(void)
 void
 show_lusers(struct Client *client)
 {
-  if (ConfigServerHide.hide_servers == 0 || HasUMode(client, UMODE_OPER))
-    sendto_one_numeric(client, &me, RPL_LUSERCLIENT, (dlink_list_length(&global_client_list) - Count.invisi),
-                       Count.invisi, dlink_list_length(&global_server_list));
+  if (ConfigServerHide.hide_servers == 0 || user_mode_has_flag(client, UMODE_OPER))
+    sendto_one_numeric(client, &me, RPL_LUSERCLIENT, (list_length(&global_client_list) - Count.invisi),
+                       Count.invisi, list_length(&global_server_list));
   else
-    sendto_one_numeric(client, &me, RPL_LUSERCLIENT, (dlink_list_length(&global_client_list) - Count.invisi),
+    sendto_one_numeric(client, &me, RPL_LUSERCLIENT, (list_length(&global_client_list) - Count.invisi),
                        Count.invisi, 1);
 
   if (Count.oper)
     sendto_one_numeric(client, &me, RPL_LUSEROP, Count.oper);
 
-  if (dlink_list_length(&unknown_list))
-    sendto_one_numeric(client, &me, RPL_LUSERUNKNOWN, dlink_list_length(&unknown_list));
+  if (list_length(&unknown_list))
+    sendto_one_numeric(client, &me, RPL_LUSERUNKNOWN, list_length(&unknown_list));
 
-  if (dlink_list_length(channel_get_list()))
-    sendto_one_numeric(client, &me, RPL_LUSERCHANNELS, dlink_list_length(channel_get_list()));
+  if (list_length(channel_get_list()))
+    sendto_one_numeric(client, &me, RPL_LUSERCHANNELS, list_length(channel_get_list()));
 
-  if (ConfigServerHide.hide_servers == 0 || HasUMode(client, UMODE_OPER))
+  if (ConfigServerHide.hide_servers == 0 || user_mode_has_flag(client, UMODE_OPER))
   {
-    sendto_one_numeric(client, &me, RPL_LUSERME, dlink_list_length(&local_client_list), dlink_list_length(&local_server_list));
-    sendto_one_numeric(client, &me, RPL_LOCALUSERS, dlink_list_length(&local_client_list), Count.max_loc);
-    sendto_one_numeric(client, &me, RPL_GLOBALUSERS, dlink_list_length(&global_client_list), Count.max_tot);
+    sendto_one_numeric(client, &me, RPL_LUSERME, list_length(&local_client_list), list_length(&local_server_list));
+    sendto_one_numeric(client, &me, RPL_LOCALUSERS, list_length(&local_client_list), Count.max_loc);
+    sendto_one_numeric(client, &me, RPL_GLOBALUSERS, list_length(&global_client_list), Count.max_tot);
     sendto_one_numeric(client, &me, RPL_STATSCONN, Count.max_loc_con, Count.max_loc, Count.totalrestartcount);
   }
   else
   {
-    sendto_one_numeric(client, &me, RPL_LUSERME, dlink_list_length(&global_client_list), 0);
-    sendto_one_numeric(client, &me, RPL_LOCALUSERS, dlink_list_length(&global_client_list), Count.max_tot);
-    sendto_one_numeric(client, &me, RPL_GLOBALUSERS, dlink_list_length(&global_client_list), Count.max_tot);
+    sendto_one_numeric(client, &me, RPL_LUSERME, list_length(&global_client_list), 0);
+    sendto_one_numeric(client, &me, RPL_LOCALUSERS, list_length(&global_client_list), Count.max_tot);
+    sendto_one_numeric(client, &me, RPL_GLOBALUSERS, list_length(&global_client_list), Count.max_tot);
   }
 }
 
@@ -197,31 +152,23 @@ report_and_set_user_flags(struct Client *client, const struct MaskItem *conf)
  *                from a remote connect.
  */
 static void
-introduce_client(struct Client *client)
+user_introduce(struct Client *client)
 {
-  char buf[UMODE_MAX_STR] = "";
-
-  send_umode(client, MyConnect(client), 0, buf);
   monitor_signon(client);
 
-  if (buf[0] == '\0')
-  {
-    buf[0] = '+';
-    buf[1] = '\0';
-  }
-
-  sendto_server(client, 0, 0, ":%s UID %s %u %ju %s %s %s %s %s %s %s :%s",
-                client->servptr->id, client->name, client->hopcount + 1,
-                client->tsinfo, buf, client->username, client->host,
-                client->realhost, client->sockhost, client->id,
-                client->account, client->info);
+  sendto_servers(client, 0, 0, ":%s UID %s %u %ju %s %s %s %s %s %s %s :%s",
+                 client->servptr->id, client->name, client->hopcount + 1,
+                 client->tsinfo, user_mode_to_str(client->umodes),
+                 client->username, client->host, client->realhost,
+                 client->sockhost, client->id, client->account, client->info);
 
   if (!EmptyString(client->tls_certfp))
-    sendto_server(client, 0, 0, ":%s CERTFP %s", client->id, client->tls_certfp);
+    sendto_servers(client, 0, 0, ":%s CERTFP %s", client->id, client->tls_certfp);
 
   if (!EmptyString(client->tls_cipher))
-    sendto_server(client, 0, 0, ":%s METADATA client %s cipher :%s",
-                  client->servptr->id, client->id, client->tls_cipher);
+    sendto_servers(client, 0, 0, ":%s METADATA client %s cipher :%s",
+                   client->servptr->id, client->id, client->tls_cipher);
+  AddFlag(client, FLAGS_INTRODUCED);
 }
 
 /* user_welcome()
@@ -241,20 +188,24 @@ user_welcome(struct Client *client)
 
   if (HasFlag(client, FLAGS_TLS))
   {
-    AddUMode(client, UMODE_SECURE);
+    user_mode_set_flag(client, UMODE_SECURE);
 
-    client->tls_cipher = xstrdup(tls_get_cipher(&client->connection->fd->tls));
+    client->tls_cipher = io_strdup(tls_get_cipher(&client->connection->fd->tls));
     sendto_one_notice(client, &me, ":*** Connected securely via %s",
                       client->tls_cipher);
+
+    if (!EmptyString(client->tls_certfp))
+      sendto_one_notice(client, &me, ":*** Your client certificate fingerprint is %s",
+                        client->tls_certfp);
   }
 
-  sendto_one_numeric(client, &me, RPL_WELCOME, ConfigServerInfo.network_name,
-                     client->name, client->username, client->realhost);
+  sendto_one_numeric(client, &me, RPL_WELCOME,
+                     ConfigServerInfo.network_name, client->name, client->username, client->realhost);
   sendto_one_numeric(client, &me, RPL_YOURHOST,
                      listener_get_name(client->connection->listener), IRCD_VERSION);
   sendto_one_numeric(client, &me, RPL_CREATED, built_date);
-  sendto_one_numeric(client, &me, RPL_MYINFO, me.name, IRCD_VERSION, umode_buffer,
-                     cmode_rpl04[0], cmode_rpl04[1]);
+  sendto_one_numeric(client, &me, RPL_MYINFO,
+                     me.name, IRCD_VERSION, user_mode_get_string(), cmode_rpl04[0], cmode_rpl04[1]);
 
   isupport_show(client);
   show_lusers(client);
@@ -267,14 +218,13 @@ user_welcome(struct Client *client)
  * \param client Pointer to given client to introduce
  */
 void
-register_local_user(struct Client *client)
+user_register_local(struct Client *client)
 {
-  const struct MaskItem *conf = NULL;
-
   assert(client == client->from);
   assert(client->connection->registration == 0);
   assert(MyConnect(client));
   assert(IsUnknown(client));
+  assert(list_find(&unknown_list, client));
 
   if (ConfigGeneral.ping_cookie)
   {
@@ -296,54 +246,24 @@ register_local_user(struct Client *client)
   if (conf_check_client(client) == false)
     return;
 
-  conf = client->connection->confs.head->data;
-
-  if (!HasFlag(client, FLAGS_GOTID))
+  const struct MaskItem *const conf = client->connection->confs.head->data;
+  if (IsNeedIdentd(conf) && !HasFlag(client, FLAGS_GOTID))
   {
-    if (IsNeedIdentd(conf))
-    {
-      ++ServerStats.is_ref;
-      sendto_one_notice(client, &me, ":*** Notice -- You need to install "
-                        "identd to use this server");
-      exit_client(client, "Install identd");
-      return;
-    }
+    sendto_one_notice(client, &me, ":*** Notice -- You need to install identd to use this server");
+    client_exit(client, "Install identd");
+    ++ServerStats.is_ref;
+    return;
   }
 
   if (valid_username(client->username, true) == false)
   {
-    char buf[sizeof("Invalid username []") + sizeof(client->username)];
-
-    sendto_realops_flags(UMODE_REJ, L_ALL, SEND_NOTICE,
-                         "Invalid username: %s (%s@%s)",
-                         client->name, client->username, client->host);
+    sendto_clients(UMODE_REJ, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Invalid username: %s (%s@%s)",
+                   client->name, client->username, client->host);
+    client_exit_fmt(client, "Invalid username [%s]", client->username);
     ++ServerStats.is_ref;
-    snprintf(buf, sizeof(buf), "Invalid username [%s]", client->username);
-    exit_client(client, buf);
     return;
   }
 
-  /* Password check */
-  if (!EmptyString(conf->passwd))
-  {
-    if (match_conf_password(client->connection->password, conf) == false)
-    {
-      ++ServerStats.is_ref;
-
-      sendto_one_numeric(client, &me, ERR_PASSWDMISMATCH);
-      exit_client(client, "Bad Password");
-      return;
-    }
-  }
-
-  xfree(client->connection->password);
-  client->connection->password = NULL;
-
-  /*
-   * Report if user has &^>= etc. and set flags as needed in client
-   */
-  report_and_set_user_flags(client, conf);
-
   /*
    * Limit clients -
    * We want to be able to have servers and F-line clients
@@ -352,33 +272,46 @@ register_local_user(struct Client *client)
    * probably be just a percentage of the MAXCLIENTS...
    *   -Taner
    */
-  if ((dlink_list_length(&local_client_list) >= GlobalSetOptions.maxclients + MAX_BUFFER) ||
-      (dlink_list_length(&local_client_list) >= GlobalSetOptions.maxclients && !HasFlag(client, FLAGS_NOLIMIT)))
+  unsigned int max_clients = GlobalSetOptions.maxclients;
+  if (IsConfExemptLimits(conf))
+    max_clients += MAX_BUFFER;
+  if (list_length(&local_client_list) >= max_clients)
   {
-    sendto_realops_flags(UMODE_REJ, L_ALL, SEND_NOTICE,
-                         "Too many clients, rejecting %s[%s].",
-                         client->name, client->host);
+    sendto_clients(UMODE_REJ, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "Too many clients, rejecting %s[%s].",
+                   client->name, client->host);
+    client_exit(client, "Sorry, server is full - try later");
     ++ServerStats.is_ref;
-    exit_client(client, "Sorry, server is full - try later");
     return;
   }
 
-  if (!HasFlag(client, FLAGS_EXEMPTXLINE))
+  if (!IsConfExemptXline(conf))
   {
     const struct GecosItem *gecos = gecos_find(client->info, match);
     if (gecos)
     {
-      sendto_realops_flags(UMODE_REJ, L_ALL, SEND_NOTICE,
-                           "X-line Rejecting [%s] [%s], user %s [%s]",
-                           client->info, gecos->reason,
-                           client_get_name(client, HIDE_IP),
-                           client->sockhost);
+      sendto_clients(UMODE_REJ, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE, "X-line Rejecting [%s] [%s], user %s [%s]",
+                     client->info, gecos->reason, client_get_name(client, HIDE_IP), client->sockhost);
+      client_exit(client, "Bad user info");
       ++ServerStats.is_ref;
-      exit_client(client, "Bad user info");
       return;
     }
   }
 
+  /* Password check */
+  if (!EmptyString(conf->passwd))
+  {
+    if (match_conf_password(client->connection->password, conf) == false)
+    {
+      sendto_one_numeric(client, &me, ERR_PASSWDMISMATCH);
+      client_exit(client, "Bad Password");
+      ++ServerStats.is_ref;
+      return;
+    }
+  }
+
+  io_free(client->connection->password);
+  client->connection->password = NULL;
+
   const char *id;
   while (hash_find_id((id = uid_get())))
     ;
@@ -386,54 +319,49 @@ register_local_user(struct Client *client)
   strlcpy(client->id, id, sizeof(client->id));
   hash_add_id(client);
 
-  sendto_realops_flags(UMODE_CCONN, L_ALL, SEND_NOTICE,
-                       "Client connecting: %s (%s@%s) [%s] {%s} [%s] <%s>",
-                       client->name, client->username, client->realhost,
-                       client->sockhost,
-                       class_get_name(&client->connection->confs),
-                       client->info, client->id);
-
-  if (ConfigGeneral.invisible_on_connect)
-  {
-    AddUMode(client, UMODE_INVISIBLE);
-    ++Count.invisi;
-  }
+  hook_dispatch(ircd_hook_user_register_local, &(ircd_hook_user_register_ctx){ .client = client });
 
   SetClient(client);
 
   client->servptr = &me;
-  client->connection->last_privmsg = event_base->time.sec_monotonic;
+  client->connection->last_privmsg = io_time_get(IO_TIME_MONOTONIC_SEC);
 
-  dlinkAdd(client, &client->lnode, &client->servptr->serv->client_list);
-  dlinkAdd(client, &client->node, &global_client_list);
+  list_add(client, &client->lnode, &client->servptr->serv->client_list);
+  list_add(client, &client->node, &global_client_list);
+  list_move_node(&client->connection->node, &unknown_list, &local_client_list);
 
-  assert(dlinkFind(&unknown_list, client));
-
-  dlink_move_node(&client->connection->lclient_node,
-                  &unknown_list, &local_client_list);
-
-  if (dlink_list_length(&local_client_list) > Count.max_loc)
+  if (list_length(&local_client_list) > Count.max_loc)
   {
-    Count.max_loc = dlink_list_length(&local_client_list);
+    Count.max_loc = list_length(&local_client_list);
 
     if (!(Count.max_loc % 10))
-      sendto_realops_flags(UMODE_SERVNOTICE, L_ALL, SEND_NOTICE,
-                           "New maximum local client connections: %u",
-                           Count.max_loc);
+      sendto_clients(UMODE_SERVNOTICE, SEND_RECIPIENT_OPER_ALL, SEND_TYPE_NOTICE,
+                     "New peak in local client connections: %u",
+                     Count.max_loc);
   }
 
-  if ((dlink_list_length(&local_client_list) +
-       dlink_list_length(&local_server_list)) > Count.max_loc_con)
-    Count.max_loc_con = dlink_list_length(&local_client_list) +
-                        dlink_list_length(&local_server_list);
+  if ((list_length(&local_client_list) + list_length(&local_server_list)) > Count.max_loc_con)
+    Count.max_loc_con = list_length(&local_client_list) + list_length(&local_server_list);
 
-  if (dlink_list_length(&global_client_list) > Count.max_tot)
-    Count.max_tot = dlink_list_length(&global_client_list);
+  if (list_length(&global_client_list) > Count.max_tot)
+    Count.max_tot = list_length(&global_client_list);
   ++Count.totalrestartcount;
 
+  if (ConfigGeneral.invisible_on_connect)
+    user_mode_set_flag_exec(client, UMODE_INVISIBLE, USER_MODE_SOURCE_REGULAR);
+
+  /*
+   * Report if user has &^>= etc. and set flags as needed in client
+   */
+  report_and_set_user_flags(client, conf);
+
   user_welcome(client);
 
-  introduce_client(client);
+  user_mode_set_flag_exec(client, UMODE_CLOAK, USER_MODE_SOURCE_REGULAR);
+
+  user_mode_send(client, 0, USER_MODE_SEND_CLIENT);
+
+  user_introduce(client);
 }
 
 /* register_remote_user()
@@ -447,7 +375,7 @@ register_local_user(struct Client *client)
  *		  is introduced by a server.
  */
 void
-register_remote_user(struct Client *client)
+user_register_remote(struct Client *client)
 {
   assert(client->servptr->from == client->from);
 
@@ -459,20 +387,15 @@ register_remote_user(struct Client *client)
     AddFlag(client, FLAGS_SERVICE);
 
   SetClient(client);
-  dlinkAdd(client, &client->lnode, &client->servptr->serv->client_list);
-  dlinkAdd(client, &client->node, &global_client_list);
+  list_add(client, &client->lnode, &client->servptr->serv->client_list);
+  list_add(client, &client->node, &global_client_list);
 
-  if (dlink_list_length(&global_client_list) > Count.max_tot)
-    Count.max_tot = dlink_list_length(&global_client_list);
+  if (list_length(&global_client_list) > Count.max_tot)
+    Count.max_tot = list_length(&global_client_list);
 
-  if (HasFlag(client->servptr, FLAGS_EOB))
-    sendto_realops_flags(UMODE_FARCONNECT, L_ALL, SEND_NOTICE,
-                         "Client connecting at %s: %s (%s@%s) [%s] [%s] <%s>",
-                         client->servptr->name,
-                         client->name, client->username, client->realhost,
-                         client->sockhost, client->info, client->id);
+  hook_dispatch(ircd_hook_user_register_remote, &(ircd_hook_user_register_ctx){ .client = client });
 
-  introduce_client(client);
+  user_introduce(client);
 }
 
 /* valid_hostname()
@@ -584,91 +507,8 @@ valid_nickname(const char *nickname, bool local)
   return p - nickname <= NICKLEN;
 }
 
-/*! \brief Builds a mode change string to buffer pointed by \a buf
- * \param client  Pointer to client
- * \param send    Whether to send a MODE message to client
- * \param old     Old user mode to compare against when building new mode buffer
- * \param buf     Pointer to buffer to build string in
- */
 void
-send_umode(struct Client *client, bool send, unsigned int old, char *buf)
-{
-  char *m = buf;
-  int what = MODE_QUERY;
-
-  /*
-   * Build a string in umode_buf to represent the change in the user's
-   * mode between the new (client->umodes) and 'old'.
-   */
-  for (const struct user_modes *tab = umode_tab; tab->c; ++tab)
-  {
-    if ((tab->flag & old) && !HasUMode(client, tab->flag))
-    {
-      if (what == MODE_DEL)
-        *m++ = tab->c;
-      else
-      {
-        what = MODE_DEL;
-        *m++ = '-';
-        *m++ = tab->c;
-      }
-    }
-    else if (!(tab->flag & old) && HasUMode(client, tab->flag))
-    {
-      if (what == MODE_ADD)
-        *m++ = tab->c;
-      else
-      {
-        what = MODE_ADD;
-        *m++ = '+';
-        *m++ = tab->c;
-      }
-    }
-  }
-
-  *m = '\0';
-
-  if (send == true && *buf)
-    sendto_one(client, ":%s!%s@%s MODE %s :%s",
-               client->name, client->username,
-               client->host, client->name, buf);
-}
-
-/* send_umode_out()
- *
- * inputs	-
- * output	- NONE
- * side effects - Only send ubuf out to servers that know about this client
- */
-void
-send_umode_out(struct Client *client, unsigned int old)
-{
-  char buf[UMODE_MAX_STR] = "";
-
-  send_umode(client, MyConnect(client), old, buf);
-
-  if (buf[0])
-    sendto_server(client, 0, 0, ":%s MODE %s :%s",
-                  client->id, client->id, buf);
-}
-
-const char *
-user_get_mode_str(unsigned int modes)
-{
-  static char buf[UMODE_MAX_STR];
-  char *m = buf;
-
-  *m++ = '+';
-  for (const struct user_modes *tab = umode_tab; tab->c; ++tab)
-    if (modes &tab->flag)
-      *m++ = tab->c;
-  *m = '\0';
-
-  return buf;
-}
-
-void
-user_set_hostmask(struct Client *client, const char *hostname)
+user_set_hostmask(struct Client *client, const char *hostname, bool svshost)
 {
   if (strcmp(client->host, hostname) == 0)
     return;
@@ -678,11 +518,14 @@ user_set_hostmask(struct Client *client, const char *hostname)
                                  client->name, client->username, client->host);
 
   sendto_common_channels_local(client, true, CAP_CHGHOST, 0, ":%s!%s@%s CHGHOST %s %s",
-                               client->name, client->username,
-                               client->host, client->username, hostname);
+                               client->name, client->username, client->host, client->username, hostname);
 
   strlcpy(client->host, hostname, sizeof(client->host));
 
+  if (svshost)
+    sendto_servers(client, 0, 0, ":%s SVSHOST %s %ju %s",
+                   client->servptr->id, client->id, client->tsinfo, client->host);
+
   if (MyConnect(client))
   {
     sendto_one_numeric(client, &me, RPL_VISIBLEHOST, client->host);
@@ -692,18 +535,18 @@ user_set_hostmask(struct Client *client, const char *hostname)
   if (ConfigGeneral.cycle_on_host_change == 0)
     return;
 
-  dlink_node *node;
-  DLINK_FOREACH(node, client->channel.head)
+  list_node_t *node;
+  LIST_FOREACH(node, client->channel.head)
   {
-    char modebuf[CMEMBER_STATUS_FLAGS_LEN + 1];
     char nickbuf[CMEMBER_STATUS_FLAGS_LEN * NICKLEN + CMEMBER_STATUS_FLAGS_LEN] = "";
+    char modebuf[CMEMBER_STATUS_FLAGS_LEN + 1];
     char *p = modebuf;
     int len = 0;
     const struct ChannelMember *member = node->data;
 
     for (const struct chan_mode *tab = cflag_tab; tab->letter; ++tab)
     {
-      if (member_has_flags(member, tab->flag) == true)
+      if (member_has_flags(member, tab->flag))
       {
         *p++ = tab->letter;
         len += snprintf(nickbuf + len, sizeof(nickbuf) - len, len ? " %s" : "%s", client->name);
@@ -724,8 +567,6 @@ user_set_hostmask(struct Client *client, const char *hostname)
   }
 
   if (client->away[0])
-    sendto_common_channels_local(client, false, CAP_AWAY_NOTIFY, CAP_CHGHOST,
-                                 ":%s!%s@%s AWAY :%s",
-                                 client->name, client->username,
-                                 client->host, client->away);
+    sendto_common_channels_local(client, false, CAP_AWAY_NOTIFY, CAP_CHGHOST, ":%s!%s@%s AWAY :%s",
+                                 client->name, client->username, client->host, client->away);
 }
